/************************************************************************/
/*BEGINPROLOGUE************************************************************************
 * @copyright(disclaimer)                                                             *
 * Licensed Materials - Property of IBM                                               *
 * IBM Enterprise Information Portal for Multiplatforms V8.1 (program number 5724-B43)*
 * (c ) Copyright IBM Corp. 1998,  2002. All Rights Reserved.                         *
 *                                                                                    *
 * US Government Users Restricted Rights                                              *
 * Use, duplication or disclosure restricted by GSA ADP Schedule                      *
 * Contract with IBM Corp.                                                            *
 *                                                                                    *
 * DISCLAIMER OF WARRANTIES :                                                         *
 *                                                                                    *
 * Permission is granted to copy and modify this Sample code, and to                  *
 * distribute modified versions provided that both the copyright                      *
 * notice,- and this permission notice and warranty disclaimer appear in all copies   * 
 * and modified versions.                                                             *
 *                                                                                    *
 * THIS SAMPLE CODE IS LICENSED TO YOU AS-IS.                                         *
 * IBM AND ITS SUPPLIERS AND LICENSORS DISCLAIM ALL WARRANTIES, EITHER EXPRESS OR     *
 * IMPLIED, IN SUCH SAMPLE CODE, INCLUDING THE WARRANTY OF NON-INFRINGEMENT AND THE   *
 * IMPLIED WARRANTIES OF MERCHANTABILITY OR FITNESS FOR A PARTICULAR PURPOSE. IN NO   *
 * EVENT WILL IBM OR ITS LICENSORS OR SUPPLIERS BE LIABLE FOR ANY DAMAGES ARISING OUT *
 * OF THE USE OF OR INABILITY TO USE THE SAMPLE CODE, DISTRIBUTION OF THE SAMPLE CODE,*
 * OR COMBINATION OF THE SAMPLE CODE WITH ANY OTHER CODE. IN NO EVENT SHALL IBM OR ITS*
 * LICENSORS AND SUPPLIERS BE LIABLE FOR ANY LOST REVENUE, LOST PROFITS OR DATA, OR   *
 * FOR DIRECT, INDIRECT, SPECIAL, CONSEQUENTIAL,INCIDENTAL OR PUNITIVE DAMAGES,       *
 * HOWEVER CAUSED AND REGARDLESS OF THE THEORY OF LIABILITY,-, EVEN IF IBM OR ITS     *
 * LICENSORS OR SUPPLIERS HAVE BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGES.       *
 *                                                                                    *
 * @endCopyright                                                                      *
 **************************************************************************************/
/************************************************************************/
/*
 * Filename       : TxdoAddBsmsDL.cpp
 * Description    : Test DKBlobDL for add object with specified storage collection
 *                  (add from buffer)
 * setup:  1. bring up SMS server
 *         2. bring up System Admin Client
 *         3. create migration policy(managementClass) MEIMGT1 with
 *            retension period = 30 days & storage calss = FIXED
 *         4. create workstation collection CBR.MEICOLLECT1 with
 *            migration policy = MEIMGT1 & storage group = GROUP01
 *         5. pick a known itemId from sample loader's output log
 *
 * hard code: need to modify library server/userid/password for dsDL.connect                 
 * invoke: TxdoAddBsmsDL <fileName to be added> <partId> <itemId>
 *
 */

#include <iostream.h>
#include <stdio.h>  
#include <stdlib.h>
#include <DKString.hpp>

#include <DKConstantDL.h>
#include <DKDatastoreDL.hpp>
#include <DKException.hpp>
#include <DKHandle.hpp>
#include <DKBlobDL.hpp>
#include <DKSearchEngineInfoDL.hpp>
#include <DKAnnotationDL.hpp>
#include <DKMediaStreamInfoDL.hpp>
#include <DKStorageManageInfoDL.hpp>
#include <DKEnvironment.hpp>

void main(int argc, char *argv[])
{
    cout << "begin..." << endl;
    // Indicate this is a Console Application on Windows NT/95 for error
    // message code page conversion to OEM code page. If this was a Windows
    // GUI (Graphic User Interface) Application this would not be necessary
    // because DK_SS_WINDOWS is the default.
    DKEnvironment::setSubSystem(DK_SS_CONSOLE);
    
    DKDatastoreDL dsDL;
    DKString itemId, fileName;
    int partId;
    fileName = "e:\\dlv200j\\test\\notepart.txt";
    partId = 0;
    itemId = "P5ZJ4YKMS0KLT711";
    
    cout <<"argc is "<<argc<<endl;
    if (argc == 1)
    {
      cout<<"invoke: TxdoAddBsmsDL <fileName> <partId> <itemId>"<<endl;
      cout<<" no parameter, following default will be provided:"<<endl;
      cout<<"The supplied default fileName = "<<fileName<<endl;
      cout<<"The supplied default partId = "<<partId<<endl;
      cout<<"The supplied default itemId = "<<itemId<<endl;
    }
    else if (argc == 2)
    {
     fileName = DKString(argv[1]);
     cout<<"you enter: TxdoAddBsmsDL "<<argv[1]<<endl;
     cout<<"The supplied default partId = "<<partId<<endl;
     cout<<"The supplied default itemId = "<<itemId<<endl;
    }
    else if (argc == 3)
    {
     fileName = DKString(argv[1]);
     partId = atoi(argv[2]);
     cout<<"you enter: TxdoAddBsmsDL "<<argv[1]<<" "<<argv[2]<<endl;
     cout<<"The supplied default itemId = "<<itemId<<endl;
    }
    else if (argc ==  4)
    {
     fileName = DKString(argv[1]);
     partId = atoi(argv[2]);
     itemId = DKString(argv[3]);
     cout<<"you enter: TxdoAddBsmsDL "<<argv[1]<<" "<<argv[2]<< " "<<argv[3]<<endl;
    }

    cout << "connecting to datastore.." << endl;
    try {
     dsDL.connect("LIBSRVRP","FRNADMIN","PASSWORD");
     cout << "datastore connected" << endl;
        
     DKBlobDL* axdo = new DKBlobDL(&dsDL);       
     DKString rtype;
     DKPidXDODL*  apid = new DKPidXDODL;
     cout <<"create apid " << apid <<endl;
     apid ->setPartId(partId);
     apid ->setPrimaryId(itemId);
     axdo ->setPidObject(apid);
     axdo ->setContentClass(DK_DL_CC_ASCII);
     cout <<"itemId= " << axdo->getItemId()<<endl;
     cout <<"partId="<<axdo->getPartId()<<endl;
     cout <<"repType="<< axdo->getRepType()<<endl;  
     cout <<"contentclass="<< axdo->getContentClass()<<endl;  

     //---set SMS -----
     DKStorageManageInfoDL aSMS;
     aSMS.setRetention(688);
     aSMS.setCollectionName("CBR.MEICOLLECT1");
     aSMS.setManagementClass("MEIMGT1");
     aSMS.setStorageClass("FIXED");
     axdo->setExtension("DKStorageManageInfoDL", (dkExtension*)&aSMS);

     axdo ->setContentFromClientFile(fileName);
     cout <<"length of lobdata = "<<(axdo->getContent()).length()<<endl;
     cout <<"about to do add()...."<<endl;
     axdo ->add();       
     cout<<"add successfully.... "<<endl;
     cout<<"after add() partId="<<axdo->getPartId()<<endl;

     cout<<"about to retrieve/open() the added object..."<<endl;
     axdo ->retrieve(); // retrieve the added object
     cout<<"retrieve success.."<<endl;
     cout<<"contentclass="<<axdo->getContentClass()<<endl;
     cout<<"mimeType="<<axdo->getMimeType()<<endl;
     int concls = axdo->getContentClass();
     if (concls == DK_DL_CC_ASCII)
         axdo->setInstanceOpenHandler("notepad", TRUE);
     else if (concls == DK_DL_CC_GIF)
         axdo->setInstanceOpenHandler("lviewpro", TRUE);  //use lviewpro in NT
     else if (concls == DK_DL_CC_AVI)
         axdo->setInstanceOpenHandler("mplay32", TRUE);
     axdo ->open();
     cout <<"aff type= "<<axdo->getAffiliatedType()<<endl;
     int atype = axdo->getAffiliatedType();
     if (atype == DK_DL_ANNOTATION)
     {
      cout <<"aff pageNumber= "<<axdo->getAffiliatedData().getPageNumber()<<endl;
      cout <<"aff Y= "<<axdo->getAffiliatedData().getY()<<endl;
      cout <<"aff X= "<<axdo->getAffiliatedData().getX()<<endl;
     }

     //***** after retrieve get SMS 
       cout <<"about to retrieve the DKStorageManageInfoDL ...."<<endl; 
       DKStorageManageInfoDL* aSMS2 =  (DKStorageManageInfoDL*)axdo->getExtension("DKStorageManageInfoDL");
       cout<<"  collectionName= "<<aSMS2->getCollectionName()<<endl;
       cout<<"  managementClass="<<aSMS2->getManagementClass()<<endl;
       cout<<"  storageClass="<<aSMS2->getStorageClass()<<endl;
       cout<<"  retention="<<aSMS2->getRetention()<<endl;
       cout<<"  storeSite="<<aSMS2->getStoreSite()<<endl;
    
     delete axdo;
     delete apid;
     dsDL.disconnect();
     cout<<"datastore disconnected"<<endl;
    }
    catch(DKException &exc)
    {
      cout << "Error id" << exc.errorId() << endl;
      cout << "Exception id " << exc.exceptionId() << endl;
      for(unsigned long i = 0; i< exc.textCount();i++)
      {
       cout << "Error text:" << exc.text(i) << endl;
      }
      for (unsigned long g=0; g< exc.locationCount();g++)
      {
       const DKExceptionLocation* p = exc.locationAtIndex(g);
       cout << "Filename: " << p->fileName() << endl;
       cout << "Function: " << p->functionName() << endl;
       cout << "LineNumber: " << p->lineNumber() << endl;
      }
      cout << "Exception Class Name: " << exc.name() << endl;
    }   
    cout << "done ..." << endl;      
}


