/************************************************************************/
/*BEGINPROLOGUE************************************************************************
 * @copyright(disclaimer)                                                             *
 * Licensed Materials - Property of IBM                                               *
 * IBM Enterprise Information Portal for Multiplatforms V8.1 (program number 5724-B43)*
 * (c ) Copyright IBM Corp. 1998,  2002. All Rights Reserved.                         *
 *                                                                                    *
 * US Government Users Restricted Rights                                              *
 * Use, duplication or disclosure restricted by GSA ADP Schedule                      *
 * Contract with IBM Corp.                                                            *
 *                                                                                    *
 * DISCLAIMER OF WARRANTIES :                                                         *
 *                                                                                    *
 * Permission is granted to copy and modify this Sample code, and to                  *
 * distribute modified versions provided that both the copyright                      *
 * notice,- and this permission notice and warranty disclaimer appear in all copies   * 
 * and modified versions.                                                             *
 *                                                                                    *
 * THIS SAMPLE CODE IS LICENSED TO YOU AS-IS.                                         *
 * IBM AND ITS SUPPLIERS AND LICENSORS DISCLAIM ALL WARRANTIES, EITHER EXPRESS OR     *
 * IMPLIED, IN SUCH SAMPLE CODE, INCLUDING THE WARRANTY OF NON-INFRINGEMENT AND THE   *
 * IMPLIED WARRANTIES OF MERCHANTABILITY OR FITNESS FOR A PARTICULAR PURPOSE. IN NO   *
 * EVENT WILL IBM OR ITS LICENSORS OR SUPPLIERS BE LIABLE FOR ANY DAMAGES ARISING OUT *
 * OF THE USE OF OR INABILITY TO USE THE SAMPLE CODE, DISTRIBUTION OF THE SAMPLE CODE,*
 * OR COMBINATION OF THE SAMPLE CODE WITH ANY OTHER CODE. IN NO EVENT SHALL IBM OR ITS*
 * LICENSORS AND SUPPLIERS BE LIABLE FOR ANY LOST REVENUE, LOST PROFITS OR DATA, OR   *
 * FOR DIRECT, INDIRECT, SPECIAL, CONSEQUENTIAL,INCIDENTAL OR PUNITIVE DAMAGES,       *
 * HOWEVER CAUSED AND REGARDLESS OF THE THEORY OF LIABILITY,-, EVEN IF IBM OR ITS     *
 * LICENSORS OR SUPPLIERS HAVE BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGES.       *
 *                                                                                    *
 * @endCopyright                                                                      *
 **************************************************************************************/
/************************************************************************/
/*
 * Filename       : TxdosAddFsmsDL.cpp
 * Description    : Test DLBlobDL for add object with specified storage collection
 *                  (add from file to be search indexed)
 * setup:  1. bring up SMS server
 *         2. bring up System Admin Client
 *         3. create migration policy(managementClass) MEIMGT1 with
 *            retension period = 30 days & storage calss = FIXED
 *         4. create workstation collection CBR.MEICOLLECT1 with
 *            migration policy = MEIMGT1 & storage group = GROUP01
 *         5. pick a known itemId from sample loader's output log
 *
 * hard code: need to modify library server/userid/password for dsDL.connect 
 *            need to modify search engine informations for yours                             
 * invoke: TxdosAddFsmsDL <fileName to be added> <partId> <itemId>
 *
 */

#include <iostream.h>
#include <stdio.h>  
#include <stdlib.h>
#include <DKString.hpp>

#include <DKConstantDL.h>
#include <DKDatastoreDL.hpp>
#include <DKException.hpp>
#include <DKHandle.hpp>
#include <DKBlobDL.hpp>
#include <DKSearchEngineInfoDL.hpp>
#include <DKAnnotationDL.hpp>
#include <DKMediaStreamInfoDL.hpp>
#include <DKStorageManageInfoDL.hpp>
#include <DKEnvironment.hpp>

void main(int argc, char *argv[])
{
   cout << "begin..." << endl;
   // Indicate this is a Console Application on Windows NT/95 for error
   // message code page conversion to OEM code page. If this was a Windows
   // GUI (Graphic User Interface) Application this would not be necessary
   // because DK_SS_WINDOWS is the default.
   DKEnvironment::setSubSystem(DK_SS_CONSOLE); 
    
    DKDatastoreDL dsDL;
    DKString itemId, fileName;
    int partId;
    fileName = "e:\\dlv200j\\test\\notepart.txt";
    partId = 30;
    itemId = "P5ZJ4YKMS0KLT711";
    
    cout <<"argc is "<<argc<<endl;
    if (argc == 1)
    {
      cout<<"invoke: TxdosAddFsmsDL <fileName> <partId> <itemId>"<<endl;
      cout<<" no parameter, following default will be provided:"<<endl;
      cout<<"The supplied default fileName = "<<fileName<<endl;
      cout<<"The supplied default partId = "<<partId<<endl;
      cout<<"The supplied default itemId = "<<itemId<<endl;
    }
    else if (argc == 2)
    {
     fileName = DKString(argv[1]);
     cout<<"you enter: TxdosAddFsmsDL "<<argv[1]<<endl;
     cout<<"The supplied default partId = "<<partId<<endl;
     cout<<"The supplied default itemId = "<<itemId<<endl;
    }
    else if (argc == 3)
    {
     fileName = DKString(argv[1]);
     partId = atoi(argv[2]);
     cout<<"you enter: TxdosAddFsmsDL "<<argv[1]<<" "<<argv[2]<<endl;
     cout<<"The supplied default itemId = "<<itemId<<endl;
    }
    else if (argc ==  4)
    {
     fileName = DKString(argv[1]);
     partId = atoi(argv[2]);
     itemId = DKString(argv[3]);
     cout<<"you enter: TxdosAddFsmsDL "<<argv[1]<<" "<<argv[2]<< " "<<argv[3]<<endl;
    }

    cout << "Connecting to datastore.." << endl;
    try {
     dsDL.connect("LIBSRVRP","FRNADMIN","PASSWORD");
     cout << "datastore connected" << endl;

     DKBlobDL* axdo = new DKBlobDL(&dsDL);
     DKString rtype;
     DKPidXDODL*  apid = new DKPidXDODL;

     cout <<"create apid " << apid <<endl;
     apid ->setPartId(partId);
     apid ->setItemId(itemId);
     axdo ->setPidObject(apid);
     axdo ->setContentClass(DK_DL_CC_ASCII);
     cout <<"itemId= " <<((DKPidXDODL*)(axdo->getPidObject()))->getItemId()<<endl;
     cout<<"partId="<<((DKPidXDODL*)(axdo->getPidObject()))->getPartId()<<endl;
     cout <<"repType="<<axdo->getRepType()<<endl;  
     cout <<"contentclass="<<axdo->getContentClass()<<endl;

     cout<<"about to set searchEngine.."<<endl; 
     //---set searchEngine -----
     DKSearchEngineInfoDL aSrchEx;
     aSrchEx.setSearchEngine("SM");
     aSrchEx.setSearchIndex("TM-TMINDEX");
     aSrchEx.setSearchInfo("ENU");
     axdo->setExtension("DKSearchEngineInfoDL", (dkExtension*)&aSrchEx);
     cout<<"after setExtension(DKSearchEngineInfoDL)"<<endl;

     cout<<"about to set DKAnnotation.."<<endl; 
     //------- set DKAnnotation-------
     DKAnnotationDL ann;
     ann.setPart(9);
     ann.setPageNumber(9);
     ann.setX(49);
     ann.setY(19);    
     axdo->setExtension("DKAnnotationDL", (dkExtension*)&ann);
     axdo ->setAffiliatedType(DK_ANNOTATION);
     cout<<"after setExtension(DKAnnotationDL)"<<endl;
     
     cout<<"about to set DKStorageManageInfoDL.."<<endl; 
     //---set SMS -----
     DKStorageManageInfoDL aSMS;
     aSMS.setRetention(555);
     aSMS.setCollectionName("CBR.MEICOLLECT1");
     aSMS.setManagementClass("MEIMGT1");
     aSMS.setStorageClass("FIXED");
     axdo->setExtension("DKStorageManageInfoDL", (dkExtension*)&aSMS);
     cout<<"after setExtension(DKStorageManageInfoDL)"<<endl;

     cout <<"about to do add(fileName)"<<endl;
     axdo ->add(fileName); 
     cout<<"add successfully.... "<<endl;

     cout<<"after add() partId="<<axdo->getPartId()<<endl;

     cout<<"about to retrieve/open() the added object..."<<endl;
     axdo ->retrieve(); // retrieve the 1st added object
     int concls = axdo->getContentClass();
     if (concls == DK_DL_CC_ASCII)
	     axdo->setInstanceOpenHandler("notepad", TRUE);
     else if (concls == DK_DL_CC_GIF)
         axdo->setInstanceOpenHandler("lviewpro", TRUE);  //use lviewpro in NT
     else if (concls == DK_DL_CC_AVI)
  	     axdo->setInstanceOpenHandler("mplay32", TRUE);
     axdo ->open();
     cout <<"aff type= "<<axdo->getAffiliatedType()<<endl;
     int atype = axdo->getAffiliatedType();
     if (atype == DK_DL_ANNOTATION)
     {
        DKAnnotationDL* ann = (DKAnnotationDL*)axdo->getExtension("DKAnnotationDL");
	    cout<<"  getName="<<ann->getName()<<endl;
        cout<<"  pageNumber= "<<ann->getPageNumber()<<endl;
        cout<<"  partId="<<ann->getPart()<<endl;
        cout<<"  X= "<<ann->getX()<<endl;
        cout<<"  Y= "<<ann->getY()<<endl;
     }

     //***** after retrieve get SMS 
       cout <<"about to retrieve the DKStorageManageInfoDL ...."<<endl; 
       DKStorageManageInfoDL* aSMS2 =  (DKStorageManageInfoDL*)axdo->getExtension("DKStorageManageInfoDL");
       cout<<"  collectionName= "<<aSMS2->getCollectionName()<<endl;
       cout<<"  managementClass="<<aSMS2->getManagementClass()<<endl;
       cout<<"  storageClass="<<aSMS2->getStorageClass()<<endl;
       cout<<"  retention="<<aSMS2->getRetention()<<endl;
       cout<<"  storeSite="<<aSMS2->getStoreSite()<<endl;
    
     //**** reset partId to 0 and add again to see if SMS is correct as NULL-default
     apid ->setPartId(0);
     axdo ->setPid(apid);
     cout <<"about to do add(fileName) again"<<endl;
     axdo ->add(fileName);       
     cout<<"add success...";
     cout<<"after add()partId="<<((DKPidXDODL*)(axdo->getPidObject()))->getPartId()<<endl;

     delete axdo;
     delete apid;
     dsDL.disconnect();
     cout<<"datastore disconnected"<<endl;
    }
    catch(DKException &exc)
    {
      cout << "Error id" << exc.errorId() << endl;
      cout << "Exception id " << exc.exceptionId() << endl;
      for(unsigned long i = 0; i< exc.textCount();i++)
      {
       cout << "Error text:" << exc.text(i) << endl;
      }
      for (unsigned long g=0; g< exc.locationCount();g++)
      {
       const DKExceptionLocation* p = exc.locationAtIndex(g);
       cout << "Filename: " << p->fileName() << endl;
       cout << "Function: " << p->functionName() << endl;
       cout << "LineNumber: " << p->lineNumber() << endl;
      }
      cout << "Exception Class Name: " << exc.name() << endl;
    }   
    cout << "done ..." << endl;      
}


