/*****************************************************************************************/
/*BEGINPROLOGUE************************************************************************
 * @copyright(disclaimer)                                                             *
 * Licensed Materials - Property of IBM                                               *
 * IBM Enterprise Information Portal for Multiplatforms V8.1 (program number 5724-B43)*
 * (c ) Copyright IBM Corp. 1998,  2002. All Rights Reserved.                         *
 *                                                                                    *
 * US Government Users Restricted Rights                                              *
 * Use, duplication or disclosure restricted by GSA ADP Schedule                      *
 * Contract with IBM Corp.                                                            *
 *                                                                                    *
 * DISCLAIMER OF WARRANTIES :                                                         *
 *                                                                                    *
 * Permission is granted to copy and modify this Sample code, and to                  *
 * distribute modified versions provided that both the copyright                      *
 * notice,- and this permission notice and warranty disclaimer appear in all copies   * 
 * and modified versions.                                                             *
 *                                                                                    *
 * THIS SAMPLE CODE IS LICENSED TO YOU AS-IS.                                         *
 * IBM AND ITS SUPPLIERS AND LICENSORS DISCLAIM ALL WARRANTIES, EITHER EXPRESS OR     *
 * IMPLIED, IN SUCH SAMPLE CODE, INCLUDING THE WARRANTY OF NON-INFRINGEMENT AND THE   *
 * IMPLIED WARRANTIES OF MERCHANTABILITY OR FITNESS FOR A PARTICULAR PURPOSE. IN NO   *
 * EVENT WILL IBM OR ITS LICENSORS OR SUPPLIERS BE LIABLE FOR ANY DAMAGES ARISING OUT *
 * OF THE USE OF OR INABILITY TO USE THE SAMPLE CODE, DISTRIBUTION OF THE SAMPLE CODE,*
 * OR COMBINATION OF THE SAMPLE CODE WITH ANY OTHER CODE. IN NO EVENT SHALL IBM OR ITS*
 * LICENSORS AND SUPPLIERS BE LIABLE FOR ANY LOST REVENUE, LOST PROFITS OR DATA, OR   *
 * FOR DIRECT, INDIRECT, SPECIAL, CONSEQUENTIAL,INCIDENTAL OR PUNITIVE DAMAGES,       *
 * HOWEVER CAUSED AND REGARDLESS OF THE THEORY OF LIABILITY,-, EVEN IF IBM OR ITS     *
 * LICENSORS OR SUPPLIERS HAVE BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGES.       *
 *                                                                                    *
 * @endCopyright                                                                      *
 **************************************************************************************/
/*****************************************************************************************/

// sample connect/execute/retrieve/delete note/disconnect
/*
 * Filename       : TDelAnnotationOD.cpp
 * Description    : Connect to OD Datastore.
 *                  Create query string.
 *                  Specify to retrieve folder and documents.
 *                  Specify to retrieve a max result of 5 items.
 *                  Get the result in a resultset cursor.
 *                  Loop thru results.
 *                  Display documents found and their parts and attributes,
 *                   if folders found display them and their documents/
 *                   folders.
 *                  Delete the first note found
 *                  Disconnect from OD Datastore.
 */

#include <iostream.h>
#include <stdio.h>
#include <DKConstant.h>
#include <DKDatastoreOD.hpp>
#include <DKException.hpp>
#include <DKString.hpp>
#include <dkResultSetCursor.hpp>
#include <DKDDO.hpp>
#include <DKAny.hpp>
#include <DKPid.hpp>
#include <dkCollection.hpp>
#include <DKBlobOD.hpp>
#include <DKPidXDOOD.hpp>
#include <DKEnvironment.hpp>

#ifdef _DEBUG
#define new DEBUG_NEW
#endif

static DKBoolean CompareIgnoreCase(DKString str1, DKString str2);
static DKString ODdate( int month, int day, int year );
static void displayNotes(DKDDO* ddo);
static void deleteNotes(DKDDO* ddo);
static void setAGQueryCriteria(int testcase, DKString libSrv);

static DKString appgrp = "";
static DKString cmd    = "";

void main(int argc, char *argv[])
{
  cout << "begin..." << endl;

  // Indicate this is a Console Application on Windows NT/95 for error
  // message code page conversion to OEM code page. If this was a Windows
  // GUI (Graphic User Interface) Application this would not be necessary
  // because DK_CM_SS_WINDOWS is the default.
  DKEnvironment::setSubSystem(DK_CM_SS_CONSOLE);

  //--------------------------------------------------------------
  // Checking for input parameters
  //--------------------------------------------------------------
  if (argc < 4)
  {
    cout<<"Usage: "<<endl;
    cout<<"  TDelAnnotationOD <libsrv> <userid> <pw> <connect_string>" <<endl;
    return;
  }
  // ### get input parameters
  DKString libsrv = DKString(argv[1]);
  DKString userid = DKString(argv[2]);
  DKString pw = DKString(argv[3]);
  DKString connect_string = "";
  if (argc > 4 )
    connect_string = argv[4];
  DKString cmd;

  DKDatastoreOD dsOD; // must be outside try/catch block

  try
  {
    dkResultSetCursor* pCur = 0;

    //@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
    // Select a testcase by changing this number!!!
    int testcase = 10;
    //@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
    setAGQueryCriteria(testcase, libsrv);

    DKNVPair parms[4];
    parms[0] = DKNVPair("APPL_GROUP", appgrp);
    parms[1] = DKNVPair("MAX_RESULTS", DKString(1));
    parms[2] = DKNVPair("CONTENT", DKString("YES"));
    parms[3] = DKNVPair( DK_CM_PARM_END, DKAny((long)0) );

    cout << "\nApplication group =>" << appgrp << "<=" << endl;
    cout << "Query string =>" << cmd << "<=\n" << endl;

    cout << "Datastore OD created" << endl;
    cout << "connecting to datastore" << endl;
    dsOD.connect(libsrv,userid,pw,connect_string);
    cout << "datastore connected " << libsrv << " userid - " << userid << endl;
    cout << "connect_string=" << connect_string << endl;
    cout << "executing query" << endl;

    DKTimestamp timeBefore;
    cout << "Before query, time stamp is " << timeBefore.asString(NULL) << endl;

    cout << "executing query" << endl;
    pCur = dsOD.execute(cmd,DK_CM_SQL_QL_TYPE,parms);
    cout << "query executed" << endl;

    DKTimestamp timeAfter;
    cout << "After query, time stamp is " << timeAfter.asString(NULL) << endl;
    cout << "cardinality: " << pCur->cardinality() << endl;
    cout << "datastore executed query" << endl;

    if (pCur == NULL || pCur->isValid() == FALSE)
    {
      cout << "cursor is NULL or invalid!!!" << endl;
      return;
    }

    DKDDO* ddo = 0;
    ddo = pCur->fetchNext();
    if (ddo != NULL)
    {
      displayNotes(ddo);
      deleteNotes(ddo);
    }
    delete ddo;
    delete pCur;

    // find out if the note is indeed added
    cout << "\nexecuting query 2nd time..." << endl;
    pCur = dsOD.execute(cmd,DK_CM_SQL_QL_TYPE,parms);
    cout << "number of hits: " << pCur->cardinality() << endl;

    if (pCur == NULL || pCur->isValid() == FALSE)
    {
      cout << "cursor is NULL or invalid!!!" << endl;
      return;
    }

    ddo = pCur->fetchNext();
    if (ddo != NULL)
    {
      displayNotes(ddo);
    }
    delete pCur;
    dsOD.disconnect();
    cout << "\ndatastore disconnected" << endl;
  }

  catch (DKException &exc)
  {
    cout << "Error id " << exc.errorId() << endl;
    cout << "Exception id " << exc.exceptionId() << endl;
    for (unsigned long i = 0; i< exc.textCount();i++)
    {
      cout << "Error text: " << exc.text(i) << endl;
    }
    for (unsigned long g=0; g< exc.locationCount();g++)
    {
      const DKExceptionLocation* p = exc.locationAtIndex(g);
      cout << "Filename: " << p->fileName() << endl;
      cout << "Function: " << p->functionName() << endl;
      cout << "LineNumber: " << p->lineNumber() << endl;
    }
    cout << "Exception Class Name: " << exc.name() << endl;
  }
  cout << "done ..." << endl;
}

// --------------------------------------------------------------------
// Convert regular month, day, year values to an OnDemand integer date
// Asumming (and hopefully) the year is in the yyyy format
// --------------------------------------------------------------------
static DKString ODdate( int month, int day, int year )
{
  int days[] = {31,28,31,30,31,30,31,31,30,31,30,31};
  int OD_days = 0;

  // Just in case someone entered a 2 digit year
  if ( year < 100 )
  {
    // 21st century
    if (year < 70)
      year += 2000;
    else
      year += 1900;
  }

  // Get the numbers of days since 1/1/1970 to 1/1/yyyy
  OD_days = ( year - 1970 ) * 365;

  // Add days from leap years
  for ( int yr = 1970; yr < year; yr++ )
  {
    if ((((yr) % 4) == 0) && ((((yr) % 100) != 0) || (((yr) % 400) == 0)))
    {
      OD_days++;
    }
  }

  // Add days sicne 1/1 of this year
  for ( int mon = 0; mon < month-1; mon++ )
  {
    OD_days += days[mon];
  }

  // Add the current day
  OD_days += day;

  // Additional Leap year adjustment
  if ((((year) % 4) == 0) && ((((year) % 100) != 0) || (((year) % 400) == 0)) && month > 2)
    OD_days++;

  DKString od_days_str( OD_days );

  return od_days_str;
}

//--------------------------------------------------------------------
// Display notes
//--------------------------------------------------------------------
static void displayNotes(DKDDO* ddo)
{
  int i = 0;
  dkXDO* pXDO = NULL;
  DKString MimeType;
  DKString noteString;

  dkDataObjectBase * pDO = NULL;

  try
  {
    short propId = ddo->dataId("DKAnnotations");
    if (propId == 0)
    {
      DKAny any;
      cout << "No annotations for document!" << endl;
    }
    else
    {
      DKAny a = ddo->getData(propId);
      dkCollection* notes = a;

      DKAnnotationOD* note;

      int numNotes = notes->cardinality();
      if (numNotes > 1)
        cout << "\nFound " << numNotes << " annotations." << endl;
      else
        cout << "\nFound " << numNotes << " annotation." << endl;

      dkIterator* iter = notes->createIterator();
      if (iter)
      {
        while (iter->more())
        {
          i++;
          a = *(iter->next());
          pDO = a;
          pXDO = (DKBlobOD*)pDO;

          cout << "   NOTE #" << i << endl;

          // Retrieve and open instance handler for an XDO
          pXDO->retrieve();
          MimeType = pXDO->getMimeType();
          cout << "   MimeType = " << MimeType << endl;

          note = (DKAnnotationOD*)((DKBlobOD*)pXDO)->getExtension("dkAnnotationExt");

          noteString  = "   Note Info: " + note->getTimestamp();
          noteString += " by " + note->getUserID();
          noteString += " for page " + DKString(note->getPageNumber());
          cout << noteString << endl;

          noteString = note->isPublic() ? "   Public" : "   Private";
          noteString += "--";
          noteString += note->isOkToCopy() ? "Can" : "Cannot";
          noteString += " be copied to another server";
          cout << noteString << endl;

          noteString = "   Note Text: " + note->getText();
          cout <<  noteString << endl;
          cout << "\n" << endl;

        } //while (iter->more())

        delete iter;
      }

    } //if (propId != 0)

  } //try

  catch (DKException &exc)
  {

    cout << "Error id " << exc.errorId() << endl;
    cout << "Exception id " << exc.exceptionId() << endl;
    for (unsigned long i = 0; i< exc.textCount();i++)
    {
      cout << "Error text: " << exc.text(i) << endl;
    }
    for (unsigned long g=0; g< exc.locationCount();g++)
    {
      const DKExceptionLocation* p = exc.locationAtIndex(g);
      cout << "Filename: " << p->fileName() << endl;
      cout << "Function: " << p->functionName() << endl;
      cout << "LineNumber: " << p->lineNumber() << endl;
    }
    cout << "Exception Class Name: " << exc.name() << endl;
  }
}

//--------------------------------------------------------------------
// Delete notes
//--------------------------------------------------------------------
static void deleteNotes(DKDDO* ddo)
{
  int i = 0;
  dkXDO* pXDO = NULL;
  dkDataObjectBase * pDO = NULL;

  DKString MimeType;

  try
  {
    short propId = ddo->dataId("DKAnnotations");
    if (propId == 0)
    {
      cout << "\nNo annotations to delete!" << endl;
    }
    else
    {
      DKAny a = ddo->getData(propId);
      dkCollection* notes = a;

      dkIterator* iter = notes->createIterator();
      if (iter)
      {
        while (iter->more())
        {
          i++;
          a = *(iter->next());
          pDO = a;
          pXDO = (DKBlobOD*)pDO;

          // Retrieve the content for XDO
          pXDO->retrieve();
          MimeType = pXDO->getMimeType();
          cout << "  MimeType =>" << MimeType << "<=" << endl;

          // only delete the first one
          if (i == 1)
          {
            cout << "\ndeleting the first annotation..." << endl;
            pXDO->del();
            break;
          }

        } //while (iter->more())

        delete iter;
      }

    } //if (propId != 0)

  } //try

  catch (DKException &exc)
  {
    cout << "Error id " << exc.errorId() << endl;
    cout << "Exception id " << exc.exceptionId() << endl;
    for (unsigned long i = 0; i< exc.textCount();i++)
    {
      cout << "Error text: " << exc.text(i) << endl;
    }
    for (unsigned long g=0; g< exc.locationCount();g++)
    {
      const DKExceptionLocation* p = exc.locationAtIndex(g);
      cout << "Filename: " << p->fileName() << endl;
      cout << "Function: " << p->functionName() << endl;
      cout << "LineNumber: " << p->lineNumber() << endl;
    }
    cout << "Exception Class Name: " << exc.name() << endl;
  }
}

// --------------------------------------------------------------------
// Compare two strings case insensitively
// --------------------------------------------------------------------
static DKBoolean CompareIgnoreCase(DKString str1, DKString str2)
{
#ifdef WIN32
  return (str1.CompareNoCase(str2) == 0);
#else
  return (IString::lowerCase(str1) == IString::lowerCase(str2));
#endif
}

// --------------------------------------------------------------------
// set application group and query SQL
// --------------------------------------------------------------------
static void setAGQueryCriteria(int testcase, DKString libsrv)
{
  int month = 6;
  int day   = 2;
  int year  = 1997;

  DKString  mon_str( month );
  DKString  day_str( day );
  DKString  yr_str( year );
  DKString  date_str;
  date_str = mon_str + "/" + day_str + "/" + yr_str;
  DKString od_date = ODdate( month, day, year );

  // Application Group and SQL string
  // OD/390 v2.1
  if (CompareIgnoreCase(libsrv, "9.91.121.31"))
  {
    switch (testcase)
    {
      default:
      case 1:  // AFP
        appgrp = "INSURE  01 ";
        cmd = "WHERE (POLICY_NUMBER LIKE '324-1443255%')";  //'324-1443255-11' ";
        break;
      case 2:
        appgrp = "CHKSTMN301 ";
        cmd = "WHERE ACCOUNT_NUMBER LIKE '%' ";
        break;
      case 3:  // line/large object
        appgrp = "DELINQCY02 ";
        cmd = "WHERE ( LOAN_NUMBER LIKE '%' )";
        break;
      case 4:  // pdf
        appgrp = "KEYBANK 01 ";
        cmd = "WHERE ( ACCOUNT_NUMBER LIKE '%' )";
    }
  }
  // OD/390 v7.1
  else if (CompareIgnoreCase(libsrv, "9.91.121.34"))
  {
    switch (testcase)
    {
      default:
      case 1: // AFP data
        appgrp = "Credit Card Statements";
        cmd = "WHERE ( name LIKE '%' )";
        break;
      case 2:
        appgrp = "CHKSTMNT";
        cmd = "WHERE ACCOUNT_NUMBER LIKE '%' ";
        break;
      case 3:  // line data
        appgrp = "Delinqcy-AG";
        cmd = "WHERE ( pagenum != 0 )";
        break;
      case 4:  // line data large object
        appgrp = "LOAN LO";
        od_date = ODdate( 10, 1, 1994 );
        cmd = "WHERE ( load_date = " + od_date + " )";
        //od_date = ODdate( 2, 2, 1970 );
        //cmd  = "WHERE (( load_date > " + od_date + ") AND ";
        //cmd += "(bgn_loan_num <= 10000 AND end_loan_num >= 10000) )";
        break;
      case 5:  // pdf
        appgrp = "PDF landscape";
        cmd = "WHERE ( region LIKE '%' )";
      case 6:  // AFP
        appgrp = "afpdata";
        cmd = "WHERE (TaxYear = 1995)";
        break;
    }
  }
  else if (CompareIgnoreCase(libsrv, "gunnar"))
  {
    switch (testcase)
    {
      default:
      case 1: // Large object AFP - gunnar
        appgrp = "OnDemand Publications";
        cmd = "where bookname LIKE '%'";
        break;
    }
  }
  else if (CompareIgnoreCase(libsrv.subString(1, 7), "everest"))
  {
    switch (testcase)
    {
      default:
      case 1:
        // AFP data
        appgrp = "CRD";
        cmd = "WHERE ( name LIKE '%' )";
        break;
      case 2:
        // Large Object line data
        appgrp = "sp-ebcdic-lo";
        //od_date = ODdate( 1, 1, 1990 );
        //od_date2 = ODdate( 12, 31, 2001 );
        //cmd = "WHERE ( rpt_date BETWEEN " + od_date + " AND " + od_date2 + " )";
        od_date = ODdate( 12, 15, 2000 );
        cmd = "WHERE ( rpt_date > 0 )"; //= " + od_date + " )";
        break;
      case 3: // Loan Delinquency Reports - Large Objects
        appgrp = "LOAN LO";
        od_date = ODdate( 2, 2, 1970 );   // 2/2/1970
        cmd  = "WHERE (( load_date > " + od_date + ") AND ";
        cmd += "(bgn_loan_num <= 10000 AND end_loan_num >= 10000) )";
        break;
      case 4:
        // Large Object line data
        appgrp = "eliz-largeline";
        od_date = ODdate( 3, 19, 1999 );
        cmd = "WHERE ( rptdate = " + od_date + " )";
        break;
      case 5:
        // PDF
        appgrp = "43MBPDF";
        cmd = "WHERE ( rtitle LIKE '%' )";
        break;
    }
  }
  // OD/400 v4.5
  else if (CompareIgnoreCase(libsrv.subString(1, 7), "rdr400c"))
  {
    switch (testcase)
    {
      default:
      case 1:
        appgrp = "CHECKSTMTS ";
        cmd = "WHERE ( KY1VAL LIKE '%' )";
        break;
    }
  }
  // OD/400 v5.1
  else if (CompareIgnoreCase(libsrv.subString(1, 7), "rdr400g"))
  {
    switch (testcase)
    {
      default:
      case 1:
        appgrp = "CheckStatements";
        cmd = "WHERE ( name LIKE '%' )";
        break;
    }
  }
  else // can be any server!
  {
    switch (testcase)
    {
      default:
      case 1: // AFP
        // native OD integer date
        od_date = ODdate( month, day, year );
        cout << "The date '" << date_str <<"' equals to the OnDemand integer date '" << od_date<<"'" << endl;

        appgrp = "CRD";
        //cmd = "WHERE ( crd_date = " + od_date + " )";
        cmd = "WHERE ( name LIKE '%')";
        break;
      case 2:  // line
        appgrp = "Balance";
        cmd = "WHERE ( hiercode LIKE '..21...002')"; //'%.....006')";
        break;
    }
  }
}
