/*BEGINPROLOGUE************************************************************************
 * @copyright(disclaimer)                                                             *
 * Licensed Materials - Property of IBM                                               *
 * IBM Information Integrator for Content V8.2  (program number 5724-B43)             *
 * (c ) Copyright IBM Corp. 1998, 2002, 2003. All Rights Reserved.                    *
 *                                                                                    *
 * US Government Users Restricted Rights                                              *
 * Use, duplication or disclosure restricted by GSA ADP Schedule                      *
 * Contract with IBM Corp.                                                            *
 *                                                                                    *
 * DISCLAIMER OF WARRANTIES :                                                         *
 *                                                                                    *
 * Permission is granted to copy and modify this Sample code, and to                  *
 * distribute modified versions provided that both the copyright                      *
 * notice,- and this permission notice and warranty disclaimer appear in all copies   * 
 * and modified versions.                                                             *
 *                                                                                    *
 * THIS SAMPLE CODE IS LICENSED TO YOU AS-IS.                                         *
 * IBM AND ITS SUPPLIERS AND LICENSORS DISCLAIM ALL WARRANTIES, EITHER EXPRESS OR     *
 * IMPLIED, IN SUCH SAMPLE CODE, INCLUDING THE WARRANTY OF NON-INFRINGEMENT AND THE   *
 * IMPLIED WARRANTIES OF MERCHANTABILITY OR FITNESS FOR A PARTICULAR PURPOSE. IN NO   *
 * EVENT WILL IBM OR ITS LICENSORS OR SUPPLIERS BE LIABLE FOR ANY DAMAGES ARISING OUT *
 * OF THE USE OF OR INABILITY TO USE THE SAMPLE CODE, DISTRIBUTION OF THE SAMPLE CODE,*
 * OR COMBINATION OF THE SAMPLE CODE WITH ANY OTHER CODE. IN NO EVENT SHALL IBM OR ITS*
 * LICENSORS AND SUPPLIERS BE LIABLE FOR ANY LOST REVENUE, LOST PROFITS OR DATA, OR   *
 * FOR DIRECT, INDIRECT, SPECIAL, CONSEQUENTIAL,INCIDENTAL OR PUNITIVE DAMAGES,       *
 * HOWEVER CAUSED AND REGARDLESS OF THE THEORY OF LIABILITY,-, EVEN IF IBM OR ITS     *
 * LICENSORS OR SUPPLIERS HAVE BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGES.       *
 *                                                                                    *
 * @endCopyright                                                                      *
 **************************************************************************************

Item Types can be deleted by using either their own del() functions or by the Datastore 
Definition's del() function.

Depending on the product configuration, subEntities / child components / multivalue attributes
may be required to be deleted first, before the root Item Type can be deleted.  If this is 
the case, please refer to Sample SItemTypeRetrievalICM for learning how to access the
Sub-Entities (See Listing Sub-Entitiesin SItemTYpeRetrievalICM).  Once a reference to the 
entitiy definition is objetained, the del() method may be called to deleted it.

NOTE:  When retrieving Item Type Definitions, if it does not exist, null is returned.  
       In this sample, if null is detected, no action will take place for that particular 
       Item Type definition because it was not defined.

********************************************************************/

// Includes / Imports
#include <iostream.h>
#include <DKDatastoreICM.hpp>
#include <DKItemTypeDefICM.hpp>
#include <DKItemTypeRelationDefICM.hpp>
#include <DKComponentTypeDefICM.hpp>
#include <DKConstant.h>
#include <DKConstantICM.h>
#include "SConnectDisconnectICM.hpp"
#include "SItemTypeDeletionICM.hpp"

/************************************************************************************************
 *          FILENAME: SItemTypeDeletionICM.cpp
 *                    ---------------------------------------------------------------------------
 *       DESCRIPTION: Deleting Item Type Definitions
 *                    ---------------------------------------------------------------------------
 *     DEMONSTRATION: Deleting Item Type Definitions Using its del() Method
 *                    Deleting Item Type Definitions Using the Datastore Definition's del() Method
 *                    Deletes all Insurance Model Item Types
 *                    Deletes all Item Types Defined in Sample SItemTypeCreationICM
 *                    ---------------------------------------------------------------------------
 * COMMANDLINE USAGE: SItemTypeDeletionICM <database> <userName> <password>
 *                    ---------------------------------------------------------------------------
 *      PREREQUISITS: Item Types & Component Types should be defined.  If not, you may run Sample 
 *                    SItemTypeCreationICM first, otherwise sample will complete, but since
 *                    none of the Item Types will be found, no action will take place.
 *                    ---------------------------------------------------------------------------
 *    FILES REQUIRED: SConnectDisconnectICM.hpp/cpp
 *                    SItemTypeDeletionICM_launch.cpp
 *                    SItemTypeDeletionICM.hpp
 ************************************************************************************************/

    //-------------------------------------------------------------
    // Main
    //-------------------------------------------------------------
    /**
     * Run the Sample.
     * @param argc   Argument Count / Number of Arguments Specified
     * @param argv[] String Array containing arguments.  Optional arguments are <databse> <userName> <password>.
     * @return  Returns SConnectDisconnectICM_RC_SAMPLE_PASSED (0) if no sample program completed, SConnectDisconnectICM_RC_SAMPLE_FAILED (-1) if any errors / exceptions were caught.
     **/
    int SItemTypeDeletionICM::main(int argc, char *argv[]){

        // Defaults for connecting to the database.
        char * database = SConnectDisconnectICM_DEFAULT_DATABASE;
        char * userName = SConnectDisconnectICM_DEFAULT_USERNAME;
        char * password = SConnectDisconnectICM_DEFAULT_PASSWORD;

        //------------------------------------------------------------
        // Checking for input parameters
        //--------------------------------------------------------------
        if (argc < 4) { // if not all 3 arguments were specified, use defaults and report correct usage.
            cout << "Usage: "                                                  << endl;
            cout << "  SItemTypeDeletionICM <database> <userName> <password>"  << endl;
            cout << "  *** Some parameters not specified, using defaults..."   << endl;
            cout << endl;
        } else {  // otherwise enough parameters were specified, use the first 3 for the 3 parameters.
            if (argc > 1) database = argv[1];
            if (argc > 2) userName = argv[2];
            if (argc > 3) password = argv[3];
        }//end else
        char * ver = SConnectDisconnectICM_EIP_VERSION;

        cout << "==========================================="     << endl;
        cout << "IBM Information Integrator for Content v" << ver << endl;
        cout << "Sample Program:  SItemTypeDeletionICM"           << endl;
        cout << "-------------------------------------------"     << endl;
        cout << " Database: " << database                         << endl;
        cout << " UserName: " << userName                         << endl;
        cout << "===========================================\n"   << endl;

        try{
            //-------------------------------------------------------------
            // Connect to datastore
            //-------------------------------------------------------------
            // See Sample SConnectDisconnectICM for more information
            cout << "Connecting to datastore (Database '" << database << "', UserName '" << userName << "')..." << endl;

                DKDatastoreICM* dsICM = new DKDatastoreICM();  // Create new datastore object.
                dsICM->connect(database,userName,password,""); // Connect to the datastore.

            cout << "Connected to datastore (Database '" << dsICM->datastoreName() << "', UserName '" << dsICM->userName() << "')." << endl;

            //-------------------------------------------------------------------
            // Deleting Item Type Definition using it's del() method
            //-------------------------------------------------------------------
            // Item Types can be deleted by using either their own del() functions
            // or by the Datastore Definition's del() function.
            cout << "Retrieving & Deleting Item Type Definition of 'S_simple'..." << endl;

                DKDatastoreDefICM* dsDefICM = (DKDatastoreDefICM*) dsICM->datastoreDef(); // Get the datastore definition object from the connected datastore.
                DKItemTypeDefICM* itemTypeDef = (DKItemTypeDefICM*) dsDefICM->retrieveEntity("S_simple"); // Retrieve the Item Type Definition.

                if(itemTypeDef!=NULL){   // if it is not already deleted...
                    itemTypeDef->del();  // Delete the Item Type Definition
                    delete(itemTypeDef); // Free Memory
                }

            cout << "Deleted Item Type Definition of 'S_simple'." << endl;

            //----------------------------------------------------------------------------
            // Deleting Item Type Definition using the Datastore Definition's del() method
            //----------------------------------------------------------------------------
            cout << "Retrieving & Deleting Item Type Definition of 'S_withMultAttrs'..." << endl;

                itemTypeDef = (DKItemTypeDefICM*) dsDefICM->retrieveEntity("S_withMultAttrs"); // Retrieve the Item Type Definition.

                if(itemTypeDef!=NULL){          // if it is not already deleted...
                    dsDefICM->del(itemTypeDef); // Delete the Item Type Definition
                    delete(itemTypeDef);        // Free Memory
                }

            cout << "Deleted Item Type Definition of 'S_withMultAttrs'." << endl;

            //----------------------------------------------------------------
            // Delete the rest of the Item Types created in the creation sample
            //----------------------------------------------------------------
            // Delete all Item Types created in the SItemTypeCreationICM
            // sample, including the Insurance Model.
            deleteItemTypeDef(dsICM,"S_autoClaim");
            deleteItemTypeDef(dsICM,"S_adjustrReport");
            deleteItemTypeDef(dsICM,"S_policeReport");
            deleteItemTypeDef(dsICM,"S_insurePolicy");
            deleteItemTypeDef(dsICM,"S_autoPhoto");
            deleteItemTypeDef(dsICM,"S_docModel");
            deleteItemTypeDef(dsICM,"S_lob");
            deleteItemTypeDef(dsICM,"S_text");
            deleteItemTypeDef(dsICM,"S_image");
            deleteItemTypeDef(dsICM,"S_withChild");

            //-------------------------------------------------------------
            // Disconnect from datastore & Destroy Reference
            //-------------------------------------------------------------
            // See Sample SConnectDisconnectICM for more information
            cout << "Disconnecting from datastore & destroying reference..." << endl;            

                dsICM->disconnect();
                delete(dsICM);

            cout << "Disconnected from datastore & destroying reference." << endl;
    
            //-------------------------------------------------------------
            // Sample program completed without exception
            //-------------------------------------------------------------
            cout << "\n==========================================" << endl;
            cout << "Sample program completed."                    << endl;
            cout << "==========================================\n" << endl;
        }
        //------------------------------------------------------------
        // Catch & Print Exceptions        
        //------------------------------------------------------------
        catch (DKException &exc){
            SConnectDisconnectICM::printException(exc);    // Print the exception to the screen
            return(SConnectDisconnectICM_RC_SAMPLE_FAILED);  // Report using return code that it failed
        }
        return(SConnectDisconnectICM_RC_SAMPLE_PASSED);    // If no exceptions were thrown, return a code to indicate passed.
    }// end main

    //=================================================================
    // Wrapper Functions
    //=================================================================
    // The following are wrapper functions for functionality covered
    // in this sample.  These functions may be used by other samples.

   /**
    * Delete the specified Item Type Definition
    * @param dsICM         Connected DKDatastoreICM object.
    * @param itemTypeName  Item Type Name for the Item Type Definition to Delete.
    **/
    void SItemTypeDeletionICM::deleteItemTypeDef(DKDatastoreICM * dsICM, char * itemTypeName){
        cout << "Deleting the Item Type Definition '" << itemTypeName << "'..." << endl;

            // First, we need acces to the Datastore Definition object.
            DKDatastoreDefICM* dsDefICM = (DKDatastoreDefICM*) dsICM->datastoreDef(); // Get the datastore definition object from the connected datastore.
            DKItemTypeDefICM* itemTypeDef = (DKItemTypeDefICM*) dsDefICM->retrieveEntity(itemTypeName); // Retrieve the definition.
            if(itemTypeDef!=NULL){    // if it is not already deleted...
                // If it is a CM Doc Model Item Type, delete an Item Type Relations
                if(itemTypeDef->getClassification()==DK_ICM_ITEMTYPE_CLASS_DOC_MODEL){
                    cout << "Deleting Item Type Relations..." << endl;
                    dkCollection* coll = dsDefICM->retrieveItemTypeRelations(itemTypeDef->getItemTypeId());
                    dkIterator* iter = coll->createIterator();
                    while(iter->more()){
                        DKItemTypeRelationDefICM* itemTypeRel = (DKItemTypeRelationDefICM*) iter->next()->value();
                        itemTypeRel->del();
                        delete(itemTypeRel);  // Free Memory
                    }
                    delete(iter);             // Free Memory
                    delete(coll);
                }
                itemTypeDef->del();   // Delete the Attribute Definition
                delete(itemTypeDef);  // Free Memory
            }

        cout << "Deleted the Item Type Definition '" << itemTypeName << "'." << endl;
    }
