/*BEGINPROLOGUE************************************************************************
 * @copyright(disclaimer)                                                             *
 * Licensed Materials - Property of IBM                                               *
 * IBM Information Integrator for Content V8.2  (program number 5724-B43)             *
 * (c ) Copyright IBM Corp. 1998, 2002, 2003. All Rights Reserved.                    *
 *                                                                                    *
 * US Government Users Restricted Rights                                              *
 * Use, duplication or disclosure restricted by GSA ADP Schedule                      *
 * Contract with IBM Corp.                                                            *
 *                                                                                    *
 * DISCLAIMER OF WARRANTIES :                                                         *
 *                                                                                    *
 * Permission is granted to copy and modify this Sample code, and to                  *
 * distribute modified versions provided that both the copyright                      *
 * notice,- and this permission notice and warranty disclaimer appear in all copies   * 
 * and modified versions.                                                             *
 *                                                                                    *
 * THIS SAMPLE CODE IS LICENSED TO YOU AS-IS.                                         *
 * IBM AND ITS SUPPLIERS AND LICENSORS DISCLAIM ALL WARRANTIES, EITHER EXPRESS OR     *
 * IMPLIED, IN SUCH SAMPLE CODE, INCLUDING THE WARRANTY OF NON-INFRINGEMENT AND THE   *
 * IMPLIED WARRANTIES OF MERCHANTABILITY OR FITNESS FOR A PARTICULAR PURPOSE. IN NO   *
 * EVENT WILL IBM OR ITS LICENSORS OR SUPPLIERS BE LIABLE FOR ANY DAMAGES ARISING OUT *
 * OF THE USE OF OR INABILITY TO USE THE SAMPLE CODE, DISTRIBUTION OF THE SAMPLE CODE,*
 * OR COMBINATION OF THE SAMPLE CODE WITH ANY OTHER CODE. IN NO EVENT SHALL IBM OR ITS*
 * LICENSORS AND SUPPLIERS BE LIABLE FOR ANY LOST REVENUE, LOST PROFITS OR DATA, OR   *
 * FOR DIRECT, INDIRECT, SPECIAL, CONSEQUENTIAL,INCIDENTAL OR PUNITIVE DAMAGES,       *
 * HOWEVER CAUSED AND REGARDLESS OF THE THEORY OF LIABILITY,-, EVEN IF IBM OR ITS     *
 * LICENSORS OR SUPPLIERS HAVE BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGES.       *
 *                                                                                    *
 * @endCopyright                                                                      *
 **************************************************************************************

3rd Party Ingest
    Resource Content may be obtained from a 3rd Party source.  In this sample, Lob 
    Item(s) will be created from content stored or managed by a 3rd Party.
    
    The content may be loaded into the Resource Manager synchronously or asynchronously, as
    explained below:
    
    Synchronous Loading
        The content is fully-loaded into the Resource Manager before proceeding.
    
    Asynchronous Loading
        The upload is begun for loading the content into the Resource Manager, but the 
        program does not wait for the upload to complete.

    3rd Party Ingest Properties
        The following properties pertain to 3rd Party Ingest.
        
        Host Name
            The name of the computer storing the content to be loaded.

        Port
            Port at which to access the 3rd Party Host using the specified protocol.

        Protocol
            Protocol to be used.  Currently, "ftp" is the supported protocol.  Other
            protocols such as "http" & "file" are not currently supported.
            
        User Name
            User Name to log onto the 3rd Party Host specified by the Host Name.
            
        Password
            Password for the User Name to log onto the 3rd Party Host.

        File Name
            Fully-qualified path and file name for the content stored on the 3rd Party Host.
            Default directory is the defefault log-on directory.

        Content Length
            The file size in bytes / length of the entire content set to be stored.  If 
            multiple files are to be stored, the length will be the total length of all
            files.

        File Path (Optional)
            The directory where the specified File Name may be located.
        
        File List (Optional)
            Currently Not Supported for Standard Resources in Content Manager v8.2.  
            
            IMPORTANT:  This property should not be specified.
            
            In general, it is a comma-separated list of files.  Multiple files will be stored as one large 
            object, concatonated together.  The User-Program is responsible for breaking
            multiple files apart when retrieved.  They will be retrieved as one object.
                        
            A single file may be specified, matching the "File Name" property to load
            a single file.

********************************************************************/

// Includes / Imports
#include <iostream.h>
#include <DKConstant.h>
#include <DKConstantICM.h>
#include <DKDatastoreICM.hpp>
#include <DKLobICM.hpp>
#include <DKTextICM.hpp>
#include <DKImageICM.hpp>
#include "SConnectDisconnectICM.hpp"
#include "SResourceItem3rdPartyIngestICM.hpp"

/************************************************************************************************
 *          FILENAME: SResourceItem3rdPartyIngestICM.cpp
 *                    ---------------------------------------------------------------------------
 *       DESCRIPTION: Using 3rd Party Ingest to Create Items with Lob Resource Content.
 *                    ---------------------------------------------------------------------------
 *     DEMONSTRATION: 3rd Party Ingest
 *                    Synchronous vs Asynchronous
 *                    Creating Items with Lob Resource Content.
 *                    ---------------------------------------------------------------------------
 * COMMANDLINE USAGE  SResourceItem3rdPartyIngestICM  <3rd Party Host Name>
 *                                                    <3rd Party Port>
 *                                                    <3rd Party Protocol (http|ftp|file)>
 *                                                    <3rd Party User Name>
 *                                                    <3rd Party Password>
 *                                                    <3rd Party Fully-Qualified File Name>
 *                                                    <3rd Party File Path>
 *                                                    <3rd Party MIME Type>
 *                                                    <3rd Party Content Length>
 *                                                    <database> <userName> <password>
 *
 *                    NOTE:  All 3rd Party Information is Required
 *                        
 *                    NOTE:  Database, UserName, & Password are optional.  If all not specified
 *                           defaults will be used.
 *                           
 *                    Ex: SResourceItem3rdPartyIngestICM myFtpHostName 21 ftp myUserName myPassword SResourceItemICM_Document1.doc // application/msword 7168
 *                    Ex: SResourceItem3rdPartyIngestICM myFtpHostName 21 ftp myUserName myPassword SResourceItemICM_Picture1.bmp // image/mpg 120054 icmnlsdb icmadmin password
 *                    ---------------------------------------------------------------------------
 *      PREREQUISITS: The Data Model must be defined.  If needed please run the following Samples  
 *                        - SAttributeDefinitionCreationICM
 *                        - SAttributeGroupDefCreationICM
 *                        - SReferenceAttrDefCreationICM
 *                        - SItemTypeCreationICM
 *                    The Item Types must be set to use the specified default Resource Manager.
 *                    Plese run the following samples.
 *                        - SResourceMgrDefSetDefaultICM
 *                        - SSMSCollectionDefSetDefaultICM
 *                    NOTE:  Sample SSampleModelBuildICM pulls together all of the above.  It 
 *                           must be edited first for the RM Confiuration information.
 *                    The file specified in the commandline parameter must exist on a 3rd Party
 *                    host.  A recommended solution is to setup an FTP server, placing the 
 *                    SResourceItemICM_Document1.doc in the root directory.
 *                    ---------------------------------------------------------------------------
 *    FILES REQUIRED: SConnectDisconnectICM.hpp/cpp
 *                    SResourceItem3rdPartyIngestICM_launch.cpp
 *                    SResourceItem3rdPartyIngestICM.hpp
 *                    SResourceItemICM_Document1.doc -> On 3rd Party Host
 ************************************************************************************************/

    //-------------------------------------------------------------
    // Main
    //-------------------------------------------------------------
    /**
     * Run the Sample.
     * @param argc   Argument Count / Number of Arguments Specified
     * @param argv[] String Array containing arguments.  Optional arguments are <databse> <userName> <password>.
     * @return  Returns SConnectDisconnectICM_RC_SAMPLE_PASSED (0) if no sample program completed, SConnectDisconnectICM_RC_SAMPLE_FAILED (-1) if any errors / exceptions were caught.
     **/
    int SResourceItem3rdPartyIngestICM::main(int argc, char *argv[]){

        // Defaults for connecting to the database.
        char * database = SConnectDisconnectICM_DEFAULT_DATABASE;
        char * userName = SConnectDisconnectICM_DEFAULT_USERNAME;
        char * password = SConnectDisconnectICM_DEFAULT_PASSWORD;

        // 3rd Party Information
        char * thirdPartyHostName       = "NOT_SPECIFIED";    // Please refer to the header documentation for descriptions.
        int    thirdPartyPort           = -1;
        char * thirdPartyProtocol       = "NOT_SPECIFIED";
        char * thirdPartyUserName       = "NOT_SPECIFIED";
        char * thirdPartyPassword       = "NOT_SPECIFIED";
        char * thirdPartyFullFileName   = "NOT_SPECIFIED";     
        char * thirdPartyFilePath       = "NOT_SPECIFIED";
        char * thirdPartyMIMEType       = "NOT_SPECIFIED";
        int    thirdPartyContentLength  = -1;

        //------------------------------------------------------------
        // Checking for input parameters
        //--------------------------------------------------------------
        if (argc < 13) { // if not all arguments specified, report correct usage.
            cout << "Usage: " << endl;
            cout << "  SResourceItem3rdPartyIngestICM  <3rd Party Host Name>"                     << endl;
            cout << "                                  <3rd Party Port>"                          << endl;
            cout << "                                  <3rd Party Protocol (http|ftp|file)>"      << endl;
            cout << "                                  <3rd Party User Name>"                     << endl;
            cout << "                                  <3rd Party Password>"                      << endl;
            cout << "                                  <3rd Party Fully-Qualified File Name>"     << endl;
            cout << "                                  <3rd Party File Path>"                     << endl;
            cout << "                                  <3rd Party MIME TYpe>"                     << endl;
            cout << "                                  <3rd Party Content Length>"                << endl;
            cout << "                                  <database> <userName> <password>"          << endl;
            cout << endl;
            cout << "  NOTE:  All 3rd Party Information is Required."                             << endl;
            cout << endl;
            cout << "  NOTE:  Database, UserName, & Password are optional.  If all not specified" << endl;
            cout << "         defaults will be used."                                             << endl;
            cout << endl;
            cout << "  Ex: SResourceItem3rdPartyIngestICM myFtpHostName 21 ftp myUserName myPassword SResourceItemICM_Document1.doc // application/msword 7168"                    << endl;
            cout << "  Ex: SResourceItem3rdPartyIngestICM myFtpHostName 21 ftp myUserName myPassword SResourceItemICM_Picture1.bmp // image/mpg 120054 icmnlsdb icmadmin password" << endl;
            cout << endl;
        }
        if(argc < 10){ // If any 3rd Party info not specified, terminate
            cout << "===================================================" << endl;
            cout << " TERMINATED:  All 3rd Party info must be specified." << endl;
            cout << "===================================================" << endl;
            return(SConnectDisconnectICM_RC_SAMPLE_FAILED);        
        } else if(argc < 13) {
            cout << "*** Some parameters not specified.  Using defaults for rest. ***" << endl;
        }
        // Set Command-line options
        if (argc > 1)  thirdPartyHostName      = argv[1];
        if (argc > 2)  thirdPartyPort          = atoi(argv[2]);
        if (argc > 3)  thirdPartyProtocol      = argv[3];
        if (argc > 4)  thirdPartyUserName      = argv[4];
        if (argc > 5)  thirdPartyPassword      = argv[5];
        if (argc > 6)  thirdPartyFullFileName  = argv[6];
        if (argc > 7)  thirdPartyFilePath      = argv[7];
        if (argc > 8)  thirdPartyMIMEType      = argv[8];
        if (argc > 9)  thirdPartyContentLength = atoi(argv[9]);
        if (argc > 10) database                = argv[10];
        if (argc > 11) userName                = argv[11];
        if (argc > 12) password                = argv[12];

        char * ver = SConnectDisconnectICM_EIP_VERSION;

        cout << "==============================================="     << endl;
        cout << "IBM Information Integrator for Content     v" << ver << endl;
        cout << "Sample Program:  SResourceItem3rdPartyIngestICM"     << endl;
        cout << "-----------------------------------------------"     << endl;
        cout << " Content Manager:"                                   << endl;
        cout << "       Database:  " << database                      << endl;
        cout << "       UserName:  " << userName                      << endl;
        cout << " 3rd Party:"                                         << endl;
        cout << "      Host Name:  " << thirdPartyHostName            << endl;
        cout << "           Port:  " << thirdPartyPort                << endl;
        cout << "       Protocol:  " << thirdPartyProtocol            << endl;
        cout << "       UserName:  " << thirdPartyUserName            << endl;
        cout << "       Password:  " << thirdPartyPassword            << endl;
        cout << "           File:  " << thirdPartyFullFileName        << endl;
        cout << "      Mime Type:  " << thirdPartyMIMEType            << endl;
        cout << " Content Length:  " << thirdPartyContentLength       << endl;
        cout << "==============================================="     << endl;

        try{
            //-------------------------------------------------------------
            // Connect to datastore
            //-------------------------------------------------------------
            // See Sample SConnectDisconnectICM for more information
            cout << "Connecting to datastore (Database '" << database << "', UserName '" << userName << "')..." << endl;

                DKDatastoreICM* dsICM = new DKDatastoreICM();  // Create new datastore object.
                dsICM->connect(database,userName,password,""); // Connect to the datastore.

            cout << "Connected to datastore (Database '" << dsICM->datastoreName() << "', UserName '" << dsICM->userName() << "')." << endl;

            //-------------------------------------------------------------
            // Create Empty Resource Items
            //-------------------------------------------------------------
            cout << "Creating Empty Resource Items..." << endl;
    
                DKLobICM* lob1 = (DKLobICM*)dsICM->createDDO("S_lob", DK_CM_ITEM); 
                DKLobICM* lob2 = (DKLobICM*)dsICM->createDDO("S_lob", DK_CM_ITEM); 

            cout << "Created Empty Resource Items." << endl;

            //-------------------------------------------------------------
            // Setting the MIME Type (type of content)
            //-------------------------------------------------------------
            // mimeType: The type of content, such as "image/mpg"
            //
            // For a list of common MIME Types, please refer to the 
            // header documentation and SResourceItemMimeTypesICM.txt
			// located in the <CMBROOT>/Samples/java/icm directory.
            cout << "Setting MIME Type / Type of Content..." << endl;

                lob1->setMimeType(thirdPartyMIMEType);
                lob2->setMimeType(thirdPartyMIMEType);

            cout << "Set MIME Type / Type of Content." << endl;

            //-------------------------------------------------------------
            // Set 3rd Party Server Info
            //-------------------------------------------------------------
            // When the resource content is to be obtained from a 3rd party,
            // the 3rd party information may be specified as shown below.   
            // Please refer to the Application Programming Reference
            // for more informaiton.
            cout << "Setting 3rd Party Server Info..." << endl;

                // Lob 1
                lob1->setThirdPartyHostname(thirdPartyHostName);
                lob1->setThirdPartyPort(thirdPartyPort);
                lob1->setInputProtocol(thirdPartyProtocol);
                lob1->setThirdPartyUserid(thirdPartyUserName);
                lob1->setThirdPartyPassword(thirdPartyPassword);
                lob1->setInputFilename(thirdPartyFullFileName);
                lob1->setInputFilepath(thirdPartyFilePath);
                lob1->setInputContentlength(thirdPartyContentLength);

                // Lob 2
                lob2->setThirdPartyHostname(thirdPartyHostName);
                lob2->setThirdPartyPort(thirdPartyPort);
                lob2->setInputProtocol(thirdPartyProtocol);
                lob2->setThirdPartyUserid(thirdPartyUserName);
                lob2->setThirdPartyPassword(thirdPartyPassword);
                lob2->setInputFilename(thirdPartyFullFileName);
                lob2->setInputFilepath(thirdPartyFilePath);
                lob2->setInputContentlength(thirdPartyContentLength);

            cout << "Set 3rd Party Server Info." << endl;

            //-------------------------------------------------------------
            // Synchronous Store
            //-------------------------------------------------------------
            cout << "Storing Synchronously..." << endl;

                lob1->addFrom();  // Add from the 3rd Party specified above.
                
                // OR  the following command may be used if all 3rd Party info were not
                //     specified above.
                
                // lob1->addFrom(thirdPartyHostName, thirdPartyUserName, thirdPartyPassword, thirdPartyProtocol, thirdPartyPort, thirdPartyFullFileName);

            cout << "Stored Synchronously." << endl;

            //-------------------------------------------------------------
            // Asynchronous Store
            //-------------------------------------------------------------
            cout << "Storing Asynchronously..." << endl;

                // REQRUIEMENT:  In order for Asynchronous operations to complete,
                // the Resource Manager must be able to talk to the Library Server
                // independently, after this operation has completed.  In v8.2
                // this requires that both Library Server be cataloged on the
                // Resource Manager and/or be on the same machine.  If your system
                // meets these conditions, you may uncomment out the line below.

                //lob2->addFromAsync();  // Add from the 3rd Party specified above.

                // OR  the following command may be used if all 3rd Party info were not
                //     specified above.

                // lob2->addFromAsync(thirdPartyJostName, thirdPartyUserName, thirdPartyPassword, thirdPartyProtocol, thirdPartyPort, thirdPartyFullFileName);

                delete(lob1);          // Free Memory for object not needed any more in this sample.
                delete(lob2);

            cout << "Stored Asynchronously." << endl;

            //-------------------------------------------------------------
            // Disconnect from datastore & Destroy Reference
            //-------------------------------------------------------------
            // See Sample SConnectDisconnectICM for more information
            cout << "Disconnecting from datastore & destroying reference..." << endl;            

                dsICM->disconnect();
                delete(dsICM);

            cout << "Disconnected from datastore & destroying reference." << endl;
    
            //-------------------------------------------------------------
            // Sample program completed without exception
            //-------------------------------------------------------------
            cout << "\n==========================================" << endl;
            cout << "Sample program completed."                    << endl;
            cout << "==========================================\n" << endl;
        }
        //------------------------------------------------------------
        // Catch & Print Exceptions        
        //------------------------------------------------------------
        catch (DKException &exc){
            SConnectDisconnectICM::printException(exc);    // Print the exception to the screen
            return(SConnectDisconnectICM_RC_SAMPLE_FAILED);  // Report using return code that it failed
        }
        return(SConnectDisconnectICM_RC_SAMPLE_PASSED);    // If no exceptions were thrown, return a code to indicate passed.
    }// end main

    //=================================================================
    // Wrapper Functions
    //=================================================================
    // The following are wrapper functions for functionality covered
    // in this sample.  These functions may be used by other samples.

    // No additional wrapper functions from this sample.
