/*BEGINPROLOGUE************************************************************************
 * @copyright(disclaimer)                                                             *
 * Licensed Materials - Property of IBM                                               *
 * IBM Information Integrator for Content V8.2  (program number 5724-B43)             *
 * (c ) Copyright IBM Corp. 1998, 2002, 2003. All Rights Reserved.                    *
 *                                                                                    *
 * US Government Users Restricted Rights                                              *
 * Use, duplication or disclosure restricted by GSA ADP Schedule                      *
 * Contract with IBM Corp.                                                            *
 *                                                                                    *
 * DISCLAIMER OF WARRANTIES :                                                         *
 *                                                                                    *
 * Permission is granted to copy and modify this Sample code, and to                  *
 * distribute modified versions provided that both the copyright                      *
 * notice,- and this permission notice and warranty disclaimer appear in all copies   * 
 * and modified versions.                                                             *
 *                                                                                    *
 * THIS SAMPLE CODE IS LICENSED TO YOU AS-IS.                                         *
 * IBM AND ITS SUPPLIERS AND LICENSORS DISCLAIM ALL WARRANTIES, EITHER EXPRESS OR     *
 * IMPLIED, IN SUCH SAMPLE CODE, INCLUDING THE WARRANTY OF NON-INFRINGEMENT AND THE   *
 * IMPLIED WARRANTIES OF MERCHANTABILITY OR FITNESS FOR A PARTICULAR PURPOSE. IN NO   *
 * EVENT WILL IBM OR ITS LICENSORS OR SUPPLIERS BE LIABLE FOR ANY DAMAGES ARISING OUT *
 * OF THE USE OF OR INABILITY TO USE THE SAMPLE CODE, DISTRIBUTION OF THE SAMPLE CODE,*
 * OR COMBINATION OF THE SAMPLE CODE WITH ANY OTHER CODE. IN NO EVENT SHALL IBM OR ITS*
 * LICENSORS AND SUPPLIERS BE LIABLE FOR ANY LOST REVENUE, LOST PROFITS OR DATA, OR   *
 * FOR DIRECT, INDIRECT, SPECIAL, CONSEQUENTIAL,INCIDENTAL OR PUNITIVE DAMAGES,       *
 * HOWEVER CAUSED AND REGARDLESS OF THE THEORY OF LIABILITY,-, EVEN IF IBM OR ITS     *
 * LICENSORS OR SUPPLIERS HAVE BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGES.       *
 *                                                                                    *
 * @endCopyright                                                                      *
 **************************************************************************************

Extension of SResourceItem3rdPartyIngestICM Sample
    This Sample is an extension of the documentation of the 3rd Party Ingest
    Sample.  This Sample will explain concepts that pertain specifically to 
    Items with Video Resource Content.  Please refer to SResourceITem3rdPartyIngestICM
    for all information on importing from a 3rd party source that is not
    specific to Video Resource Items.
    
Creating Video Items
    Video Items extend standard resource functionality.  They include all 
    functionality covered in DKLobICM as shown in SResourceItemCreationICM.  
    Content may be leaded from the local file system, by stream, from memory,
    and from 3rd party sources.  Video Items extend this the functionality 
    further to new functionality in the DKStreamICM & DKVideoStreamICM objects.
    The class functionality extension is shown below.
    
        DKLobICM -> DKStreamICM -> DKVideoStreamICM
        
    This particular sample will use 3rd Party Ingest to Create Video Items.
    
    Additional Requirements
        Video Charger must installed and properly configured with Resource Manager.
        
    Additional Options & Properties
        Additional options & properties that apply to Stream & Video Stream objects.
        Please refer to the Application Programming Reference (javadoc) for complete
        information on DKStreamICM & DKVideoStreamICM.  A few of these properties are
        explained below:
        
        Stream Group (Optional)
            Group that a video or stream will belong to within the Video Charger.
            If not specified, the Video Charger's default group will be used.

********************************************************************/

// Includes / Imports
#include <iostream.h>
#include <DKConstant.h>
#include <DKDatastoreICM.hpp>
#include <DKVideoStreamICM.hpp>
#include "SConnectDisconnectICM.hpp"
#include "SVideoItemCreateICM.hpp"

/************************************************************************************************
 *          FILENAME: SVideoItemCreateICM.cpp
 *                    ---------------------------------------------------------------------------
 *       DESCRIPTION: Creating Items with Video Resource Content.
 *                    ---------------------------------------------------------------------------
 *     DEMONSTRATION: Creating Items with Video Resource Content.
 *                    Storing Videos from a 3rd-Pary Source.
 *                    ---------------------------------------------------------------------------
 * COMMANDLINE USAGE  SVideoItemCreateICM  <3rd Party Host Name>
 *                                         <3rd Party Port>
 *                                         <3rd Party Protocol (http|ftp|file)>
 *                                         <3rd Party User Name>
 *                                         <3rd Party Password>
 *                                         <3rd Party Fully-Qualified File Name>
 *                                         <3rd Party File Path>
 *                                         <3rd Party MIME Type>
 *                                         <3rd Party Content Length>
 *                                         <database> <userName> <password>
 *
 *                    NOTE:  All 3rd Party Information is Required
 *                        
 *                    NOTE:  Database, UserName, & Password are optional.  If all not specified
 *                           defaults will be used.
 *                           
 *                    Ex: SVideoItemCreateICM myFtpHostName 21 ftp myUserName myPassword SVideoItemICM_Video1.mpg // video/mpeg 2791428
 *                    Ex: SVideoItemCreateICM myFtpHostName 21 ftp myUserName myPassword SVideoItemICM_Video2.mpg // video/mpeg 1882116 icmnlsdb icmadmin password
 *                    ---------------------------------------------------------------------------
 *      PREREQUISITS: The Sample Video Resource Item Type(s) must already be defined.  If not,
 *                    please run the following sample:
 *                        - SVideoItemTypeCreateICM
 *                    The file specified in the commandline parameter must exist on a 3rd Party
 *                    host.  A recommended solution is to setup an FTP server, placing the 
 *                    provided videos in the root directory:
 *                        - SVideoItemICM_Video1.mpg
 *                        - SVideoItemICM_Video2.mpg
 *                    ---------------------------------------------------------------------------
 *    FILES REQUIRED: SConnectDisconnectICM.hpp/cpp
 *                    SVideoItemCreationICM_launch.cpp
 *                    SVideoItemCreationICM.hpp
 *                    SVideoItemICM_Video1.mpg -> On 3rd Party Host
 *                    AND/OR SVideoItemICM_Video2.mpg -> On 3rd Party Host
 *                    AND/OR Other Video -> On 3rd Party Host
 ************************************************************************************************/

    //-------------------------------------------------------------
    // Main
    //-------------------------------------------------------------
    /**
     * Run the Sample.
     * @param argc   Argument Count / Number of Arguments Specified
     * @param argv[] String Array containing arguments.  Optional arguments are <databse> <userName> <password>.
     * @return  Returns SConnectDisconnectICM_RC_SAMPLE_PASSED (0) if no sample program completed, SConnectDisconnectICM_RC_SAMPLE_FAILED (-1) if any errors / exceptions were caught.
     **/
    int SVideoItemCreateICM::main(int argc, char *argv[]){

        // Defaults for connecting to the database.
        char * database = SConnectDisconnectICM_DEFAULT_DATABASE;
        char * userName = SConnectDisconnectICM_DEFAULT_USERNAME;
        char * password = SConnectDisconnectICM_DEFAULT_PASSWORD;

        // 3rd Party Information
        char * thirdPartyHostName       = "NOT_SPECIFIED";    // Please refer to the header documentation for descriptions.
        int    thirdPartyPort           = -1;
        char * thirdPartyProtocol       = "NOT_SPECIFIED";
        char * thirdPartyUserName       = "NOT_SPECIFIED";
        char * thirdPartyPassword       = "NOT_SPECIFIED";
        char * thirdPartyFullFileName   = "NOT_SPECIFIED";     
        char * thirdPartyFilePath       = "NOT_SPECIFIED";
        char * thirdPartyMIMEType       = "NOT_SPECIFIED";
        int    thirdPartyContentLength  = -1;

        //------------------------------------------------------------
        // Checking for input parameters
        //--------------------------------------------------------------
        if (argc < 13) { // if not all 12 arguments were specified, report usage.
            cout << "Usage: "                                                        << endl;
            cout << "  SVideoItemCreateICM  <3rd Party Host Name>"                   << endl;
            cout << "                       <3rd Party Port>"                        << endl;
            cout << "                       <3rd Party Protocol (http|ftp|file)>"    << endl;
            cout << "                       <3rd Party User Name>"                   << endl;
            cout << "                       <3rd Party Password>"                    << endl;
            cout << "                       <3rd Party Fully-Qualified File Name>"   << endl;
            cout << "                       <3rd Party File Path>"                   << endl;
            cout << "                       <3rd Party MIME TYpe>"                   << endl;
            cout << "                       <3rd Party Content Length>"              << endl;
            cout << "                       <database> <userName> <password>"        << endl;
            cout << endl;
            cout << "  NOTE:  All 3rd Party Information is Required."                << endl;
            cout << endl;
            cout << "  NOTE:  Database, UserName, & Password are optional.  If all not specified" << endl;
            cout << "         defaults will be used."                                             << endl;
            cout << endl;
            cout << "  Ex: SVideoItemCreateICM myFtpHostName 21 ftp myUserName myPassword SVideoItemICM_Video1.mpg // video/mpeg 2791428"                            << endl;
            cout << "  Ex: SVideoItemCreateICM myFtpHostName 21 ftp myUserName myPassword SVideoItemICM_Video2.mpg // video/mpeg 1882116 icmnlsdb icmadmin password" << endl;
            cout << endl;
        }
        if(argc < 10){ // If any 3rd Party info not specified, terminate
            cout << "===================================================" << endl;
            cout << " TERMINATED:  All 3rd Party info must be specified." << endl;
            cout << "===================================================" << endl;
            return(SConnectDisconnectICM_RC_SAMPLE_FAILED);  
        } else if(argc < 13) {
            cout << "*** Some parameters not specified.  Using defaults for rest. ***" << endl;
        }
        // Set Command-line options
        if (argc > 1)  thirdPartyHostName      = argv[1];
        if (argc > 2)  thirdPartyPort          = atoi(argv[2]);
        if (argc > 3)  thirdPartyProtocol      = argv[3];
        if (argc > 4)  thirdPartyUserName      = argv[4];
        if (argc > 5)  thirdPartyPassword      = argv[5];
        if (argc > 6)  thirdPartyFullFileName  = argv[6];
        if (argc > 7)  thirdPartyFilePath      = argv[7];
        if (argc > 8)  thirdPartyMIMEType      = argv[8];
        if (argc > 9)  thirdPartyContentLength = atoi(argv[9]);
        if (argc > 10) database                = argv[10];
        if (argc > 11) userName                = argv[11];
        if (argc > 12) password                = argv[12];
        
        char * ver = SConnectDisconnectICM_EIP_VERSION;
        
        cout << "==========================================="     << endl;
        cout << "IBM Information Integrator for Content v" << ver << endl;
        cout << "Sample Program:  SVideoItemCreateICM"            << endl;
        cout << "-------------------------------------------"     << endl;
        cout << " Content Manager:"                               << endl;
        cout << "       Database:  " << database                  << endl;
        cout << "       UserName:  " << userName                  << endl;
        cout << " 3rd Party:"                                     << endl;
        cout << "      Host Name:  " << thirdPartyHostName        << endl;
        cout << "           Port:  " << thirdPartyPort            << endl;
        cout << "       Protocol:  " << thirdPartyProtocol        << endl;
        cout << "       UserName:  " << thirdPartyUserName        << endl;
        cout << "       Password:  " << thirdPartyPassword        << endl;
        cout << "           File:  " << thirdPartyFullFileName    << endl;
        cout << "      Mime Type:  " << thirdPartyMIMEType        << endl;
        cout << " Content Length:  " << thirdPartyContentLength   << endl;
        cout << "==========================================="     << endl;

        try{
            //-------------------------------------------------------------
            // Connect to datastore
            //-------------------------------------------------------------
            // See Sample SConnectDisconnectICM for more information
            cout << "Connecting to datastore (Database '" << database << "', UserName '" << userName << "')..." << endl;

                DKDatastoreICM* dsICM = new DKDatastoreICM();  // Create new datastore object.
                dsICM->connect(database,userName,password,""); // Connect to the datastore.

            cout << "Connected to datastore (Database '" << dsICM->datastoreName() << "', UserName '" << dsICM->userName() << "')." << endl;

            //-------------------------------------------------------------
            // Create Empty Video Resource Items
            //-------------------------------------------------------------
            // Both Video Stream and the class that it exends, Stream, are 
            // usefull classes that the new DDO may be cast to.
            cout << "Creating Empty Video Resource Items..." << endl;
    
                DKVideoStreamICM* video1 = (DKVideoStreamICM*)dsICM->createDDO("S_videoType", DK_CM_ITEM); 
                DKVideoStreamICM* video2 = (DKVideoStreamICM*)dsICM->createDDO("S_videoType", DK_CM_ITEM); 

            cout << "Created Empty Video Resource Items." << endl;

            //-------------------------------------------------------------
            // Set the User-Defined Attributes
            //-------------------------------------------------------------
            cout << "Setting User-Defined Attributes..." << endl;

                video1->setData(video1->dataId(DK_CM_NAMESPACE_ATTR,"S_author"), DKString("IBM")); 
                video1->setData(video1->dataId(DK_CM_NAMESPACE_ATTR,"S_length"), DKString("48Hours")); 

                video2->setData(video1->dataId(DK_CM_NAMESPACE_ATTR,"S_author"), DKString("IBM")); 
                video2->setData(video1->dataId(DK_CM_NAMESPACE_ATTR,"S_length"), DKString("48Hours")); 

            cout << "Set User-Defined Attributes." << endl;

            //-------------------------------------------------------------
            // Setting the MIME Type (type of content)
            //-------------------------------------------------------------
            // mimeType: The type of content, such as "image/mpg"
            //
            // For a list of common MIME Types, please refer to the 
            // header documentation and SResourceItemMimeTypesICM.txt
			// located in the <CMBROOT>/Samples/java/icm directory.
            cout << "Setting MIME Type / Type of Content..." << endl;

                video1->setMimeType(thirdPartyMIMEType);
                video2->setMimeType(thirdPartyMIMEType);

            cout << "Set MIME Type / Type of Content." << endl;

            //-------------------------------------------------------------
            // Set 3rd Party Server Info
            //-------------------------------------------------------------
            // When the resource content is to be obtained from a 3rd party,
            // the 3rd party information may be specified as shown below.   
            // Please refer to the Application Programming Reference
            // for more informaiton.
            cout << "Setting 3rd Party Server Info..." << endl;

                // Video 1
                video1->setThirdPartyHostname(thirdPartyHostName);
                video1->setThirdPartyPort(thirdPartyPort);
                video1->setInputProtocol(thirdPartyProtocol);
                video1->setThirdPartyUserid(thirdPartyUserName);
                video1->setThirdPartyPassword(thirdPartyPassword);
                video1->setInputFilename(thirdPartyFullFileName);
                video1->setInputFilepath(thirdPartyFilePath);
                video1->setInputContentlength(thirdPartyContentLength);

                // Video 2
                video2->setThirdPartyHostname(thirdPartyHostName);
                video2->setThirdPartyPort(thirdPartyPort);
                video2->setInputProtocol(thirdPartyProtocol);
                video2->setThirdPartyUserid(thirdPartyUserName);
                video2->setThirdPartyPassword(thirdPartyPassword);
                video2->setInputFilename(thirdPartyFullFileName);
                video2->setInputFilepath(thirdPartyFilePath);
                video2->setInputContentlength(thirdPartyContentLength);

            cout << "Set 3rd Party Server Info." << endl;

            //-------------------------------------------------------------
            // Synchronous Store
            //-------------------------------------------------------------
            cout << "Storing Synchronously..." << endl;

                video1->addFrom();  // Add from the 3rd Party specified above.
                
                // OR  the following command may be used if all 3rd Party info were not
                //     specified above.
                
                // video1->addFrom(thirdPartyHostName, thirdPartyUserName, thirdPartyPassword, thirdPartyProtocol, thirdPartyPort, thirdPartyFullFileName);

            cout << "Stored Synchronously." << endl;

            //-------------------------------------------------------------
            // Asynchronous Store
            //-------------------------------------------------------------
            cout << "Storing Asynchronously..." << endl;

                // REQUIREMENT:  In order for Asynchronous operations to complete,
                // the Resource Manager must be able to talk to the Library Server
                // independently, after this operation has completed.  In v8.2
                // this requires that both Library Server be cataloged on the
                // Resource Manager and/or be on the same machine.  If your system
                // meets these conditions, you may uncomment out the line below.

                //video2->addFromAsync();  // Add from the 3rd Party specified above.

                // OR  the following command may be used if all 3rd Party info were not
                //     specified above.

                // video2->addFromAsync(thirdPartyJostName, thirdPartyUserName, thirdPartyPassword, thirdPartyProtocol, thirdPartyPort, thirdPartyFullFileName);

                delete(video1);   // Free Memory
                delete(video2);

            cout << "Stored Asynchronously." << endl;

            //-------------------------------------------------------------
            // Disconnect from datastore & Destroy Reference
            //-------------------------------------------------------------
            // See Sample SConnectDisconnectICM for more information
            cout << "Disconnecting from datastore & destroying reference..." << endl;            

                dsICM->disconnect();
                delete(dsICM);

            cout << "Disconnected from datastore & destroying reference." << endl;
    
            //-------------------------------------------------------------
            // Sample program completed without exception
            //-------------------------------------------------------------
            cout << "\n==========================================" << endl;
            cout << "Sample program completed."                    << endl;
            cout << "==========================================\n" << endl;
        }
        //------------------------------------------------------------
        // Catch & Print Exceptions        
        //------------------------------------------------------------
        catch (DKException &exc){
            SConnectDisconnectICM::printException(exc);      // Print the exception to the screen
            return(SConnectDisconnectICM_RC_SAMPLE_FAILED);  // Report using return code that it failed
        }
        return(SConnectDisconnectICM_RC_SAMPLE_PASSED);      // If no exceptions were thrown, return a code to indicate passed.
    }// end main

    //=================================================================
    // Wrapper Functions
    //=================================================================
    // The following are wrapper functions for functionality covered
    // in this sample.  These functions may be used by other samples.

    // No additional wrapper functions from this sample.
