/*BEGINPROLOGUE************************************************************************
 * @copyright(disclaimer)                                                             *
 * Licensed Materials - Property of IBM                                               *
 * IBM Enterprise Information Portal for Multiplatforms V8.1 (program number 5724-B43)*
 * (c ) Copyright IBM Corp. 1998,  2002. All Rights Reserved.                         *
 *                                                                                    *
 * US Government Users Restricted Rights                                              *
 * Use, duplication or disclosure restricted by GSA ADP Schedule                      *
 * Contract with IBM Corp.                                                            *
 *                                                                                    *
 * DISCLAIMER OF WARRANTIES :                                                         *
 *                                                                                    *
 * Permission is granted to copy and modify this Sample code, and to                  *
 * distribute modified versions provided that both the copyright                      *
 * notice,- and this permission notice and warranty disclaimer appear in all copies   * 
 * and modified versions.                                                             *
 *                                                                                    *
 * THIS SAMPLE CODE IS LICENSED TO YOU AS-IS.                                         *
 * IBM AND ITS SUPPLIERS AND LICENSORS DISCLAIM ALL WARRANTIES, EITHER EXPRESS OR     *
 * IMPLIED, IN SUCH SAMPLE CODE, INCLUDING THE WARRANTY OF NON-INFRINGEMENT AND THE   *
 * IMPLIED WARRANTIES OF MERCHANTABILITY OR FITNESS FOR A PARTICULAR PURPOSE. IN NO   *
 * EVENT WILL IBM OR ITS LICENSORS OR SUPPLIERS BE LIABLE FOR ANY DAMAGES ARISING OUT *
 * OF THE USE OF OR INABILITY TO USE THE SAMPLE CODE, DISTRIBUTION OF THE SAMPLE CODE,*
 * OR COMBINATION OF THE SAMPLE CODE WITH ANY OTHER CODE. IN NO EVENT SHALL IBM OR ITS*
 * LICENSORS AND SUPPLIERS BE LIABLE FOR ANY LOST REVENUE, LOST PROFITS OR DATA, OR   *
 * FOR DIRECT, INDIRECT, SPECIAL, CONSEQUENTIAL,INCIDENTAL OR PUNITIVE DAMAGES,       *
 * HOWEVER CAUSED AND REGARDLESS OF THE THEORY OF LIABILITY,-, EVEN IF IBM OR ITS     *
 * LICENSORS OR SUPPLIERS HAVE BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGES.       *
 *                                                                                    *
 * @endCopyright                                                                      *
 **************************************************************************************/
/************************************************************************/
/*
 * Filename       : TDatastorePoolDD.cpp
 * Description    : test DKDatastorePool for datastore connection pool
 *
 */
#include <iostream.h>
#include <stdio.h>
#include <DKConstant.h>
#include <DKBlobDD.hpp>
#include <DKDatastoreDD.hpp>
#include <DKDatastorePool.hpp>
#include <DKRoomDefDD.hpp>
#include <DKException.hpp>
#include <DKString.hpp>
#include <DKResults.hpp>
#include <DKDDO.hpp>
#include <DKAny.hpp>
#include <dkCollection.hpp>
#include <DKEnvironment.hpp>

void printSubEnts( DKEntityDefDD*, DKDatastoreDD&, int );

void main(int argc, char *argv[])
{
   cout << "begin..." << endl;
   // Indicate this is a Console Application on Windows NT/95 for error
   // message code page conversion to OEM code page. If this was a Windows
   // GUI (Graphic User Interface) Application this would not be necessary
   // because DK_SS_WINDOWS is the default.
   DKEnvironment::setSubSystem(DK_SS_CONSOLE);

    //-------------------------------------------------------------- 
    // Checking for input parameters
    //--------------------------------------------------------------   
    if (argc < 4)
    {
     cout<<"Usage: "<<endl;
     cout<<"  TDatastorePoolDD <libsrv> <userid> <pw>" << endl;
     return;
    }
    // ### get input parameters
    DKString libsrv = DKString(argv[1]);
    DKString userid = DKString(argv[2]);
    DKString pw = DKString(argv[3]);
    DKString factoryDDLName = "cmbddfac816d";

   try {
      DKDatastorePool* dsrDD = new DKDatastorePool("DD", factoryDDLName);
      dsrDD->setDatastoreName(libsrv);
      dsrDD->setMaxPoolSize(10);
      dsrDD->setMinPoolSize(3);
      //dsrDD->setTimeOut(1); 

      dsrDD->initConnections(userid, pw, 2);
      dsrDD->setTimeOut(1); 
      cout<<" #getMaxPoolSize =  " << dsrDD->getMaxPoolSize()<<endl;
      cout<<" #getMinPoolSize =  " << dsrDD->getMinPoolSize()<<endl;
      cout<<" #getTimeOut=  " << dsrDD->getTimeOut()<<endl;

      dkDatastore *dsDD1, *dsDD2, *dsDD3, *dsDD4;;
      dsDD1 = dsrDD->getConnection(userid, pw);  //new ds1
      cout<<" #getConnection, dsDD1(ds1) =  " << dsDD1 << endl;
      dsDD2 = dsrDD->getConnection(userid, pw);  // new ds2
      cout<<" #getConnection, dsDD2(ds2) =  " << dsDD2 << endl;

      DKDatastoreSession *dsIdDD1, *dsIdDD2, *dsIdDD3, *dsIdDD4;
      dsIdDD1 = dsrDD->getConnection(userid, pw, 0);        //new dsid1
      cout<<" #getConnection, dsIdDD1(dsid1)="<< dsIdDD1->getDatastore()
        <<", sessionId=" << dsIdDD1->getId()<<endl;
      dsIdDD2 = dsrDD->getConnection(userid, pw, 0);        //new dsid2
      cout<<" #getConnection, dsIdDD2(dsid2)="<<dsIdDD2->getDatastore()
        <<", sessionId=" << dsIdDD2->getId()<<endl;

      dsrDD->returnConnection(dsIdDD2);                     //dsid2 in _freeIdPool
      cout<<" #returnConnection, dsIdDD2(dsid2) =  " << 
        dsIdDD2->getDatastore() << ", sessionId=" << dsIdDD2->getId()<<endl;
      dsrDD->returnConnection(dsDD2);                       //ds2 in _freePool
      cout<<" #returnConnection, dsDD2(ds2) =  " << dsDD2 <<endl; 

      dsIdDD3 = dsrDD->getConnection(userid, pw, 1); //ds2 in _usedPool
      cout<<" #getConnection, dsIdDD3(ds2)=" <<
        dsIdDD3->getDatastore() << ", sessionId=" << dsIdDD3->getId()<<endl;
      dsIdDD4 = dsrDD->getConnection(userid, pw, 5); //new dsid3  
      cout<<" #getConnection, dsIdDD4(dsid3)=" <<
        dsIdDD4->getDatastore() << ", sessionId=" << dsIdDD4->getId() <<endl;


      dsIdDD4 = dsrDD->getConnection(userid, pw, 6); // new dsid4
      cout<< " #getConnection, dsIdDD4(dsid4)=" <<
        dsIdDD4->getDatastore() << ", sessionId=" << dsIdDD4->getId()<<endl;

      dsDD1 = dsrDD->getConnection(userid, pw);  // new ds3
      cout<<" #getConnection, dsDD1(ds3) =  " << dsDD1 <<endl;
      dsDD2 = dsrDD->getConnection(userid, pw);  // new ds4
      cout<<" #getConnection, dsDD2(ds4) =  " << dsDD2 <<endl;
 
      dsDD3 = dsrDD->getConnection(userid, pw);  // new ds5
      cout<< " #getConnection, dsDD3(ds5) =  " << dsDD3 <<endl;
      dsDD4 = dsrDD->getConnection(userid, pw);  // new ds6
      cout<<" #getConnection, dsDD4(ds6) =  " << dsDD4 <<endl;


      dsIdDD3 = dsrDD->getConnection(userid, pw, 9); // dsid2 
      cout<<" #getConnection, dsIdDD3(dsid2)=" <<
      dsIdDD3->getDatastore() << ", sessionId=" << dsIdDD3->getId()<<endl;


/**/
      //** will have some error results due to hayward is not a upgraded level
      //**** use dsDD2 to do listEntities
      cout<<"||**** start: use dsDD2 to do listEntities ****||"<<endl;
       dkCollection* pColl = dsDD2->listEntities();
       long nbrEnts = pColl->cardinality();
       dkIterator* itEnts = pColl-> createIterator();
       while( itEnts->more() )
       { // For each returned dkEntityDef...
         DKRoomDefDD* pEnt = (DKRoomDefDD*)itEnts->next()->value();
         cout << "Room title: " << pEnt->getName() << endl;
         cout << "  Has SubEntities: " << pEnt->hasSubEntities() << endl;

         // print subEntities (Cabinets->Binders->Documents)
         printSubEnts(pEnt, *(DKDatastoreDD*)dsDD2, 1);

         delete pEnt;
       }
       delete itEnts;
       delete pColl;
      cout<<"||**** end: use dsDD2 to do listEntities ****||"<<endl;
      //****end of use dsDD2 to do listEntities
/**/
      dsrDD->returnConnection(dsDD2);
      cout<<" #returnConnection, dsDD2(ds4) =  "<< dsDD2 <<endl;
      dsDD4 = dsrDD->getConnection(userid, pw);
      cout<<" #getConnection, dsDD4 =  "<< dsDD4 <<endl;
      cout<<"    dsDD4 == dsDD2 =  " <<(dsDD4 == dsDD2)<<endl;

    //**** you can add codes to use dsDD4 to do other thing here  
    //**** end of use dsDD4 to do other thing

      dsrDD->returnConnection(dsDD1);
      cout<<" #returnConnection, dsDD1(ds3) =  " << dsDD1 <<endl;
      dsrDD->returnConnection(dsDD3);
      cout<<" #returnConnection, dsDD3(ds5) =  " << dsDD3 <<endl;
      dsrDD->returnConnection(dsIdDD4); //dsid4 in _freeIdPool
      cout<<" #returnConnection, dsIdDD4(dsid4) =  " <<
        dsIdDD4->getDatastore() << ", sessionId=" << dsIdDD4->getId() <<endl;
     

      cout<<"**** start sleep for 120050 mini sec*******"<<endl;
#ifdef WIN32
     Sleep(120050);     
#elif AIX
     sleep(120050);
#endif    
      cout<<"**** end sleep for 120050 mini sec*******"<<endl;


      dsDD2 = (DKDatastoreDD*)dsrDD->getConnection(userid, pw);  //new ds7
      cout<<" #getConnection, dsDD2(ds7) =  " << dsDD2<<endl;
      cout<<" #call delete dsrDD..." <<endl;
      delete dsrDD;  //take a while for all 4 in usedPool to be disconnected
   }
   catch(DKException &exc)
   {
      cout << "Error id " << exc.errorId() << endl;
      for(unsigned long i = 0; i< exc.textCount();i++)
      {
         cout << "Error text: " << exc.text(i) << endl;
      }
      for (unsigned long g=0; g< exc.locationCount();g++)
      {
         const DKExceptionLocation* p = exc.locationAtIndex(g);
         cout << "Filename: " << p->fileName() << endl;
         cout << "Function: " << p->functionName() << endl;
         cout << "LineNumber: " << p->lineNumber() << endl;
      }

      cout << "Exception Class Name: " << exc.name() << endl;
   }
   
   cout << "done ..." << endl;   
}

//=============================================================================
// Print SubEntities
//=============================================================================
void printSubEnts( DKEntityDefDD* pEnt, DKDatastoreDD& domDoc, int indents )
{
  // indents: 1=Cabinets; 2=Binders; 3=Documents
  DKString indentation = "";

  for(int i = 0; i < indents; i++)
  { // make the spacing nice
    indentation += "  ";
  }

  if( pEnt->hasSubEntities() )
  {
    dkCollection* pColl = pEnt->listSubEntities();
    long nbrEnts = pColl->cardinality();
    dkIterator* itEnts = pColl-> createIterator();
    while( itEnts->more() )
    {
      DKEntityDefDD* pEnt = (DKEntityDefDD*)itEnts->next()->value();
      cout<< indentation << "SubEntity title: " << pEnt->getName() << endl;
      printSubEnts(pEnt, domDoc, indents+1);
      delete pEnt;
    }
    delete itEnts;
    delete pColl;
  }
  return;
}
