/************************************************************************/
/*BEGINPROLOGUE************************************************************************
 * @copyright(disclaimer)                                                             *
 * Licensed Materials - Property of IBM                                               *
 * IBM Enterprise Information Portal for Multiplatforms V8.1 (program number 5724-B43)*
 * (c ) Copyright IBM Corp. 1998,  2002. All Rights Reserved.                         *
 *                                                                                    *
 * US Government Users Restricted Rights                                              *
 * Use, duplication or disclosure restricted by GSA ADP Schedule                      *
 * Contract with IBM Corp.                                                            *
 *                                                                                    *
 * DISCLAIMER OF WARRANTIES :                                                         *
 *                                                                                    *
 * Permission is granted to copy and modify this Sample code, and to                  *
 * distribute modified versions provided that both the copyright                      *
 * notice,- and this permission notice and warranty disclaimer appear in all copies   * 
 * and modified versions.                                                             *
 *                                                                                    *
 * THIS SAMPLE CODE IS LICENSED TO YOU AS-IS.                                         *
 * IBM AND ITS SUPPLIERS AND LICENSORS DISCLAIM ALL WARRANTIES, EITHER EXPRESS OR     *
 * IMPLIED, IN SUCH SAMPLE CODE, INCLUDING THE WARRANTY OF NON-INFRINGEMENT AND THE   *
 * IMPLIED WARRANTIES OF MERCHANTABILITY OR FITNESS FOR A PARTICULAR PURPOSE. IN NO   *
 * EVENT WILL IBM OR ITS LICENSORS OR SUPPLIERS BE LIABLE FOR ANY DAMAGES ARISING OUT *
 * OF THE USE OF OR INABILITY TO USE THE SAMPLE CODE, DISTRIBUTION OF THE SAMPLE CODE,*
 * OR COMBINATION OF THE SAMPLE CODE WITH ANY OTHER CODE. IN NO EVENT SHALL IBM OR ITS*
 * LICENSORS AND SUPPLIERS BE LIABLE FOR ANY LOST REVENUE, LOST PROFITS OR DATA, OR   *
 * FOR DIRECT, INDIRECT, SPECIAL, CONSEQUENTIAL,INCIDENTAL OR PUNITIVE DAMAGES,       *
 * HOWEVER CAUSED AND REGARDLESS OF THE THEORY OF LIABILITY,-, EVEN IF IBM OR ITS     *
 * LICENSORS OR SUPPLIERS HAVE BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGES.       *
 *                                                                                    *
 * @endCopyright                                                                      *
 **************************************************************************************/
/************************************************************************/

/*
 * Filename  : TQueryStrPQandTQFed
 * Description : test Federated String query
 *     Sample program to run a federated query against
 *     two DL back-end datastore : LSMUFASA and LSCHIUN and TMMUF.
 *  Before running this program, you need to run the
 *  two following programs first:
 *    1) TADLServers feddb edms1 edms1 add
 *    to define servers participating in this federation
 *    2) TADLsmSample feddb edms1 edms1 add
 *    to define schema mapping to map federated entity and
 *    attributes to each back-end entity and attributes
 *    3) TADLsmTM feddb edms1 edms1 add
 *    to define schema mapping to map text federated entity
 *  Once the servers and mapping information is establish you
 *  can run this program to create and execute a query expression
 *
 */

#include <iostream.h>
#include <stdio.h>    
#include <DKConstant.h>
#include <DKString.hpp>
#include <DKAny.hpp>
#include <DKDatastoreFed.hpp>
#include <DKDatastoreDefFed.hpp>
#include <DKException.hpp>
#include <DKEnvironment.hpp>
#include <DKAttrDefFed.hpp>
#include <DKEntityDefFed.hpp>
#include <DKAttrMappingFed.hpp>
#include <DKAttrMapDef.hpp>
#include <dkResultSetCursor.hpp>
#include <DKDatastoreAdminFed.hpp>
#include <DKServerMapDefFed.hpp>
#include <DKFederatedCollection.hpp>
#include <DKParts.hpp>
#include <DKResultsFed.hpp>

void displayDDO (DKDDO* ddo);

void main(int argc, char *argv [])
{
   cout << "begin..." << endl;
   // Indicate this is a Console Application on Windows NT/95 for error
   // message code page conversion to OEM code page. If this was a Windows
   // GUI (Graphic User Interface) Application this would not be necessary
   // because DK_SS_WINDOWS is the default.
   DKEnvironment::setSubSystem(DK_SS_CONSOLE);

   //-------------------------------------------------------------- 
   // Checking for input parameters
   //-------------------------------------------------------------- 
   if (argc < 4)
   {
      cout <<"Usage syntax: "<<endl;
      cout <<"\t TQueryStrPQandTQFed <feddb> <userid> <pw> " << endl;
      return;
   }
   // ### get input parameters
   DKString libsrv(argv[1]);
   DKString userid(argv[2]);
   DKString pw (argv[3]);

   // DKDatastoreFed needs to be declared outside of the try block
   DKDatastoreFed dsFed;

   try
   {
      // All set; Connect and execute the compound query expression
      cout << " *** connecting to " << libsrv << " *** " << endl;
      dsFed.connect(libsrv, userid, pw);

      cout << " \t -----> connected " << endl;
      cout << "  user " << dsFed.userName() << ", dsName " << dsFed.datastoreName() << endl;

      // get the Def
      dkDatastoreDef*  defFed = dsFed.datastoreDef();

      // associate data sources to this entity:
      // get the Fed DS Def and Admin
      DKDatastoreAdminFed* adm = (DKDatastoreAdminFed*) defFed->datastoreAdmin();
      // list back-end servers
      cout << " *** list all back-end servers : " << endl;
      long i, count;
      DKString* servers = adm->listServerNames(count);
      for (i = 0; i < count; i++)
      {
         cout << "server [" << i << "] = " << servers[i] 
         << " Strlen = " << servers[i].length() << endl;
      }  

      // retrieve the definition of the first server
      DKServerMapDefFed* serverdat;
      DKString serverName;
      for (i = 0; i < count; i ++)
      {
         serverdat = adm->getServer(servers[i]);
         serverName = serverdat->getName();
         cout << " DKServerMapDefFed = " << serverName << endl;
         cout << "  name: " << serverName << endl;
         cout << "  type: " << serverdat->getServerType() << endl;
         cout << " initParm: " << serverdat->getConnectString() << endl;
         cout << " desc : " << serverdat->getDescription() << endl;
         cout << " refTime : " << serverdat->getRefreshTime() << endl;
         delete serverdat;
      }  
      delete [] servers;

// ================================================  

      DKCQExpr* cqe = 0;

      DKString qryString;
      qryString = "PARAMETRIC_SEARCH = (ENTITY = FedSample, MAX_RESULTS = 100, ";
      qryString += "COND = (FedAuthor <> NULL)); OPTION = (CONTENT = ATTRONLY) ";
      qryString += "AND TEXT_SEARCH=(COND=(UNIX)); OPTION=(SEARCH_INDEX=FedTMINDEX)";


      cout << "query: " << qryString << endl;

      dkResultSetCursor* rsc = dsFed.execute (qryString, DK_CM_FEDERATED_QL_TYPE, 0);

      //  get result set cursor
      DKDDO* ddo;
      if (!rsc)
         cout << " *** cursor is null" << endl;
      else
      {
         int hitCount  = rsc->cardinality();
         int afterPos = 0;
         cout << " hit count = " << hitCount << endl;

         while (rsc->isValid())
         {
            cout << "before fetch, position = " << rsc->getPosition() << endl;
            ddo = rsc->fetchNext();
            afterPos = rsc->getPosition();
            cout << "after fetch, position = " << afterPos << endl;

            if (ddo)
            {
               cout << " pidString : " << ((DKPid&)ddo->getPid()).pidString() << endl;
               displayDDO(ddo);
               delete ddo;
            }
            else break;
         }
         delete rsc;
      }

      // disconnects
      cout << "\n\n *** disconnect from the datastore Fed. ***" << endl;
      dsFed.disconnect();
      cout << " *** datastore Fed disconnected ***" << endl;

      // that is all, folks
   }
   catch (DKException &exc)
   {
      // exc.printTrace();
      cout << "Error id " << exc.errorId() << endl;
      for (unsigned long i = 0; i< exc.textCount();i++)
      {
         cout << "Error text: " << exc.text(i) << endl;
      }
      for (unsigned long g=0; g< exc.locationCount();g++)
      {
         const DKExceptionLocation* p = exc.locationAtIndex(g);
         cout << "Filename: " << p->fileName() << endl;
         cout << "Function: " << p->functionName() << endl;
         cout << "LineNumber: " << p->lineNumber() << endl;
      }

      cout << "Exception Class Name: " << exc.name() << endl;
   }

   cout << "done ..." << endl; 
}

// simple method to show DDO hit 
void displayDDO(DKDDO* ddo) 
{
   DKString dataName;
   short  cnt;
   DKAny  any;

   if (ddo)
   {
      cnt = ddo->dataCount();
      for (short i = 1; i <= cnt; i++)
      {
         any = ddo->getData(i);
         dataName = ddo->getDataName(i);
         cout << i << ": Attribute Name :" << dataName << endl;
         if (!any.isNull())
            cout << ",\t Value :" << any << endl;
         else
            cout << ",\t Value : null"  << endl;
      }  
   }
}
