/*
 * @copyright(disclaimer)
 * Licensed Materials - Property of IBM
 * IBM  Enterprise Information Portal  for Multiplatforms V8.1  (program number 5724-B43)
 * (c ) Copyright IBM Corp. 1998,  2002. All Rights Reserved.
 *
 * US Government Users Restricted Rights
 *  Use, duplication or disclosure restricted by GSA ADP Schedule
 *  Contract with IBM Corp.
 *
 * DISCLAIMER OF WARRANTIES :
 *
 * Permission is granted to copy and modify this  Sample code, and to
 * distribute modified versions provided that both the copyright
 * notice,- and this permission notice and warranty disclaimer  appear in all copies and modified
 * versions.
 *
 * THIS SAMPLE CODE IS LICENSED TO YOU AS-IS.
 * IBM  AND ITS SUPPLIERS AND LICENSORS  DISCLAIM
 * ALL WARRANTIES, EITHER EXPRESS OR IMPLIED, IN
 * SUCH SAMPLE CODE, INCLUDING THE WARRANTY OF
 * NON-INFRINGEMENT AND THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY OR FITNESS FOR A PARTICULAR
 * PURPOSE. IN NO EVENT WILL IBM OR ITS LICENSORS OR
 * SUPPLIERS BE LIABLE FOR ANY DAMAGES ARISING OUT
 * OF THE USE OF  OR INABILITY TO USE THE SAMPLE CODE,
 * DISTRIBUTION OF THE SAMPLE CODE, OR COMBINATION
 * OF THE SAMPLE CODE WITH ANY OTHER CODE. IN NO
 * EVENT SHALL IBM OR ITS LICENSORS AND SUPPLIERS BE
 * LIABLE FOR ANY LOST REVENUE, LOST PROFITS OR DATA,
 * OR FOR DIRECT, INDIRECT, SPECIAL, CONSEQUENTIAL,
 * INCIDENTAL OR PUNITIVE DAMAGES, HOWEVER CAUSED
 * AND REGARDLESS OF THE THEORY OF LIABILITY,-, EVEN IF
 * IBM OR ITS LICENSORS OR SUPPLIERS HAVE BEEN ADVISED
 * OF THE POSSIBILITY OF SUCH DAMAGES.
 *
 * @endCopyright
 */
 import java.awt.*;
 import java.awt.event.*;
 import java.util.*;
 import java.text.*;
 import javax.swing.*;
 import javax.swing.event.*;
 import javax.swing.text.*;
 import javax.swing.border.*;
 import com.ibm.mm.beans.gui.*;
 import com.ibm.mm.beans.*;


public class OptionsDialog extends JDialog implements ActionListener, ItemListener
{

  // Do not remove - needed for copyright protection
  private static String COPYRIGHT = ClientPane.IBM_LONG_COPYRIGHT;
  public final static String DEFAULT_MAXRESULTS_STRING="";
  public final static int DEFAULT_MAXRESULTS_INT=-1;

  private JButton okButton=null;
  private JButton cancelButton=null;
  private JButton defaultsButton=null;
  private JButton helpButton=null;

  private JLabel sortingOptionsPanelLabel=null;
  private JLabel SRVOptionsPanelLabel=null;
  private JLabel STVOptionsPanelLabel=null;

  // STL style
  private JRadioButton listStyleRadioButton = new JRadioButton();
  private JRadioButton imageListStyleRadioButton = new JRadioButton();
  private JRadioButton dropDownStyleRadioButton = new JRadioButton();
  private JLabel unselectedImageLabel = new JLabel();
  private JLabel selectedImageLabel = new JLabel();
  private JTextField selectedImageURLTextField = new JTextField();
  private JTextField unselectedImageURLTextField = new JTextField();
  private JButton selectedImageChooserButton = new JButton();
  private JButton unselectedImageChooserButton = new JButton();

  private JRadioButton systemDefaultsRadioButton=null;
  private JRadioButton lastTimeRadioButton=null;
  private JRadioButton previousViewRadioButton=null;

  private JRadioButton clearResultsRadioButton=null;
  private JRadioButton saveResultsRadioButton=null;

  // SRV max results
  private JLabel maxResultsLabel1 = null;
  private JLabel maxResultsLabel2 = null;
  private JTextField maxResultsTextField = null;

  // STV Wild card
  private JLabel parmFieldLabel = null;
  private JTextField parmFieldTextField = null;
  private JLabel textSingleCharLabel = null;
  private JTextField textSingleCharTextField = null;
  private JLabel textMultiCharsLabel = null;
  private JTextField textMultiCharsTextField = null;

  // STV Buttons visible
  private JCheckBox searchButtonsVisibleCheckBox = null;

  // SRV Grid visible
  private JCheckBox horizontalLinesVisibleCheckBox = new JCheckBox();
  private JCheckBox verticalLinesVisibleCheckBox = new JCheckBox();

  // SRV Line contrast
  private JCheckBox lineContrastCheckBox = new JCheckBox();
  private JLabel lineColorLabel = new JLabel();
  private JTextField lineColorTextField = new JTextField();
  private JButton colorChooserButton = new JButton();
    // get& set chosen color
    private void setChosenColor (Color c) {
        lineColorTextField.setBackground(c);
    }
    private Color getChosenColor () {
        return lineColorTextField.getBackground();
    }
  // SRV show tree pane
  private JCheckBox showTreePaneCheckBox = new JCheckBox();

  // L&F buttons
  private JRadioButton windowsLNFRadioButton = new JRadioButton();
  private JRadioButton motifLNFRadioButton = new JRadioButton();
  private JRadioButton metalLNFRadioButton = new JRadioButton();



  private String[] sortOptions=null;
  private  int STVOption;
  private  boolean SRVOption;
  private JButton SRVOptionButton=null;
  private JButton STVOptionButton=null;

  // Beans
  private CMBSearchTemplateList searchTemplateList = null;
  private CMBSearchTemplateViewer searchTemplateViewer=null;
  private CMBSearchPanel searchPanel = null;
  private CMBSearchResultsViewer searchResultsViewer= null;

  // Panels
  private transient JTabbedPane tabbedPane = null;
  private transient JPanel optionsPanel=null;
  private transient JPanel stlPanel = null;
  private transient JPanel stvPanel = null;
  private transient JPanel srvPanel = null;
  private transient JPanel looknfeelPanel = null;

  // Ref to parent app pane
  private ClientPane clientPane= null;

  // For BiDi calculations
  private boolean ltr = ClientPane.isLeftToRight();
  // Are templates supported or not?
  private boolean templatesSupported = false;

public OptionsDialog(ClientPane aClientPane)
{
    super(aClientPane.getAppFrame());
    // Get bean instances from the client pane
    clientPane=aClientPane;
    searchTemplateList      = clientPane.getSearchTemplateListBean();
    searchTemplateViewer    = clientPane.getSearchTemplateViewerBean();
    searchPanel             = clientPane.getSearchPanelBean();
    searchResultsViewer     = clientPane.getSearchResultsViewerBean();
    CMBConnection connection = clientPane.getConnectionBean();

    // See if templates are supported for this datastore
    // only fed and OD support for now
    templatesSupported = clientPane.isTemplatesSupported();
    Frame f = aClientPane.getAppFrame();
    if (f != null) {
        f.setIconImage(this.getImage("CC_SPLASH"));
    }
    setModal(true);
    setTitle(ResourceMgr.getString("OPTIONS"));
    // Create a new JTabbed Pane
    getContentPane().setLayout(new BorderLayout());
    tabbedPane = new JTabbedPane(JTabbedPane.TOP);
    getContentPane().add("Center", tabbedPane);
    //tabbedPane.add("Application Options", getOptionsPanel());
    if (templatesSupported) {
    tabbedPane.add("SearchTemplateList", getStlOptionsPanel());
    tabbedPane.add("SearchTemplateViewer", getStvOptionsPanel());
    }
    tabbedPane.add("SearchResultsViewer", getSrvOptionsPanel());
    tabbedPane.add("Look & Feel", getLookNFeelOptionsPanel());
    okButton = new JButton( ResourceMgr.getString("OK"));
    okButton.addActionListener( this);
    cancelButton = new JButton( ResourceMgr.getString("CANCEL"));
    cancelButton.addActionListener( this);
    defaultsButton = new JButton( ResourceMgr.getString("DEFAULTS"));
    defaultsButton.addActionListener(this);
    helpButton = new JButton( ResourceMgr.getString("HELP"));
    helpButton.addActionListener( this);
    SRVOptionsPanelLabel = new JLabel();
    SRVOptionsPanelLabel.setText( ResourceMgr.getString( "SEARCH_RESULTS_OPTIONS_PANEL"));
    sortingOptionsPanelLabel = new JLabel();
    sortingOptionsPanelLabel.setText( ResourceMgr.getString("SORTING_OPTIONS_PANEL"));


    JPanel buttonPanel = new JPanel();
    if (ltr) {
        buttonPanel.setLayout( new FlowLayout(FlowLayout.RIGHT));
        buttonPanel.add(okButton);
        buttonPanel.add(cancelButton);
        buttonPanel.add(defaultsButton);
    } else {
        buttonPanel.setLayout( new FlowLayout(FlowLayout.LEFT));
        buttonPanel.add(defaultsButton);
        buttonPanel.add(cancelButton);
        buttonPanel.add(okButton);
    }
    getContentPane().add(buttonPanel,BorderLayout.SOUTH);
    okButton.setEnabled( true);
    cancelButton.setEnabled(true);
    helpButton.setEnabled(true);
    if ( isDefaultsChosen() ) {
        defaultsButton.setEnabled(false);
    }
    else {
        defaultsButton.setEnabled(true);
    }

	}

	public void addNotify() {
	    super.addNotify();
        setBounds(300, 150, Math.max(getPreferredSize().width+40, 500),Math.max(getPreferredSize().height+40, 250));
        // adjust preferred sizes so they line up in their BoxLayouts
        Dimension ps;
        if (templatesSupported) {
        // STL
        int maxWidth = selectedImageLabel.getPreferredSize().width;
        int newWidth = unselectedImageLabel.getPreferredSize().width;
        if (maxWidth < newWidth)
            maxWidth = newWidth;
        ps = new Dimension(maxWidth, selectedImageLabel.getPreferredSize().height);
        selectedImageLabel.setPreferredSize(ps);
        unselectedImageLabel.setPreferredSize(ps);
        // STV
        maxWidth = parmFieldLabel.getPreferredSize().width;
        newWidth = textSingleCharLabel.getPreferredSize().width;
        if (maxWidth < newWidth) {
            maxWidth = newWidth;
        }
        newWidth = textMultiCharsLabel.getPreferredSize().width;
        if (maxWidth < newWidth) {
            maxWidth = newWidth;
        }
        ps = new Dimension(maxWidth, parmFieldLabel.getPreferredSize().height);
        parmFieldLabel.setPreferredSize(ps);
        textSingleCharLabel.setPreferredSize(ps);
        textMultiCharsLabel.setPreferredSize(ps);
        }
        // SRV line color text field -want it to be a small square
        ps  = lineColorTextField.getPreferredSize();
        lineColorTextField.setPreferredSize(new Dimension(ps.height, ps.height));
        lineColorTextField.setMaximumSize(new Dimension(ps.height, ps.height));

	}



/**
 * Handles action events from the push buttons.
 */
   public void actionPerformed( ActionEvent e)
   {
      if ( e.getSource() == maxResultsTextField) {
       checkTextValue();
       return;
      }

      if ( e.getSource() == okButton)
      {
         checkTextValue();
         setBeanProperties();
         dispose();
         return;
      }

      if ( e.getSource() == cancelButton) {
          cancelChanges();
          dispose();
          return;
      }

      if (e.getSource() == defaultsButton) {
         setBeanDefaultProperties();
         defaultsButton.setEnabled(false);
         return;
      }

      if ( e.getSource() == helpButton) {
          // help();
          return;
       }
       return;
   }

   /*
    * set all attributes on the options panel to bean default properties
    **/
  private void setBeanDefaultProperties()
  {
    if (templatesSupported) {
        // STL
        imageListStyleRadioButton.setSelected(true);

        switch (searchTemplateViewer.getDefaultsOption()) {
            case CMBSearchTemplateViewer.SYSTEM:
                systemDefaultsRadioButton.setSelected(true);
                break;
            case CMBSearchTemplateViewer.LAST_TIME:
                lastTimeRadioButton.setSelected(true);
                break;
            case CMBSearchTemplateViewer.PREVIOUS_VIEW:
                previousViewRadioButton.setSelected(true);
                break;
            default:
                lastTimeRadioButton.setSelected(true);
            break;
        } // end switch
        // Set wildcards
        parmFieldTextField.setText("");
        textSingleCharTextField.setText("");
        textMultiCharsTextField.setText("");

     // set STV buttons visible
     searchButtonsVisibleCheckBox.setSelected(true);
    }

     clearResultsRadioButton.setSelected(true);
     saveResultsRadioButton.setSelected(false);
     maxResultsTextField.setText(DEFAULT_MAXRESULTS_STRING);
     // set grid options
     horizontalLinesVisibleCheckBox.setSelected(false);
     verticalLinesVisibleCheckBox.setSelected(false);
     // set line contrast
     lineContrastCheckBox.setSelected(true);
     //searchResultsViewer.setLineContrastColor(getChosenColor());
     // show tree pane
     showTreePaneCheckBox.setSelected(true);

  }



     /*
     * set bean properties
     **/
   private void setBeanProperties()
   {
      int defaultOption;

      if (templatesSupported) {
        // Search Template list
        if (imageListStyleRadioButton.isSelected()) {
            searchTemplateList.setStyle(CMBSearchTemplateList.IMAGE);
        } else if (listStyleRadioButton.isSelected()) {
            searchTemplateList.setStyle(CMBSearchTemplateList.SIMPLE);
        } else {
            searchTemplateList.setStyle(CMBSearchTemplateList.DROP_DOWN);
        }

        // set search template viewer option
        if (systemDefaultsRadioButton.isSelected() ) {
            defaultOption=CMBSearchTemplateViewer.SYSTEM;
        }
        else if ( lastTimeRadioButton.isSelected() ) {
            defaultOption=CMBSearchTemplateViewer.LAST_TIME;
        }
        else  {
            defaultOption=CMBSearchTemplateViewer.PREVIOUS_VIEW;
        }
        searchTemplateViewer.setDefaultsOption(defaultOption);
        // Buttons visible check box
        if (searchButtonsVisibleCheckBox.isSelected()) {
            searchTemplateViewer.setStartSearchButtonVisible(true);
            searchTemplateViewer.setStopSearchButtonVisible(true);
        }
        else {
            searchTemplateViewer.setStartSearchButtonVisible(false);
            searchTemplateViewer.setStopSearchButtonVisible(false);
        }
        // Set wildcards
        String wc = parmFieldTextField.getText();
        if (wc != null && wc.trim().length() > 0) {
            searchTemplateViewer.setParmFieldWildcard(wc.charAt(0));
        } else {
            searchTemplateViewer.setParmFieldWildcard(' ');
        }
        wc = textSingleCharTextField.getText();
        if (wc != null && wc.trim().length() > 0) {
            searchTemplateViewer.setTextSingleCharWildcard(wc.charAt(0));
        } else {
            searchTemplateViewer.setTextSingleCharWildcard(' ');
        }
        wc = textMultiCharsTextField.getText();
        if (wc != null && wc.trim().length() > 0) {
            searchTemplateViewer.setTextMultiCharsWildcard(wc.charAt(0));
        } else {
            searchTemplateViewer.setTextMultiCharsWildcard(' ');
        }
      }
      String currentMaxTextValue=maxResultsTextField.getText();
      int intMaxValue= DEFAULT_MAXRESULTS_INT; // -1
      currentMaxTextValue=currentMaxTextValue.trim();
      if (currentMaxTextValue != null && !currentMaxTextValue.equals(DEFAULT_MAXRESULTS_STRING)) {
          intMaxValue=new Integer(currentMaxTextValue).intValue();
      }
      if (intMaxValue < DEFAULT_MAXRESULTS_INT ) { // less than -1
          intMaxValue=DEFAULT_MAXRESULTS_INT;
      }
      if (templatesSupported) {
            searchTemplateViewer.setMaxResults(intMaxValue);
      } else {
            searchPanel.setMaxResults(intMaxValue);
      }

      // set search results viewer option
      if (clearResultsRadioButton.isSelected()) {
         searchResultsViewer.setSaveResultsPerTemplate(false);
      }
      else  {
         searchResultsViewer.setSaveResultsPerTemplate(true);
      }
      // set grid options
      searchResultsViewer.setDetailsHorizontalLinesVisible(horizontalLinesVisibleCheckBox.isSelected());
      searchResultsViewer.setDetailsVerticalLinesVisible(verticalLinesVisibleCheckBox.isSelected());
      // set line contrast
      searchResultsViewer.setTableLineContrast(lineContrastCheckBox.isSelected());
      searchResultsViewer.setLineContrastColor(getChosenColor());
      // set tree pane visible
      searchResultsViewer.setTreePaneVisible(showTreePaneCheckBox.isSelected());

}


/**
 * Gets the OptionsPanel property value.
 * If the panel is null then it is defined by creating a panel with
 * options.
 * "OK" ,"CANCEL" "Defaults", and "Help" push buttons.
 *
 * @return The optionsPanel property value.
 */
 /*  public JPanel getOptionsPanel()
   {
      if (optionsPanel == null)
      {
         try
         {
            optionsPanel = new JPanel();

            optionsPanel.setLayout( new BorderLayout() );

            int anchor = GridBagConstraints.WEST;
            if (!ltr) {
                anchor = GridBagConstraints.EAST;
            }

            JPanel p4 = new JPanel();
            p4.setLayout( new FlowLayout());
            p4.add( getOkButton());
            p4.add( getCancelButton());
            p4.add( getDefaultsButton());
            //p4.add( getHelpButton());
            getOkButton().setEnabled( true);
            getCancelButton().setEnabled(true);
            getHelpButton().setEnabled(true);
            if ( isDefaultsChosen() ) {
              getDefaultsButton().setEnabled(false);
            }
            else {
             getDefaultsButton().setEnabled(true);
            }
            // Set borders for the panels
            TitledBorder p1Border = BorderFactory.createTitledBorder(new EtchedBorder(), ResourceMgr.getString( "SEARCH_TEMPLATE_VIEWER_OPTIONS_PANEL"));
            p1Border.setTitlePosition(TitledBorder.BELOW_TOP);
            if (ltr) {
                p1Border.setTitleJustification(TitledBorder.LEFT);
            }
            else {
                p1Border.setTitleJustification(TitledBorder.RIGHT);
            }
            //p1.setBorder(p1Border);

            TitledBorder p2Border = BorderFactory.createTitledBorder(new EtchedBorder(), ResourceMgr.getString( "SEARCH_RESULTS_OPTIONS_PANEL"));
            p2Border.setTitlePosition(TitledBorder.BELOW_TOP);
            if (ltr) {
                p2Border.setTitleJustification(TitledBorder.LEFT);
            }
            else {
                p2Border.setTitleJustification(TitledBorder.RIGHT);
            }
            p2.setBorder(p2Border);

            p3.setBorder(BorderFactory.createEtchedBorder());
            JPanel ptop = new JPanel( new GridLayout(0,1) );
            JPanel pbottom= new JPanel( new FlowLayout());


            //ptop.add(p1);
            ptop.add(p2);
            ptop.add(p3);
            pbottom.add(p4);

            optionsPanel.add(ptop, BorderLayout.CENTER);
            optionsPanel.add(pbottom, BorderLayout.SOUTH);


         }
         catch ( java.lang.Throwable exc)
         {
            throw new CMBShouldNotOccurException( exc.toString() + ":" + exc.getMessage());
         }
      }
      return optionsPanel;

   }  */

/**
 * Gets the stlPanel property value.
 * If the panel is null then it is defined by creating a panel with
 * options.
 *
 * @return The stlPanel property value.
 */
   public JPanel getStlOptionsPanel()
   {
      if (stlPanel == null)
      {
            listStyleRadioButton.setText(ResourceMgr.getString("STL_STYLE_SIMPLE_LIST"));
            imageListStyleRadioButton.setText(ResourceMgr.getString("STL_STYLE_IMAGE_LIST"));
            dropDownStyleRadioButton.setText(ResourceMgr.getString("STL_STYLE_DROP_DOWN"));
            unselectedImageLabel.setText(ResourceMgr.getString("STL_STYLE_UNSELECTED_IMAGE_LABEL"));
            selectedImageLabel.setText(ResourceMgr.getString("STL_STYLE_SELECTED_IMAGE_LABEL"));
            unselectedImageLabel.setLabelFor(unselectedImageURLTextField);
            selectedImageLabel.setLabelFor(selectedImageURLTextField);
            selectedImageChooserButton.setText("  " + ResourceMgr.getString("ELLIPSIS") + "  ");
            unselectedImageChooserButton.setText("  " + ResourceMgr.getString("ELLIPSIS") + "  ");

            selectedImageChooserButton.setBorder(BorderFactory.createRaisedBevelBorder());
            unselectedImageChooserButton.setBorder(BorderFactory.createRaisedBevelBorder());

            // Create button group
            ButtonGroup bGrp = new ButtonGroup();
            bGrp.add(listStyleRadioButton);
            bGrp.add(imageListStyleRadioButton);
            bGrp.add(dropDownStyleRadioButton);

            stlPanel = new JPanel();
            stlPanel.setLayout(new BorderLayout());
            Box innerBox = Box.createVerticalBox();
            stlPanel.add(innerBox, BorderLayout.NORTH);
            // Search template list style
            JPanel stylePanel = new JPanel();
            stylePanel.setLayout(new BoxLayout(stylePanel, BoxLayout.Y_AXIS));
            Box box1 = Box.createHorizontalBox();
            Box box2 = Box.createHorizontalBox();
            Box box3 = Box.createHorizontalBox();
            Box box4 = Box.createHorizontalBox();
            Box box5 = Box.createHorizontalBox();
            // set layout for inner boxes
            box1.add(Box.createHorizontalStrut(5));
            box2.add(Box.createHorizontalStrut(5));
            box3.add(Box.createHorizontalStrut(5));
            box4.add(Box.createHorizontalStrut(5));
            box5.add(Box.createHorizontalStrut(5));
            if (ltr) {
                box1.add(dropDownStyleRadioButton);
                box2.add(listStyleRadioButton);
                box3.add(imageListStyleRadioButton);
                box4.add(Box.createHorizontalStrut(20));
                box4.add(unselectedImageLabel);
                box4.add(unselectedImageURLTextField);
                box4.add(unselectedImageChooserButton);
                box5.add(Box.createHorizontalStrut(20));
                box5.add(selectedImageLabel);
                box5.add(selectedImageURLTextField);
                box5.add(selectedImageChooserButton);
                box1.add(Box.createHorizontalGlue());
                box2.add(Box.createHorizontalGlue());
                box3.add(Box.createHorizontalGlue());
                box4.add(Box.createHorizontalGlue());
                box5.add(Box.createHorizontalGlue());
            }
            else {
                box1.add(Box.createHorizontalGlue());
                box2.add(Box.createHorizontalGlue());
                box3.add(Box.createHorizontalGlue());
                box4.add(Box.createHorizontalGlue());
                box5.add(Box.createHorizontalGlue());
                box1.add(dropDownStyleRadioButton);
                box2.add(listStyleRadioButton);
                box3.add(imageListStyleRadioButton);
                box4.add(unselectedImageChooserButton);
                box4.add(unselectedImageURLTextField);
                box4.add(unselectedImageLabel);
                box4.add(Box.createHorizontalStrut(20));
                box5.add(selectedImageChooserButton);
                box5.add(selectedImageURLTextField);
                box5.add(selectedImageLabel);
                box5.add(Box.createHorizontalStrut(20));
            }
            box1.add(Box.createHorizontalStrut(5));
            box2.add(Box.createHorizontalStrut(5));
            box3.add(Box.createHorizontalStrut(5));
            box4.add(Box.createHorizontalStrut(5));
            box5.add(Box.createHorizontalStrut(5));
            stylePanel.add(box1);
            stylePanel.add(box2);
            stylePanel.add(box3);
            stylePanel.add(box4);
            stylePanel.add(box5);
            // set border
            TitledBorder border1 = BorderFactory.createTitledBorder(new EtchedBorder(), ResourceMgr.getString( "STL_STYLE_BORDER_LABEL"));
            if (ltr) {
                border1.setTitleJustification(TitledBorder.LEFT);
            }
            else {
                border1.setTitleJustification(TitledBorder.RIGHT);
            }
            stylePanel.setBorder(border1);
            // set tooltip
            stylePanel.setToolTipText(ResourceMgr.getString("STL_STYLE_TOOLTIP"));

            innerBox.add(stylePanel);

            // add listeners
            listStyleListener ll = new listStyleListener();
            listStyleRadioButton.addItemListener(ll);
            imageListStyleRadioButton.addItemListener(ll);
            dropDownStyleRadioButton.addItemListener(ll);
            unselectedImageChooserButton.addActionListener(new ActionListener() {
                public void actionPerformed(ActionEvent e) {
                    String name = chooseFile(OptionsDialog.this.getParent());
                    // turn name into URL
                    name = "file:///" + name;
                    unselectedImageURLTextField.setText(name);
                }
            });
            selectedImageChooserButton.addActionListener(new ActionListener() {
                public void actionPerformed(ActionEvent e) {
                    String name = chooseFile(OptionsDialog.this.getParent());
                    // turn name into URL
                    name = "file:///" + name;
                    selectedImageURLTextField.setText(name);
                }
            });
            // initialize setting
            int style = searchTemplateList.getStyle();
            if (style == searchTemplateList.SIMPLE) {
                listStyleRadioButton.setSelected(true);
            } else if (style == searchTemplateList.DROP_DOWN) {
                dropDownStyleRadioButton.setSelected(true);
            } else {
                imageListStyleRadioButton.setSelected(true);
            }
      }
      return stlPanel;
   }

/**
 * Gets the stvPanel property value.
 * If the panel is null then it is defined by creating a panel with
 * options.
 *
 * @return The stvPanel property value.
 */
   private JPanel getStvOptionsPanel()
   {
      if (stvPanel == null)
      {
            stvPanel = new JPanel();
            stvPanel.setLayout(new BorderLayout());
            JPanel innerPanel = new JPanel();
            innerPanel.setLayout(new BoxLayout(innerPanel,BoxLayout.Y_AXIS));
            stvPanel.add(innerPanel, BorderLayout.NORTH);
            JPanel defaultsBox = new JPanel();
            defaultsBox.setLayout(new BoxLayout(defaultsBox, BoxLayout.Y_AXIS));

            ButtonGroup bgstv=new ButtonGroup();
            systemDefaultsRadioButton = new JRadioButton(ResourceMgr.getString( "STV1_OPTIONS_PANEL"));
            lastTimeRadioButton = new JRadioButton(ResourceMgr.getString( "STV2_OPTIONS_PANEL"));
            previousViewRadioButton = new JRadioButton(ResourceMgr.getString( "STV3_OPTIONS_PANEL"));
            //JLabel previousViewLabel = new JLabel(ResourceMgr.getString( "STV3_OPTIONS_PANEL_CONTINUED"));
            lastTimeRadioButton.setSelected(true);
            bgstv.add(systemDefaultsRadioButton);
            bgstv.add(lastTimeRadioButton);
            bgstv.add(previousViewRadioButton);
            // Register a listener for the radio buttons.
            RadioListener myListener = new RadioListener();
            systemDefaultsRadioButton.addActionListener(myListener);
            lastTimeRadioButton.addActionListener(myListener);
            previousViewRadioButton.addActionListener(myListener);
            Box box1 = Box.createHorizontalBox();
            Box box2 = Box.createHorizontalBox();
            Box box3 = Box.createHorizontalBox();
            //Box box4 = Box.createHorizontalBox();
            box1.add(Box.createHorizontalStrut(5));
            box2.add(Box.createHorizontalStrut(5));
            box3.add(Box.createHorizontalStrut(5));
            //box4.add(Box.createHorizontalStrut(5));
            if (ltr) {
                // add glue to right side
                box1.add(systemDefaultsRadioButton);
                box1.add(Box.createHorizontalGlue());
                box2.add(lastTimeRadioButton);
                box2.add(Box.createHorizontalGlue());
                box3.add(previousViewRadioButton);
                box3.add(Box.createHorizontalGlue());
                //box4.add(previousViewLabel);
                //box4.add(Box.createHorizontalGlue());
            }
            else {
                // reverse alignment
                systemDefaultsRadioButton.setHorizontalAlignment(SwingConstants.RIGHT);
                lastTimeRadioButton.setHorizontalAlignment(SwingConstants.RIGHT);
                previousViewRadioButton.setHorizontalAlignment(SwingConstants.RIGHT);
                systemDefaultsRadioButton.setHorizontalTextPosition(SwingConstants.LEFT);
                lastTimeRadioButton.setHorizontalTextPosition(SwingConstants.LEFT);
                previousViewRadioButton.setHorizontalTextPosition(SwingConstants.LEFT);
                // add glue to left side
                box1.add(Box.createHorizontalGlue());
                box1.add(systemDefaultsRadioButton);
                box2.add(Box.createHorizontalGlue());
                box2.add(lastTimeRadioButton);
                box3.add(Box.createHorizontalGlue());
                box3.add(previousViewRadioButton);
                //box4.add(Box.createHorizontalGlue());
                //box4.add(previousViewLabel);
            }
            box1.add(Box.createHorizontalStrut(5));
            box2.add(Box.createHorizontalStrut(5));
            box3.add(Box.createHorizontalStrut(5));
            //box4.add(Box.createHorizontalStrut(5));
            defaultsBox.add(box1);
            defaultsBox.add(box2);
            defaultsBox.add(box3);
            //defaultsBox.add(box4);

            TitledBorder p1Border = BorderFactory.createTitledBorder(new EtchedBorder(), ResourceMgr.getString( "SEARCH_TEMPLATE_VIEWER_OPTIONS_PANEL"));
            if (ltr) {
                p1Border.setTitleJustification(TitledBorder.LEFT);
            }
            else {
                p1Border.setTitleJustification(TitledBorder.RIGHT);
            }
            defaultsBox.setBorder(p1Border);

            // Wild cards
            JPanel wildCardsBox = new JPanel();
            wildCardsBox.setLayout(new BoxLayout(wildCardsBox, BoxLayout.Y_AXIS));
            parmFieldLabel = new JLabel(ResourceMgr.getString("STV_WILD_CARDS_PARM_FIELDS_LABEL"));
            parmFieldTextField = new JTextField();
            textSingleCharLabel = new JLabel(ResourceMgr.getString("STV_WILD_CARDS_TEXT_SINGLE_CHAR_LABEL"));
            textSingleCharTextField = new JTextField();
            textMultiCharsLabel = new JLabel(ResourceMgr.getString("STV_WILD_CARDS_TEXT_MULTI_CHARS_LABEL"));
            textMultiCharsTextField = new JTextField();
            // Set flyover help to describe the wildcards better
            parmFieldTextField.setToolTipText(ResourceMgr.getString("STV_WILD_CARDS_PARM_FIELDS_TOOLTIP"));
            textSingleCharTextField.setToolTipText(ResourceMgr.getString("STV_WILD_CARDS_TEXT_SINGLE_CHAR_TOOLTIP"));
            textMultiCharsTextField.setToolTipText(ResourceMgr.getString("STV_WILD_CARDS_TEXT_MULTI_CHARS_TOOLTIP"));
            parmFieldLabel.setLabelFor(parmFieldTextField);
            textSingleCharLabel.setLabelFor(textSingleCharTextField);
            textMultiCharsLabel.setLabelFor(textMultiCharsTextField);
            box1 = Box.createHorizontalBox();
            box1.add(Box.createHorizontalStrut(5));
            box1.add(parmFieldLabel);
            box1.add(parmFieldTextField);
            box1.add(Box.createHorizontalStrut(5));
            box2 = Box.createHorizontalBox();
            box2.add(Box.createHorizontalStrut(5));
            box2.add(textSingleCharLabel);
            box2.add(textSingleCharTextField);
            box2.add(Box.createHorizontalStrut(5));
            box3 = Box.createHorizontalBox();
            box3.add(Box.createHorizontalStrut(5));
            box3.add(textMultiCharsLabel);
            box3.add(textMultiCharsTextField);
            box3.add(Box.createHorizontalStrut(5));
            wildCardsBox.add(box1);
            wildCardsBox.add(box2);
            wildCardsBox.add(box3);
            TitledBorder wcBorder = BorderFactory.createTitledBorder(new EtchedBorder(), ResourceMgr.getString("STV_WILD_CARDS_BORDER_LABEL"));
            if (ltr) {
                wcBorder.setTitleJustification(TitledBorder.LEFT);
            }
            else {
                wcBorder.setTitleJustification(TitledBorder.RIGHT);
            }
            wildCardsBox.setBorder(wcBorder);

            // Search button visibility
            box1 = Box.createHorizontalBox();
            searchButtonsVisibleCheckBox = new JCheckBox(ResourceMgr.getString("STV_SEARCH_BUTTONS_VISIBLE"));
            box1.add(Box.createHorizontalStrut(5));
            if (ltr) {
                // add glue to right side
                box1.add(searchButtonsVisibleCheckBox);
                box1.add(Box.createHorizontalGlue());
            }
            else {
                // add glue to left side
                box1.add(Box.createHorizontalGlue());
                box1.add(searchButtonsVisibleCheckBox);
            }
            box1.add(Box.createHorizontalStrut(5));


            innerPanel.add(defaultsBox);
            innerPanel.add(wildCardsBox);
            innerPanel.add(box1);

            // NOW INITIALIZE VALUES, ADD LISTENERS
            // Wildcards
            // Get initial wildcard values
            parmFieldTextField.setText(new String(new char[]{searchTemplateViewer.getParmFieldWildcard()}));
            textSingleCharTextField.setText(new String(new char[]{searchTemplateViewer.getTextSingleCharWildcard()}));
            textMultiCharsTextField.setText(new String(new char[]{searchTemplateViewer.getTextMultiCharsWildcard()}));
            // Button Visibility
            // set check box state
            if (searchTemplateViewer.isStartSearchButtonVisible() || searchTemplateViewer.isStopSearchButtonVisible()) {
                searchButtonsVisibleCheckBox.setSelected(true);
                // make sure the 2 button states are the same
                searchTemplateViewer.setStartSearchButtonVisible(true);
                searchTemplateViewer.setStopSearchButtonVisible(true);
            }
            else {
                // no, neither is visible
                searchButtonsVisibleCheckBox.setSelected(false);
            }

      }
      return stvPanel;
   }

/**
 * Gets the srvPanel property value.
 * If the panel is null then it is defined by creating a panel with
 * options.
 *
 * @return The srvPanel property value.
 */
   private JPanel getSrvOptionsPanel()
   {
      if (srvPanel == null)
      {
         try
         {
            srvPanel = new JPanel();
            srvPanel.setLayout(new BorderLayout());
            Box srvBox = Box.createVerticalBox();
            srvPanel.add(srvBox, BorderLayout.NORTH);

            // Grid
            horizontalLinesVisibleCheckBox.setText(ResourceMgr.getString("SRV_HORIZ_LINES_VISIBLE"));
            verticalLinesVisibleCheckBox.setText(ResourceMgr.getString("SRV_VERT_LINES_VISIBLE"));
            JPanel gridPanel = new JPanel();
            gridPanel.setLayout(new BoxLayout(gridPanel, BoxLayout.Y_AXIS));
            Box box1 = Box.createHorizontalBox();
            Box box2 = Box.createHorizontalBox();
            box1.add(Box.createHorizontalStrut(5));
            box2.add(Box.createHorizontalStrut(5));
            if (ltr) {
                box1.add(horizontalLinesVisibleCheckBox);
                box1.add(Box.createHorizontalGlue());
                box2.add(verticalLinesVisibleCheckBox);
                box2.add(Box.createHorizontalGlue());
            } else {
                horizontalLinesVisibleCheckBox.setHorizontalAlignment(SwingConstants.LEFT);
                verticalLinesVisibleCheckBox.setHorizontalAlignment(SwingConstants.LEFT);
                box1.add(Box.createHorizontalGlue());
                box1.add(horizontalLinesVisibleCheckBox);
                box2.add(Box.createHorizontalGlue());
                box2.add(verticalLinesVisibleCheckBox);
            }
            box1.add(Box.createHorizontalStrut(5));
            box2.add(Box.createHorizontalStrut(5));
            gridPanel.add(box1);
            gridPanel.add(box2);
            TitledBorder border1 = BorderFactory.createTitledBorder(new EtchedBorder(), ResourceMgr.getString( "SRV_GRID_BORDER_LABEL"));
            if (ltr) {
                border1.setTitleJustification(TitledBorder.LEFT);
            }
            else {
                border1.setTitleJustification(TitledBorder.RIGHT);
            }
            gridPanel.setBorder(border1);

            // Line contrast
            lineContrastCheckBox.setText(ResourceMgr.getString("SRV_LINE_CONTRAST_CHECKBOX"));
            lineColorLabel.setText(ResourceMgr.getString("SRV_LINE_COLOR_LABEL"));
            lineColorLabel.setLabelFor(lineColorTextField);
            colorChooserButton.setText(ResourceMgr.getString("SRV_LINE_COLOR_CHOOSER_BUTTON"));
            JPanel lineContrastPanel = new JPanel();
            lineContrastPanel.setLayout(new BoxLayout(lineContrastPanel, BoxLayout.Y_AXIS));
            box1 = Box.createHorizontalBox();
            box2 = Box.createHorizontalBox();
            box1.add(Box.createHorizontalStrut(5));
            box2.add(Box.createHorizontalStrut(5));
            if (ltr) {
                box1.add(lineContrastCheckBox);
                box1.add(Box.createHorizontalGlue());
                box2.add(lineColorLabel);
                box2.add(Box.createHorizontalStrut(5));
                box2.add(lineColorTextField);
                box2.add(Box.createHorizontalStrut(5));
                box2.add(colorChooserButton);
                box2.add(Box.createHorizontalGlue());
            }
            else {
                lineContrastCheckBox.setHorizontalAlignment(SwingConstants.LEFT);
                box1.add(Box.createHorizontalGlue());
                box1.add(lineContrastCheckBox);
                box2.add(Box.createHorizontalGlue());
                box2.add(colorChooserButton);
                box2.add(Box.createHorizontalStrut(5));
                box2.add(lineColorTextField);
                box2.add(Box.createHorizontalStrut(5));
                box2.add(lineColorLabel);
            }
            box1.add(Box.createHorizontalStrut(5));
            box2.add(Box.createHorizontalStrut(5));
            border1 = BorderFactory.createTitledBorder(new EtchedBorder(), ResourceMgr.getString("SRV_LINE_CONTRAST_BORDER_LABEL"));
            if (ltr) {
                border1.setTitleJustification(TitledBorder.LEFT);
            }
            else {
                border1.setTitleJustification(TitledBorder.RIGHT);
            }
            lineContrastPanel.setBorder(border1);
            lineContrastPanel.add(box1);
            lineContrastPanel.add(box2);
            lineContrastPanel.setToolTipText(ResourceMgr.getString("SRV_LINE_CONTRAST_TOOLTIP"));
            // add listener for disabling/enabling controls
            lineContrastCheckBox.addActionListener(new ActionListener() {
                public void actionPerformed(ActionEvent e) {
                    if (lineContrastCheckBox.isSelected()) {
                        lineColorLabel.setEnabled(true);
                        colorChooserButton.setEnabled(true);
                    }
                    else {
                        lineColorLabel.setEnabled(false);
                        colorChooserButton.setEnabled(false);
                    }
                }
            });
            // Set background color of TextField
            lineColorTextField.setBackground(searchResultsViewer.getLineContrastColor());
            // add listener for color chooser button
            colorChooserButton.addActionListener(new ActionListener() {
                public void actionPerformed(ActionEvent e) {
                    // Bring up color chooser on current color
                    ColorChooserDialog dlg = new ColorChooserDialog((Frame)getParent(), lineColorTextField.getBackground());
                    dlg.setVisible(true);
                }
            });

            //Results options
            JPanel resultsOptionsPanel = new JPanel();
            resultsOptionsPanel.setLayout(new BoxLayout(resultsOptionsPanel, BoxLayout.Y_AXIS));
            ButtonGroup bgsrv=new ButtonGroup();
            clearResultsRadioButton = new JRadioButton(ResourceMgr.getString( "SEARCH_RESULTS_CLEAR_OPTIONS_PANEL"));
            saveResultsRadioButton = new JRadioButton(ResourceMgr.getString( "SEARCH_RESULTS_SAVE_OPTIONS_PANEL"));
            bgsrv.add(clearResultsRadioButton);
            bgsrv.add(saveResultsRadioButton);
            clearResultsRadioButton.setSelected(true);
            if (!ltr) {
                clearResultsRadioButton.setHorizontalAlignment(SwingConstants.RIGHT);
                saveResultsRadioButton.setHorizontalAlignment(SwingConstants.RIGHT);
                clearResultsRadioButton.setHorizontalTextPosition(SwingConstants.LEFT);
                saveResultsRadioButton.setHorizontalTextPosition(SwingConstants.LEFT);
            }
            RadioListener myListener = new RadioListener();
            clearResultsRadioButton.addActionListener(myListener);
            saveResultsRadioButton.addActionListener(myListener);

            Box clearBox = Box.createHorizontalBox();
            Box saveBox = Box.createHorizontalBox();
            clearBox.add(clearResultsRadioButton);
            clearBox.add(Box.createHorizontalGlue());
            saveBox.add(saveResultsRadioButton);
            saveBox.add(Box.createHorizontalGlue());
            resultsOptionsPanel.add(clearBox);
            resultsOptionsPanel.add(saveBox);
            border1 = BorderFactory.createTitledBorder(new EtchedBorder(), ResourceMgr.getString( "SEARCH_RESULTS_OPTIONS_PANEL"));
            if (ltr) {
                border1.setTitleJustification(TitledBorder.LEFT);
            }
            else {
                border1.setTitleJustification(TitledBorder.RIGHT);
            }
            resultsOptionsPanel.setBorder(border1);


            JPanel maxResultsPanel = new JPanel();
            maxResultsPanel.setLayout(new BoxLayout(maxResultsPanel, BoxLayout.Y_AXIS));
            maxResultsLabel1 = new JLabel(ResourceMgr.getString( "MAX_SEARCH_RESULTS1"));
            maxResultsLabel2 = new JLabel(ResourceMgr.getString( "MAX_SEARCH_RESULTS2"));
            maxResultsTextField = new JTextField(Integer.toString(clientPane.getSearchTemplateViewerBean().getMaxResults()),8);
            maxResultsTextField.addActionListener(this);
            maxResultsTextField.addKeyListener(new KeyAdapter() {
              public void keyReleased(KeyEvent e) {
               if ( isDefaultsChosen() ) {
                 defaultsButton.setEnabled(false);
                }
                else {
                  defaultsButton.setEnabled(true);
                 }
               }
              });
            maxResultsLabel1.setLabelFor(maxResultsTextField);

            box1 = Box.createHorizontalBox();
            box1.add(Box.createHorizontalStrut(5));
            if (ltr)  {
                maxResultsLabel1.setHorizontalAlignment(SwingConstants.LEFT);
                box1.add(maxResultsLabel1);
                box1.add(maxResultsTextField);
            } else {
                maxResultsLabel1.setHorizontalAlignment(SwingConstants.RIGHT);
                box1.add(maxResultsTextField);
                box1.add(maxResultsLabel1);
            }
            box1.add(Box.createHorizontalStrut(5));
            box1.add(Box.createHorizontalGlue());

            Box label2Box = Box.createHorizontalBox();
            label2Box.add(Box.createHorizontalStrut(5));
            if (ltr) {
                maxResultsLabel2.setHorizontalAlignment(SwingConstants.LEFT);
                label2Box.add(maxResultsLabel2);
                label2Box.add(Box.createHorizontalGlue());
            }
            else {
                maxResultsLabel2.setHorizontalAlignment(SwingConstants.RIGHT);
                label2Box.add(Box.createHorizontalGlue());
                label2Box.add(maxResultsLabel2);
            }
            label2Box.add(Box.createHorizontalStrut(5));

            maxResultsPanel.add(Box.createVerticalStrut(5));
            maxResultsPanel.add(box1);
            maxResultsPanel.add(label2Box);
            maxResultsPanel.add(Box.createVerticalStrut(5));
            Border border2 = BorderFactory.createEtchedBorder();
            maxResultsPanel.setBorder(border2);

            // Show tree pane
            showTreePaneCheckBox.setText(ResourceMgr.getString("SRV_SHOW_TREE_PANE"));
            showTreePaneCheckBox.setSelected(searchResultsViewer.isTreePaneVisible());
            JPanel treePaneBox = new JPanel();
            treePaneBox.setLayout(new BoxLayout(treePaneBox, BoxLayout.Y_AXIS));
            treePaneBox.add(Box.createHorizontalStrut(5));
            if (ltr) {
                treePaneBox.add(showTreePaneCheckBox);
                treePaneBox.add(Box.createHorizontalGlue());
            }
            else {
                showTreePaneCheckBox.setHorizontalTextPosition(SwingConstants.LEFT);
                treePaneBox.add(Box.createHorizontalGlue());
                treePaneBox.add(showTreePaneCheckBox);
            }
            treePaneBox.add(Box.createHorizontalStrut(5));

            // Add to main box
            srvBox.add(gridPanel);
            srvBox.add(lineContrastPanel);
            srvBox.add(resultsOptionsPanel);
            srvBox.add(maxResultsPanel);
            srvBox.add(treePaneBox);
            srvBox.add(Box.createVerticalGlue());

         }
         catch ( java.lang.Throwable exc)
         {
            throw new ShouldNotOccurException( exc.toString() + ":" + exc.getMessage());
         }
      }
      return srvPanel;
   }

/**
 * Gets the looknfeelPanel property value.
 * If the panel is null then it is defined by creating a panel with
 * options.
 *
 * @return The srvPanel property value
 */

   private JPanel getLookNFeelOptionsPanel()
   {
      if (looknfeelPanel == null)
      {
         try
         {
            looknfeelPanel = new JPanel();
            looknfeelPanel.setLayout(new BoxLayout(looknfeelPanel, BoxLayout.Y_AXIS));
            looknfeelPanel.setBorder(new EmptyBorder(5, 5, 5, 5));
            JPanel installedPanel = new JPanel();
            installedPanel.setLayout(new BoxLayout(installedPanel, BoxLayout.Y_AXIS));
            installedPanel.setBorder(new TitledBorder("Installed Look & Feels"));
            // Get all installed look & feels
            UIManager.LookAndFeelInfo[] installed = UIManager.getInstalledLookAndFeels();
            // Create buttons for each
            ButtonGroup rbGroup = new ButtonGroup();
            String lnfName = UIManager.getLookAndFeel().getName();
            for (int i = 0; i < installed.length; ++i) {
                String name = installed[i].getName();
                JRadioButton rb = new JRadioButton(name);
                rbGroup.add(rb);
                if (lnfName.equals(name)) {
                    rb.setSelected(true);
                }
                // Add listener
                rb.addActionListener(new lnfNameActionListener(installed[i].getClassName()));
                // Add to panel
                Box rbBox = Box.createHorizontalBox();
                rbBox.add(rb);
                rbBox.add(Box.createHorizontalGlue());
                installedPanel.add(rbBox);
            }
            looknfeelPanel.add(installedPanel);
            LookAndFeel[] aux = UIManager.getAuxiliaryLookAndFeels();
            if (aux != null) {
                JPanel auxPanel = new JPanel();
                auxPanel.setLayout(new BoxLayout(auxPanel, BoxLayout.Y_AXIS));
                auxPanel.setBorder(new TitledBorder("Auxiliary Look & Feels"));
                for (int i=0; i < aux.length; ++i) {
                    String name = aux[i].getName();
                    JRadioButton rb = new JRadioButton(name);
                    rbGroup.add(rb);
                    if (lnfName.equals(name)) {
                        rb.setSelected(true);
                    }
                    // Add listener
                    //rb.addActionListener(new lnfNameActionListener(aux[i].getClassName()));
                    // Add to panel
                    Box rbBox = Box.createHorizontalBox();
                    rbBox.add(rb);
                    rbBox.add(Box.createHorizontalGlue());
                    auxPanel.add(rbBox);
                }
                looknfeelPanel.add(auxPanel);
            }

            looknfeelPanel.add(Box.createVerticalGlue());
         }
         catch ( java.lang.Throwable exc)
         {
            exc.printStackTrace();
            //throw new CMBShouldNotOccurException( exc.toString() + ":" + exc.getMessage());
         }
      }
      return looknfeelPanel;
   }
 // Listener class used to set look & feel
 class lnfNameActionListener implements ActionListener {
    String className = "";
    public lnfNameActionListener(String name) {
        className = name;
    }
    public void actionPerformed(ActionEvent event) {
        try {
            UIManager.setLookAndFeel(className);
            SwingUtilities.updateComponentTreeUI(clientPane.getAppFrame());
            SwingUtilities.updateComponentTreeUI(OptionsDialog.this);
            SwingUtilities.updateComponentTreeUI(clientPane.getDocumentViewerBean());
        } catch (Exception ex) {
            ex.printStackTrace();
        }
    }
 }

 private boolean isDefaultsChosen() {
    if (clientPane.isTemplatesSupported()) {
   switch (searchTemplateViewer.getDefaultsOption()) {
          case CMBSearchTemplateViewer.SYSTEM:
             if (!systemDefaultsRadioButton.isSelected()) {
             return false;
             }
             break;
          case CMBSearchTemplateViewer.LAST_TIME:
             if (!lastTimeRadioButton.isSelected()) {
              return false;
              }
             break;
          case CMBSearchTemplateViewer.PREVIOUS_VIEW:
             if (!previousViewRadioButton.isSelected())  {
              return false;
              }
             break;
          default:
            return false;
    } // end switch
    }

    if (saveResultsRadioButton.isSelected() )  {
       return false;
    }
      String currentMaxTextValue=maxResultsTextField.getText();
      currentMaxTextValue=currentMaxTextValue.trim();
      if (!currentMaxTextValue.equals(DEFAULT_MAXRESULTS_STRING) ) {
         return false;
      }
         return true;

   }




 /** Listens to the radio buttons. */
    class RadioListener implements ActionListener {
        public void actionPerformed(ActionEvent e) {
          if ( isDefaultsChosen() ) {
             defaultsButton.setEnabled(false);
          }
           else {
              defaultsButton.setEnabled(true);
           }
         }
    }


	private static Image getImage(String name)
	{

      Toolkit toolkit= Toolkit.getDefaultToolkit();
      Image image	= null;
      MediaTracker mediaTracker = new MediaTracker(new Panel());
      java.net.URL url = ResourceMgr.getImageURL(name);
      if(url != null)
			image = toolkit.getImage(url);
      mediaTracker.addImage(image,0);
		try
		{
			mediaTracker.waitForAll();
		}
		catch(Exception ex)
		{
			System.out.println(ex);
		//	ex.printStackTrace();
		}
		return image;
	}


   /**
    * User selected the cancel button. Set the
    * options as they were before user edited
   */
 private void cancelChanges() {
      if ( searchResultsViewer.isSaveResultsPerTemplate()) {
         clearResultsRadioButton.setSelected(false);
         saveResultsRadioButton.setSelected(true);
      }
      else {
         clearResultsRadioButton.setSelected(true);
         saveResultsRadioButton.setSelected(false);
      }

      setBeanDefaultProperties();

      // Get original wildcards
      if (clientPane.isTemplatesSupported()) {
        parmFieldTextField.setText(new String(new char[] {searchTemplateViewer.getParmFieldWildcard()} ));
        textSingleCharTextField.setText(new String(new char[] {searchTemplateViewer.getTextSingleCharWildcard()} ));
        textMultiCharsTextField.setText(new String(new char[] {searchTemplateViewer.getTextMultiCharsWildcard()} ));
        //searchTemplateViewer.setParmFieldWildcard(parmFieldTextField.getText().charAt(0));
        //searchTemplateViewer.setTextSingleCharWildcard(textSingleCharTextField.getText().charAt(0));
        //searchTemplateViewer.setTextMultiCharsWildcard(textMultiCharsTextField.getText().charAt(0));

        int intMaxValue= searchTemplateViewer.getMaxResults();
        if (intMaxValue==DEFAULT_MAXRESULTS_INT) { // -1
            maxResultsTextField.setText(DEFAULT_MAXRESULTS_STRING);
        }
        else {
            String value= new String().valueOf(intMaxValue);
            maxResultsTextField.setText(value);
        }
      }
}



/**
 * Invoked when a JComboBox is selected.
 */
public void itemStateChanged(ItemEvent evt) {
  if (evt.getSource() instanceof JComboBox ) { // source is a JComboBox
     if ( isDefaultsChosen() ) {
       defaultsButton.setEnabled(false);
     }
     else {
      defaultsButton.setEnabled(true);
     }
  }
}


/** Is text value < -1 , if so then change text value to blank
*
*/
 private void checkTextValue(){
   String currentMaxTextValue=maxResultsTextField.getText();
   int intMaxValue= DEFAULT_MAXRESULTS_INT; // -1
    currentMaxTextValue=currentMaxTextValue.trim();
   if (currentMaxTextValue != null && !currentMaxTextValue.equals(DEFAULT_MAXRESULTS_STRING)) {
    intMaxValue=new Integer(currentMaxTextValue).intValue();
   }
   if (intMaxValue < DEFAULT_MAXRESULTS_INT ) { // less than -1
     maxResultsTextField.setText(DEFAULT_MAXRESULTS_STRING); // set to blank
   }
}

    public String chooseFile(Component parent) {
        JFileChooser chooser = new JFileChooser();
	        // Set file filter for .gif and .jpg
	        //ExtensionFileFilter filter = new ExtensionFileFilter();
	        //filter.addExtension("jpg");
	        //filter.addExtension("gif");
	        //filter.setDescription("JPG & GIF Images");
	        //chooser.setFileFilter(filter);
	        int returnVal = chooser.showOpenDialog(parent);
	        if (returnVal == JFileChooser.APPROVE_OPTION) {
	            return chooser.getSelectedFile().getAbsolutePath();
	        }
	        return null;
    }

	private class ColorChooserDialog extends JDialog {

	    private JColorChooser colorChooser;
	    private JButton OKButton = new JButton();
	    private JButton CancelButton = new JButton();
	    private Color chosenColor = Color.white;

	    public ColorChooserDialog(Frame f, Color c) {
	        super((Frame)f, false);
	        setBounds(350,200, 400,400);
	        colorChooser = new JColorChooser(c);
	        chosenColor = c;
	        getContentPane().add(colorChooser, BorderLayout.CENTER);
	        CancelButton.setText(ResourceMgr.getString("CANCEL"));
	        OKButton.setText(ResourceMgr.getString("OK"));
	        JPanel buttonPanel = new JPanel();
	        if (ltr) {
	            buttonPanel.setLayout(new FlowLayout(FlowLayout.RIGHT));
	            buttonPanel.add(OKButton);
	            buttonPanel.add(CancelButton);
	        }
	        else {
	            buttonPanel.setLayout(new FlowLayout(FlowLayout.LEFT));
	            buttonPanel.add(CancelButton);
	            buttonPanel.add(OKButton);
	        }
	        getContentPane().add(buttonPanel, BorderLayout.SOUTH);
	        // Add listeners to the buttons
	        OKButton.addActionListener(new ActionListener() {
	            public void actionPerformed(ActionEvent e) {
	                Color chosenColor = colorChooser.getColor();
	                setChosenColor(chosenColor);
	                ColorChooserDialog.this.setVisible(false);
	                ColorChooserDialog.this.dispose();
	            }
	        });
	        CancelButton.addActionListener(new ActionListener() {
	            public void actionPerformed(ActionEvent e) {
	                ColorChooserDialog.this.setVisible(false);
	                ColorChooserDialog.this.dispose();
	            }
	        });

	    }
	    public Color getChosenColor() { return chosenColor; }
	}

	/*class FileChooserDialog extends JDialog {
	    private JFileChooser fileChooserPanel = new JFileChooser();
	    private JButton OKButton = new JButton();
	    private JButton CancelButton = new JButton();
	    private Color chosenColor = Color.white;

	    FileChooserDialog(Frame f) {
	        super(f, true);
	        setBounds(350,200, 400,400);
	        getContentPane().add(fileChooserPanel, BorderLayout.CENTER);
	        CancelButton.setText(ResourceMgr.getString("CANCEL"));
	        OKButton.setText(ResourceMgr.getString("OK"));
	        JPanel buttonPanel = new JPanel();
	        if (ltr) {
	            buttonPanel.setLayout(new FlowLayout(FlowLayout.RIGHT));
	            buttonPanel.add(OKButton);
	            buttonPanel.add(CancelButton);
	        }
	        else {
	            buttonPanel.setLayout(new FlowLayout(FlowLayout.LEFT));
	            buttonPanel.add(CancelButton);
	            buttonPanel.add(OKButton);
	        }
	        getContentPane().add(buttonPanel, BorderLayout.SOUTH);
	        // Set file filter for .gif and .jpg
	        ExtensionFileFilter filter = new ExtensionFileFilter();
	        filter.addExtension("jpg");
	        filter.addExtension("gif");
	        filter.setDescription("JPG & GIF Images");
	        fileChooserPanel.setFileFilter(filter);
	        int returnVal = chooser.showOpenDialog(parent);
	        if (returnVal == JFileChooser.APPROVE_OPTION) {
	            System.out.println("You chose to open this file: " + chooser.getSelectedFile().getName());
	        }
	        // Add listeners to the buttons
	        OKButton.addActionListener(new ActionListener() {
	            public void actionPerformed(ActionEvent e) {
	                File chosenColor = colorChooser.getColor();
	                setChosenColor(chosenColor);
	                FileChooserDialog.this.setVisible(false);
	                FileChooserDialog.this.dispose();
	            }
	        });
	        CancelButton.addActionListener(new ActionListener() {
	            public void actionPerformed(ActionEvent e) {
	                FileChooserDialog.this.setVisible(false);
	                FileChooserDialog.this.dispose();
	            }
	        });

	    }

	}*/

	class listStyleListener implements ItemListener {
        public void itemStateChanged(ItemEvent e) {
            if (imageListStyleRadioButton.isSelected()) {
                // enable image choosers
                selectedImageLabel.setEnabled(true);
                selectedImageURLTextField.setEnabled(true);
                selectedImageChooserButton.setEnabled(true);
                unselectedImageLabel.setEnabled(true);
                unselectedImageURLTextField.setEnabled(true);
                unselectedImageChooserButton.setEnabled(true);
            }
            else {
                 // disable image choosers
                 selectedImageLabel.setEnabled(false);
                 selectedImageURLTextField.setEnabled(false);
                 selectedImageChooserButton.setEnabled(false);
                 unselectedImageLabel.setEnabled(false);
                 unselectedImageURLTextField.setEnabled(false);
                 unselectedImageChooserButton.setEnabled(false);
            }
        }
    }
}

