//
/*BEGINPROLOGUE************************************************************************
 * @copyright(disclaimer)                                                             *
 * Licensed Materials - Property of IBM                                               *
 * IBM Enterprise Information Portal for Multiplatforms V8.1 (program number 5724-B43)*
 * (c ) Copyright IBM Corp. 1998,  2002. All Rights Reserved.                         *
 *                                                                                    *
 * US Government Users Restricted Rights                                              *
 * Use, duplication or disclosure restricted by GSA ADP Schedule                      *
 * Contract with IBM Corp.                                                            *
 *                                                                                    *
 * DISCLAIMER OF WARRANTIES :                                                         *
 *                                                                                    *
 * Permission is granted to copy and modify this Sample code, and to                  *
 * distribute modified versions provided that both the copyright                      *
 * notice,- and this permission notice and warranty disclaimer appear in all copies   * 
 * and modified versions.                                                             *
 *                                                                                    *
 * THIS SAMPLE CODE IS LICENSED TO YOU AS-IS.                                         *
 * IBM AND ITS SUPPLIERS AND LICENSORS DISCLAIM ALL WARRANTIES, EITHER EXPRESS OR     *
 * IMPLIED, IN SUCH SAMPLE CODE, INCLUDING THE WARRANTY OF NON-INFRINGEMENT AND THE   *
 * IMPLIED WARRANTIES OF MERCHANTABILITY OR FITNESS FOR A PARTICULAR PURPOSE. IN NO   *
 * EVENT WILL IBM OR ITS LICENSORS OR SUPPLIERS BE LIABLE FOR ANY DAMAGES ARISING OUT *
 * OF THE USE OF OR INABILITY TO USE THE SAMPLE CODE, DISTRIBUTION OF THE SAMPLE CODE,*
 * OR COMBINATION OF THE SAMPLE CODE WITH ANY OTHER CODE. IN NO EVENT SHALL IBM OR ITS*
 * LICENSORS AND SUPPLIERS BE LIABLE FOR ANY LOST REVENUE, LOST PROFITS OR DATA, OR   *
 * FOR DIRECT, INDIRECT, SPECIAL, CONSEQUENTIAL,INCIDENTAL OR PUNITIVE DAMAGES,       *
 * HOWEVER CAUSED AND REGARDLESS OF THE THEORY OF LIABILITY,-, EVEN IF IBM OR ITS     *
 * LICENSORS OR SUPPLIERS HAVE BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGES.       *
 *                                                                                    *
 * @endCopyright                                                                      *
 **************************************************************************************/
//


// sample queries from a Federated Datastore

/*
 * Filename       : TCQMapExecuteDES.java
 * Description    : Connect to DES Datastore.
 *                  Create mappings to simulate a query from a Federated Datastore
 *                  Create and execute a number of query expressions
 *                  Specify to retrieve a max result of 5 items.
 *                  Get the result in a resultset cursor.
 *                  Loop thru results.
 *                  Display documents found and their parts and attributes.
 *                  Disconnect from DES Datastore.
 */

import com.ibm.mm.sdk.common.*;
import com.ibm.mm.sdk.server.*;
import java.io.*;
import java.lang.*;
import java.util.*;

// TCQMapExecuteDES class
public class TCQMapExecuteDES implements DKConstantDES, DKConstantFed
{


  // Main method
  public static void main(String argv[])
  {

   // Indicate this is a Console Application on Windows NT/95 for error
   // message display in dos console code page. If this was a Windows
   // GUI (Graphic User Interface) Application this would not be necessary
   // because DK_CM_SS_WINDOWS is the default.
   DKEnvironment.setSubSystem(DK_CM_SS_CONSOLE);

    DKDatastoreDES dsDES = null;
    try {
      //*** for input parameters
      String libSrv = new String(""),   // library server
             userid = new String(""),     //userid
  	     pw = new String(""); // pw

      //--------------------------------------------------------------
      // Checking for input parameters
      //--------------------------------------------------------------
      if (argv.length != 4)
      {
       System.out.println("Usage: " );
       System.out.println("java TCQMapExecuteDES " +
        "tcp/ipAddress " +
        "userid " +
        "userpw " +
        "PORT=80;DESAPPID=appid;DESAPPPW=apppw[;DESCFGPATH=<des cfg file>;DESLOCALE=<locale>]");
       return;
      }
      else
      {
       libSrv = argv[0];
       userid = argv[1];
       pw = argv[2];
      }
    System.out.println("*** TCQMapExecuteDES ***");
    dsDES = new DKDatastoreDES();
    dkResultSetCursor pCur = null;
    dkCollection pCol = null;
    dkIterator pIter = null;
    DKNVPair soptions[] = new DKNVPair[1];
    DKNVPair tnv = null;
    dkEntityDef entDef = null;
    dkAttrDef attrDef = null;
    dkDatastoreDef dsDef = null;
    String strNativeEntityName = "Notes Help";
    String strNativeAttrName = "Subject";
    String strNativeAttrName2 = "View";
    String strMappedEntityName = "FEDSAMPLE";
    String strMappedAttrName = "FEDSubject";
    String strMappedAttrName2 = "FEDView";
    DKSchemaMappingFed mapFed = new DKSchemaMappingFed();
    DKAttrMappingFed attrMapFed = new DKAttrMappingFed();
    DKAttrMappingFed attrMapFed2 = new DKAttrMappingFed();

    mapFed.setServerName(libSrv);
    mapFed.setServerType(DK_DES_DSTYPE);
    mapFed.setEntityName(strNativeEntityName);
    mapFed.setMappedEntityName(strMappedEntityName);
    mapFed.setMappedEntityType(DKConstantFed.MAPPED);
    int i = 0;
    String strTmp = "";
    String strAttrName = "";
    System.out.println("connecting to datastore");
    dsDES.connect(libSrv,userid,pw,argv[3]);
    System.out.println("datastore connected libSrv " + libSrv + " userid " +
                        userid );
    dsDef = dsDES.datastoreDef();
    entDef = dsDef.getEntity(strNativeEntityName);

    attrDef = entDef.createAttr();
    attrDef.setName(strMappedAttrName);
    attrDef.setType(DK_CM_VSTRING);
    attrMapFed.addMappedAttr(attrDef); // Add LHS

    attrDef = entDef.createAttr();
    attrDef.setName(strMappedAttrName2);
    attrDef.setType(DK_CM_VSTRING);
    attrMapFed2.addMappedAttr(attrDef); // Add LHS

    pCol = dsDef.listEntityAttrs(strNativeEntityName);
    pIter = pCol.createIterator();
    i = 0;
    boolean fDone = false;
    while (pIter.more() == true)
    {
     i++;
     attrDef = (dkAttrDef)pIter.next();
     strAttrName = attrDef.getName();
     if (strAttrName.equals(strNativeAttrName))
     {
      attrMapFed.addMapToAttr(attrDef); // Add RHS
     }
     else if (strAttrName.equals(strNativeAttrName2))
     {
      attrMapFed2.addMapToAttr(attrDef); // Add RHS
     }
    }

    mapFed.addAttrMapping(attrMapFed);
    mapFed.addAttrMapping(attrMapFed2);
    DKNVPair sourceMap = new DKNVPair("SCHEMA",mapFed);
    dsDES.registerMapping(sourceMap);   // register this mapping with datastore

    DKCQExpr cqe = new DKCQExpr();
    DKQExpr qe = new DKQExpr();
    DKQTerm qterm = new DKQTerm(strMappedAttrName2,DKOpCode.EQ,"How Do I?");
    DKQSubExpr sqe = new DKQSubExpr();
    cqe.setQLType(DK_CM_PARAMETRIC_QL_TYPE);
    qe.setQLType(DK_CM_PARAMETRIC_QL_TYPE);
    sqe.setTranslation(true);
    sqe.setMappingName(mapFed.getName());
    sqe.setEntityName(strMappedEntityName);
    sqe.setQueryTerm(qterm);
    // Display list- uncomment to display a set
    sqe.setDisplayList(new String[]{strMappedAttrName,strMappedAttrName2});
    strTmp = "3";
    tnv = new DKNVPair("MAX_RESULTS",strTmp);
    System.out.println("Set option MAX_RESULTS=" + strTmp);
    soptions[0] = tnv;
    sqe.setOptionList(soptions);
    qe.addSubQuery(sqe);
    cqe.setQueryExpr(qe);

    /**
     * Test query 1: ("FedView" EQ "How Do I?")
     **/
    System.out.println(" ");
    System.out.println("* * Test query 1 * *");
    System.out.println("executing query: " + qterm.toString());

    pCur = dsDES.execute(cqe);
    processResults(pCur);

    /**
     * Test query 2: ("FedView" EQ "How Do I?") OR ("FedSubject" IN "a")
     *               Second expression will be pruned out because IN is not valid in GQL
     **/
    System.out.println(" ");
    System.out.println("* * Test query 2 * *");
    qterm = new DKQTerm(new DKQTerm(strMappedAttrName2,DKOpCode.EQ,"How Do I?"),
                        DKOpCode.OR,
                        new DKQTerm(strMappedAttrName, DKOpCode.IN, "a"));
    System.out.println("executing query: " + qterm.toString());
    sqe.setQueryTerm(qterm);
    pCur = dsDES.execute(cqe);
    processResults(pCur);

    /**
     * Test query 3: ("FedView" EQ "How Do I?") AND ("FedSubject" IN "a")
     *               Second expression will be pruned out because IN is not valid in GQL
     **/
    System.out.println(" ");
    System.out.println("* * Test query 3 * *");
    qterm = new DKQTerm(new DKQTerm(strMappedAttrName2,DKOpCode.EQ,"How Do I?"),
                        DKOpCode.AND,
                        new DKQTerm(strMappedAttrName, DKOpCode.IN, "a"));
    System.out.println("executing query: " + qterm.toString());
    sqe.setQueryTerm(qterm);
    pCur = dsDES.execute(cqe);
    processResults(pCur);

    /**
     * Test query 4: ("FedView" EQ "Hello") OR ("FedSubject" EQ "Goodbye")
     *               No documents found
     **/
    System.out.println(" ");
    System.out.println("* * Test query 4 * *");
    qterm = new DKQTerm(new DKQTerm(strMappedAttrName2,DKOpCode.EQ,"Hello"),
                        DKOpCode.OR,
                        new DKQTerm(strMappedAttrName, DKOpCode.EQ, "Goodbye"));
    System.out.println("executing query: " + qterm.toString());
    sqe.setQueryTerm(qterm);
    pCur = dsDES.execute(cqe);
    processResults(pCur);

    System.out.println(" ");
    dsDES.disconnect();
    System.out.println("datastore disconnected");
    System.out.println("Test finished");
    dsDES.destroy();
    } // end try
    catch (DKException exc)
    {
     try {
     dsDES.destroy();
     }
     catch (Exception e)
     {
      e.printStackTrace();
     }
     System.out.println("Exception name " + exc.name());
     System.out.println("Exception message " + exc.getMessage());
     System.out.println("Exception error code " + exc.errorCode());
     System.out.println("Exception error state " + exc.errorState());
     exc.printStackTrace();
    }
    catch (Exception exc)
    {
     try {
     dsDES.destroy();
     }
     catch (Exception e)
     {
      e.printStackTrace();
     }
     System.out.println("Exception message " + exc.getMessage());
     exc.printStackTrace();
    }
  } // end main


  public static void processResults(dkResultSetCursor pCur)
                throws DKException, Exception
    {
    DKDDO ddo = null;
    if (pCur == null)
      {
      System.out.println("cursor null");
      return;
      }
    if (pCur instanceof DKResultSetCursorEmptyDES)
      System.out.println("Result: Empty cursor object");
    while (pCur.isValid() && !pCur.isEnd())
     {
     ddo = pCur.fetchNext();
     if (ddo != null)
       {
       System.out.println("");
       System.out.println("=====> Item " + pCur.getPosition() + " <=====");
       displayDDO(ddo);
       }
     }// end while

    pCur.destroy(); // Finished with the cursor

    }// end processResults

   // method to show DDO hit
   public static void displayDDO(DKDDO ddo)
                throws DKException, Exception
     {
     dkXDO pXDO = null;
     int i = 0;
     int numDataItems = 0;
     short k = 0;
     short j = 0;
     Integer valueCount = null;
     Object value = null;
     String dataName = null;
     dkCollection pCol = null;
     dkIterator pIter = null;
     Object anObject = null;
     numDataItems = ddo.dataCount();
     DKPid pid = ddo.getPidObject();
     System.out.println("pid string " + pid.pidString());

      System.out.println("Number of Attributes " + numDataItems);
      for (j = 1; j <= numDataItems; j++)
        {
        anObject = ddo.getData(j);
        dataName = ddo.getDataName(j);
        System.out.println(j + ": Name " + dataName );
        // determine if data item has a single value or multiple values
        Short type = (Short)ddo.getDataPropertyByName(j, DK_CM_PROPERTY_TYPE);
        k = type.shortValue();
        if (k == DK_CM_COLLECTION)
          {
          pCol = (dkCollection)anObject;
          pIter = pCol.createIterator();
          i = 0;
          while (pIter.more() == true)
            {
            i++;
            value = pIter.next();
            System.out.println("   Value" + i + " " + value);
            }
          } // end k > 1
        else
          System.out.println("   Value " + anObject);
        } // end for j
      } // end displayDDO

}

