/*BEGINPROLOGUE************************************************************************
 * @copyright(disclaimer)                                                             *
 * Licensed Materials - Property of IBM                                               *
 * IBM Information Integrator for Content V8.2  (program number 5724-B43)             *
 * (c ) Copyright IBM Corp. 1998, 2002, 2003. All Rights Reserved.                    *
 *                                                                                    *
 * US Government Users Restricted Rights                                              *
 * Use, duplication or disclosure restricted by GSA ADP Schedule                      *
 * Contract with IBM Corp.                                                            *
 *                                                                                    *
 * DISCLAIMER OF WARRANTIES :                                                         *
 *                                                                                    *
 * Permission is granted to copy and modify this Sample code, and to                  *
 * distribute modified versions provided that both the copyright                      *
 * notice,- and this permission notice and warranty disclaimer appear in all copies   * 
 * and modified versions.                                                             *
 *                                                                                    *
 * THIS SAMPLE CODE IS LICENSED TO YOU AS-IS.                                         *
 * IBM AND ITS SUPPLIERS AND LICENSORS DISCLAIM ALL WARRANTIES, EITHER EXPRESS OR     *
 * IMPLIED, IN SUCH SAMPLE CODE, INCLUDING THE WARRANTY OF NON-INFRINGEMENT AND THE   *
 * IMPLIED WARRANTIES OF MERCHANTABILITY OR FITNESS FOR A PARTICULAR PURPOSE. IN NO   *
 * EVENT WILL IBM OR ITS LICENSORS OR SUPPLIERS BE LIABLE FOR ANY DAMAGES ARISING OUT *
 * OF THE USE OF OR INABILITY TO USE THE SAMPLE CODE, DISTRIBUTION OF THE SAMPLE CODE,*
 * OR COMBINATION OF THE SAMPLE CODE WITH ANY OTHER CODE. IN NO EVENT SHALL IBM OR ITS*
 * LICENSORS AND SUPPLIERS BE LIABLE FOR ANY LOST REVENUE, LOST PROFITS OR DATA, OR   *
 * FOR DIRECT, INDIRECT, SPECIAL, CONSEQUENTIAL,INCIDENTAL OR PUNITIVE DAMAGES,       *
 * HOWEVER CAUSED AND REGARDLESS OF THE THEORY OF LIABILITY,-, EVEN IF IBM OR ITS     *
 * LICENSORS OR SUPPLIERS HAVE BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGES.       *
 *                                                                                    *
 * @endCopyright                                                                      *
 **************************************************************************************

NOTE:  When retrieving Attribute Definitions, if it does not exist, null is returned.  
       In this sample, if null is detected, no action will take place for that particular 
       Attribute definition because it was not defined.

********************************************************************/

// Imports
import com.ibm.mm.sdk.common.*;
import com.ibm.mm.sdk.server.*;
import java.io.*;

/************************************************************************************************
 *          FILENAME: SAttributeDefinitionDeletionICM.java
 *                    ---------------------------------------------------------------------------
 *       DESCRIPTION: Delete Attribute Definitions, Insurance Model Attributes, and other Sample
 *                    Attributes.
 *                    ---------------------------------------------------------------------------
 *     DEMONSTRATION: Deleting Attribute Definitions Using it's del() Method.
 *                    Deleting Attribute Definitions Using the Datastore Definition's del() Method.
 *                    Deletes Attributes Created by SAttributeDefinitionCreationICM
 *                    ---------------------------------------------------------------------------
 * COMMANDLINE USAGE: java SAttributeDefinitionDeletionICM <database> <userName> <password>
 *                    ---------------------------------------------------------------------------
 *      PREREQUISITS: Attributes should already be defined.  If not, Sample
 *                    SAttributeDefinitionCreationICM should be run first to create the 
 *                    Attribute.  This sample will still complete, but only those that exist 
 *                    will be deleted.
 *                    ---------------------------------------------------------------------------
 *    FILES REQUIRED: SConnectDisconnectICM.java
 ************************************************************************************************/
public class SAttributeDefinitionDeletionICM{
    
    //-------------------------------------------------------------
    // Main
    //-------------------------------------------------------------
    /**
     * Run the Sample.
     * @param argv[] String Array containing arguments.  Optional arguments are <databse> <userName> <password>.
     */
    public static void main(String argv[]) throws DKException, Exception{
    
        // Defaults for connecting to the database.
        String database = SConnectDisconnectICM.DEFAULT_DATABASE;
        String userName = SConnectDisconnectICM.DEFAULT_USERNAME;
        String password = SConnectDisconnectICM.DEFAULT_PASSWORD;
        
        //------------------------------------------------------------
        // Checking for input parameters
        //--------------------------------------------------------------
        if (argv.length < 3) { // if not all 3 arguments were specified, use defaults and report correct usage.
            System.out.println("Usage: " );
            System.out.println("  java SAttributeDefinitionDeletionICM <database> <userName> <password>" );
            System.out.println("  *** Some parameters not specified, using defaults..." );
            System.out.println("");
        } else {  // otherwise enough parameters were specified, use the first 3 for the 3 parameters.
            if (argv.length > 0) database = argv[0];
            if (argv.length > 1) userName = argv[1];
            if (argv.length > 2) password = argv[2];
        }//end else
        String ver = SConnectDisconnectICM.EIP_VERSION;
        
        System.out.println("================================================");
        System.out.println("IBM Information Integrator for Content      v"+ver);
        System.out.println("Sample Program:  SAttributeDefinitionDeletionICM");
        System.out.println("------------------------------------------------");
        System.out.println(" Database: "+database);
        System.out.println(" UserName: "+userName);
        System.out.println("================================================\n");
        
        try{
            //-------------------------------------------------------------
            // Connect to datastore
            //-------------------------------------------------------------
            // See Sample SConnectDisconnectICM for more information
            System.out.println("Connecting to datastore (Database '"+database+"', UserName '"+userName+"')...");

                DKDatastoreICM dsICM = new DKDatastoreICM();  // Create new datastore object.
                dsICM.connect(database,userName,password,""); // Connect to the datastore.

            System.out.println("Connected to datastore (Database '"+dsICM.datastoreName()+"', UserName '"+dsICM.userName()+"').");

            //----------------------------------------------------------------
            // Retrieve the S_varchar Attribute Definition
            //----------------------------------------------------------------
            // For more information about the defined attributes, please consult
            // the SAttributeDefinitionCreationICM sample.
            System.out.println("Retrieving the Attribute Definition of 'S_varchar'...");

                // First, we need acces to the Datastore Definition object.
                DKDatastoreDefICM dsDefICM = (DKDatastoreDefICM) dsICM.datastoreDef();    // Get the datastore definition object from the connected datastore.
                DKAttrDefICM attrDef = (DKAttrDefICM) dsDefICM.retrieveAttr("S_varchar"); // Retrieve the definition.
                
            System.out.println("Retrieved the Attribute Definition of 'S_varchar'.");

            //----------------------------------------------------------------
            // Delete the S_varchar Attribute Definition in the Datastore Definition.
            //----------------------------------------------------------------
            // The attribute definition can be deleted using its own del() method or
            // by using the Datastore Definition's del() funciton.
            System.out.println("Deleting the Attribute Definition 'S_varchar' in Datastore Definition...");

                if(attrDef!=null) // if it is not already deleted,
                    attrDef.del(); // Delete the attribute definition

            System.out.println("Deleted the Attribute Definition 'S_varchar' in Datastore Definition.");

            //----------------------------------------------------------------
            // Retrieve & Delete the S_blob Attribute Definition
            //----------------------------------------------------------------
            // This method will use the datastore definition to delete the attribute.
            System.out.println("Deleting the Attribute Definition 'S_blob'...");

                attrDef = (DKAttrDefICM) dsDefICM.retrieveAttr("S_blob"); // Retrieve the definition.
                if(attrDef!=null) // if it is not already deleted,
                    dsDefICM.del(attrDef); // Delete the attribute definition

            System.out.println("Deleted the Attribute Definition 'S_blob'.");

            //----------------------------------------------------------------
            // Delete the rest of the attributes created in the creation sample
            //----------------------------------------------------------------
            // Delete all attributes created in the SAttributeDefinitionCreationICM
            // sample, including the Insurance Model.
            deleteAttrDef(dsICM,"S_char");
            deleteAttrDef(dsICM,"S_clob");
            deleteAttrDef(dsICM,"S_date");
            deleteAttrDef(dsICM,"S_decimal");
            deleteAttrDef(dsICM,"S_double");
            deleteAttrDef(dsICM,"S_integer");
            deleteAttrDef(dsICM,"S_short");
            deleteAttrDef(dsICM,"S_time");
            deleteAttrDef(dsICM,"S_timestamp");
            deleteAttrDef(dsICM,"S_firstName");
            deleteAttrDef(dsICM,"S_lastName");
            deleteAttrDef(dsICM,"S_street");
            deleteAttrDef(dsICM,"S_city");
            deleteAttrDef(dsICM,"S_state");
            deleteAttrDef(dsICM,"S_zipCode");
            deleteAttrDef(dsICM,"S_policyNumber");
            deleteAttrDef(dsICM,"S_vin");
            deleteAttrDef(dsICM,"S_reportNumber");
            deleteAttrDef(dsICM,"S_accidentDate");
            deleteAttrDef(dsICM,"S_licensePlate");
            deleteAttrDef(dsICM,"S_driverLicense");
            deleteAttrDef(dsICM,"S_claimNumber");
            deleteAttrDef(dsICM,"S_claimantFName");
            deleteAttrDef(dsICM,"S_claimantLName");
            deleteAttrDef(dsICM,"S_claimantLicns");
            deleteAttrDef(dsICM,"S_insuredFName");
            deleteAttrDef(dsICM,"S_insuredLName");
            deleteAttrDef(dsICM,"S_adjusterFName");
            deleteAttrDef(dsICM,"S_adjusterLName");
            deleteAttrDef(dsICM,"S_adjustmntDate");

            //-------------------------------------------------------------
            // Disconnect from datastore & Destroy Reference
            //-------------------------------------------------------------
            // See Sample SConnectDisconnectICM for more information
            System.out.println("Disconnecting from datastore & destroying reference...");            

                dsICM.disconnect();
                dsICM.destroy();

            System.out.println("Disconnected from datastore & destroying reference.");

            //-------------------------------------------------------------
            // Sample program completed without exception
            //-------------------------------------------------------------
            System.out.println("\n==========================================");
            System.out.println("Sample program completed.");
            System.out.println("==========================================\n");
        }
        //------------------------------------------------------------
        // Catch & Print Exceptions        
        //------------------------------------------------------------
        catch (DKException exc){
            SConnectDisconnectICM.printException(exc);  // Print the exception using the function listed below.
            throw(exc);
        } catch (Exception exc) {
            SConnectDisconnectICM.printException(exc);  // Print the exception using the function listed below.
            throw(exc);
        }
    }// end main

    //=================================================================
    // Wrapper Functions
    //=================================================================
    // The following are wrapper functions for functionality covered
    // in this sample.  These functions may be used by other samples.

   /**
    * Delete the specified Attribute Definition
    * @param dsICM     Connected DKDatastoreICM object.
    * @param attrName  Attribute Name for the Attribute Definition to Delete.
    **/
    public static void deleteAttrDef(DKDatastoreICM dsICM, String attrName) throws DKException, Exception{
        System.out.println("Deleting the Attribute Definition '"+attrName+"'...");

            // First, we need acces to the Datastore Definition object.
            DKDatastoreDefICM dsDefICM = (DKDatastoreDefICM) dsICM.datastoreDef(); // Get the datastore definition object from the connected datastore.
            DKAttrDefICM attrDef = (DKAttrDefICM) dsDefICM.retrieveAttr(attrName); // Retrieve the definition.
            if(attrDef!=null) // if it is not already deleted...
                attrDef.del();   // Delete the Attribute Definition

        System.out.println("Deleted the Attribute Definition '"+attrName+"'.");
    }
    
}//end class SAttributeDefinitionDeletionICM
              
