/*BEGINPROLOGUE************************************************************************
 * @copyright(disclaimer)                                                             *
 * Licensed Materials - Property of IBM                                               *
 * IBM Information Integrator for Content V8.2  (program number 5724-B43)             *
 * (c ) Copyright IBM Corp. 1998, 2002, 2003. All Rights Reserved.                    *
 *                                                                                    *
 * US Government Users Restricted Rights                                              *
 * Use, duplication or disclosure restricted by GSA ADP Schedule                      *
 * Contract with IBM Corp.                                                            *
 *                                                                                    *
 * DISCLAIMER OF WARRANTIES :                                                         *
 *                                                                                    *
 * Permission is granted to copy and modify this Sample code, and to                  *
 * distribute modified versions provided that both the copyright                      *
 * notice,- and this permission notice and warranty disclaimer appear in all copies   * 
 * and modified versions.                                                             *
 *                                                                                    *
 * THIS SAMPLE CODE IS LICENSED TO YOU AS-IS.                                         *
 * IBM AND ITS SUPPLIERS AND LICENSORS DISCLAIM ALL WARRANTIES, EITHER EXPRESS OR     *
 * IMPLIED, IN SUCH SAMPLE CODE, INCLUDING THE WARRANTY OF NON-INFRINGEMENT AND THE   *
 * IMPLIED WARRANTIES OF MERCHANTABILITY OR FITNESS FOR A PARTICULAR PURPOSE. IN NO   *
 * EVENT WILL IBM OR ITS LICENSORS OR SUPPLIERS BE LIABLE FOR ANY DAMAGES ARISING OUT *
 * OF THE USE OF OR INABILITY TO USE THE SAMPLE CODE, DISTRIBUTION OF THE SAMPLE CODE,*
 * OR COMBINATION OF THE SAMPLE CODE WITH ANY OTHER CODE. IN NO EVENT SHALL IBM OR ITS*
 * LICENSORS AND SUPPLIERS BE LIABLE FOR ANY LOST REVENUE, LOST PROFITS OR DATA, OR   *
 * FOR DIRECT, INDIRECT, SPECIAL, CONSEQUENTIAL,INCIDENTAL OR PUNITIVE DAMAGES,       *
 * HOWEVER CAUSED AND REGARDLESS OF THE THEORY OF LIABILITY,-, EVEN IF IBM OR ITS     *
 * LICENSORS OR SUPPLIERS HAVE BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGES.       *
 *                                                                                    *
 * @endCopyright                                                                      *
 **************************************************************************************

Playing Video Items
    Streaming video may be played through the Video Charger Player.
    
    Online Streaming Video
        When the DKVideoStreamICM.play() function is invoked, it will play the file / 
        return information for an ivs file, which may ben be used with Video Charger
        Player to play the ivs file.
        
        If the Video Charger player has been configured for the current machine and if
        the current machine is an x86 platform, this particular Sample will automatically
        launch the Video Charger Player application, sending it the ivs file obtained.
        
        This does not mean that it will only work on x86 platform.  This particular sample
        has been configured to automatically launch the applicaiton if it detects the x86
        platform.  In any case, the ivs file will still be obtained as shown in this sample,
        regardless of platform.  Video Charger Player may be launched seperately to later
        play the ivs file.
        
    Options & Properties
        For a complete list and explanation of options & properties, please refer to the
        Application Programming Reference (javadoc) for both DKStreamICM & DKVideoStreamICM.
        Several of the key properties are outlined below.
        
        Play Start
            Specifies the beginning time to start playing at.  
            
            Specify the time in the format of hh:mm:ss:ms.  hh - Hours, mm - minutes, 
            ss - seconds, ms - milliseconds.  00:00:00:00 is used to specify the beginning.
            
        Play Stop
            Specifies the time to stop playing.  00:00:00:00 is used to simply play the
            entire file.

            Specify the time in the format of hh:mm:ss:ms.  
            hh - Hours, mm - minutes, ss - seconds, ms - milliseconds.

********************************************************************/

// Imports
import com.ibm.mm.sdk.common.*;
import com.ibm.mm.sdk.server.*;
import java.io.*;

/************************************************************************************************
 *          FILENAME: SVideoItemPlayICM.java
 *                    ---------------------------------------------------------------------------
 *       DESCRIPTION: Play a Streaming Video Item.
 *                    ---------------------------------------------------------------------------
 *     DEMONSTRATION: Playing Video Resource Items
 *                    Play a Streaming Video Online using Video Charger Player
 *                    ---------------------------------------------------------------------------
 * COMMANDLINE USAGE  java SVideoItemPlayICM  <3rd Party Host Name>
 *                                            <3rd Party Port>
 *                                            <3rd Party Protocol (http|ftp|file)>
 *                                            <3rd Party User Name>
 *                                            <3rd Party Password>
 *                                            <3rd Party Fully-Qualified File Name>
 *                                            <3rd Party File Path>
 *                                            <3rd Party MIME Type>
 *                                            <3rd Party Content Length>
 *                                            <Video Charger Player Program & Path>
 *                                            <database> <userName> <password>
 *
 *                    IMPORTANT NOTE:  This particular Sample will only launch Video Charger
 *                                     running on a x86 platform.  If not on an x86 platform,
 *                                     the ivs file will be downloaded, but VC must be
 *                                     launched seperately if available to play the video.
 *
 *                    NOTE:  No 3rd Pary information is actually required to Play a video.
 *                           This particular Sample needs the 3rd Party information to first
 *                           create the Video Item before playing it.
 * 
 *                    NOTE:  Video Charger Program, Database, UserName, & Password are
 *                           optional.  If any are not specified defaults will be used.
 *                           
 *                    Ex: java SVideoItemPlayICM myFtpHostName 21 ftp myUserName myPassword SVideoItemICM_Video1.mpg // video/mpeg 2791428
 *                    Ex: java SVideoItemPlayICM myFtpHostName 21 ftp myUserName myPassword SVideoItemICM_Video2.mpg // video/mpeg 1882116 C:\Progra~1\IBM\IBMVid~1\iscview.exe icmnlsdb icmadmin password
 *                    ---------------------------------------------------------------------------
 *      PREREQUISITS: The Sample Video Resource Item Type(s) must already be defined.  If not,
 *                    please run the following sample:
 *                        - SVideoItemTypeCreateICM
 *                    The file specified in the commandline parameter must exist on a 3rd Party
 *                    host.  A recommended solution is to setup an FTP server, placing the 
 *                    provided videos in the root directory:
 *                        - SVideoItemICM_Video1.mpg
 *                        - SVideoItemICM_Video2.mpg
 *                    ---------------------------------------------------------------------------
 *    FILES REQUIRED: SConnectDisconnectICM.java
 *                    SVideoItemICM_Video1.mpg -> On 3rd Party Host
 *                    AND/OR SVideoItemICM_Video2.mpg -> On 3rd Party Host
 *                    AND/OR Other Video -> On 3rd Party Host
 ************************************************************************************************/
public class SVideoItemPlayICM{

    //-------------------------------------------------------------
    // Main
    //-------------------------------------------------------------
    /**
     * Run the Sample.
     * @param argv[] String Array containing arguments.  Optional arguments are <databse> <userName> <password>.
     */
    public static void main(String argv[]) throws DKException, Exception{
    
        // Defaults for connecting to the database.
        String database = SConnectDisconnectICM.DEFAULT_DATABASE;
        String userName = SConnectDisconnectICM.DEFAULT_USERNAME;
        String password = SConnectDisconnectICM.DEFAULT_PASSWORD;

        // User Input Variables
        String videoChargerPlayer = "C:\\Progra~1\\IBM\\IBMVid~1\\iscview.exe";  // Default location of VideoCharger Player

        // 3rd Party Information
        String thirdPartyHostName       = "NOT_SPECIFIED";    // Please refer to the header documentation for descriptions.
        int    thirdPartyPort           = -1;
        String thirdPartyProtocol       = "NOT_SPECIFIED";
        String thirdPartyUserName       = "NOT_SPECIFIED";
        String thirdPartyPassword       = "NOT_SPECIFIED";
        String thirdPartyFullFileName   = "NOT_SPECIFIED";     
        String thirdPartyFilePath       = "NOT_SPECIFIED";
        String thirdPartyMIMEType       = "NOT_SPECIFIED";
        int    thirdPartyContentLength  = -1;
        
        //------------------------------------------------------------
        // Checking for input parameters
        //--------------------------------------------------------------
        if (argv.length < 12) { // If not all arguments specified, use defaults for those not specified and display usage info.
            System.out.println("Usage: " );
            System.out.println("  java SVideoItemPlayICM  <3rd Party Host Name>");
            System.out.println("                          <3rd Party Port>");
            System.out.println("                          <3rd Party Protocol (http|ftp|file)>");
            System.out.println("                          <3rd Party User Name>");
            System.out.println("                          <3rd Party Password>");
            System.out.println("                          <3rd Party Fully-Qualified File Name>");
            System.out.println("                          <3rd Party File Path>");
            System.out.println("                          <3rd Party MIME Type>");
            System.out.println("                          <3rd Party Content Length>");
            System.out.println("                          <Video Charger Player Program & Path>");
            System.out.println("                          <database> <userName> <password>");
            System.out.println("");
            System.out.println("  IMPORTANT NOTE:  This particular Sample will only launch Video Charger");
            System.out.println("                   running on a x86 platform.  If not on an x86 platform,");
            System.out.println("                   the ivs file will be downloaded, but VC must be");
            System.out.println("                   launched seperately if available to play the video.");
            System.out.println("");
            System.out.println("  NOTE:  No 3rd Pary information is actually required to Play a video.");
            System.out.println("         This particular Sample needs the 3rd Party information to first");
            System.out.println("         create the Video Item before playing it.");
            System.out.println("");
            System.out.println("  NOTE:  Video Charger Program, Database, UserName, & Password are optional.");
            System.out.println("         If any are not specified defaults will be used.");
            System.out.println("");         
            System.out.println("  Ex: java SVideoItemPlayICM myFtpHostName 21 ftp myUserName myPassword SVideoItemICM_Video1.mpg // video/mpeg 2791428");
            System.out.println("  Ex: java SVideoItemPlayICM myFtpHostName 21 ftp myUserName myPassword SVideoItemICM_Video2.mpg // video/mpeg 1882116 C:\\Progra~1\\IBM\\IBMVid~1\\iscview.exe icmnlsdb icmadmin password");
            System.out.println("");
        }
        if(argv.length < 9){ // If any 3rd Party info not specified, terminate
            System.out.println("===================================================");
            System.out.println(" TERMINATED:  All 3rd Party info must be specified.");
            System.out.println("===================================================");
            return;        
        } else if(argv.length < 13) {
            System.out.println("*** Some parameters not specified.  Using defaults for rest. ***");
        }
        // Set Command-line options
        if (argv.length > 0)  thirdPartyHostName      = argv[0];
        if (argv.length > 1)  thirdPartyPort          = Integer.valueOf(argv[1]).intValue();
        if (argv.length > 2)  thirdPartyProtocol      = argv[2];
        if (argv.length > 3)  thirdPartyUserName      = argv[3];
        if (argv.length > 4)  thirdPartyPassword      = argv[4];
        if (argv.length > 5)  thirdPartyFullFileName  = argv[5];
        if (argv.length > 6)  thirdPartyFilePath      = argv[6];
        if (argv.length > 7)  thirdPartyMIMEType      = argv[7];
        if (argv.length > 8)  thirdPartyContentLength = Integer.valueOf(argv[8]).intValue();
        if (argv.length > 9)  videoChargerPlayer      = argv[9];
        if (argv.length > 10) database                = argv[10];
        if (argv.length > 11) userName                = argv[11];
        if (argv.length > 12) password                = argv[12];

        String ver = SConnectDisconnectICM.EIP_VERSION;
        
        System.out.println("===========================================");
        System.out.println("IBM Information Integrator for Content v"+ver);
        System.out.println("Sample Program:  SVideoItemPlayICM");
        System.out.println("-------------------------------------------");
        System.out.println(" Content Manager:");
        System.out.println("      Database:  "+database);
        System.out.println("      UserName:  "+userName);
        System.out.println(" 3rd Party:");
        System.out.println("      Host Name:  "+thirdPartyHostName);
        System.out.println("           Port:  "+thirdPartyPort);
        System.out.println("       Protocol:  "+thirdPartyProtocol);
        System.out.println("       UserName:  "+thirdPartyUserName);
        System.out.println("       Password:  "+thirdPartyPassword);
        System.out.println("           File:  "+thirdPartyFullFileName);
        System.out.println("      Mime Type:  "+thirdPartyMIMEType);
        System.out.println(" Content Length:  "+thirdPartyContentLength);
        System.out.println("  Video Charger:  "+videoChargerPlayer);
        System.out.println("OS Architecture:  "+System.getProperty("os.arch"));
        System.out.println("===========================================");
        
        try{
            //-------------------------------------------------------------
            // Connect to datastore
            //-------------------------------------------------------------
            // See Sample SConnectDisconnectICM for more information
            System.out.println("Connecting to datastore (Database '"+database+"', UserName '"+userName+"')...");

                DKDatastoreICM dsICM = new DKDatastoreICM();  // Create new datastore object.
                dsICM.connect(database,userName,password,""); // Connect to the datastore.

            System.out.println("Connected to datastore (Database '"+dsICM.datastoreName()+"', UserName '"+dsICM.userName()+"').");

            //-------------------------------------------------------------
            // Create Video Resource Item
            //-------------------------------------------------------------
            // Create a Video Resource Item as shown in SVideoItemCreateICM.
            // For more information, please refer to that Sample.
            System.out.println("Creating Video Resource Item...");
    
                DKVideoStreamICM video = (DKVideoStreamICM)dsICM.createDDO("S_videoType", DKConstant.DK_CM_ITEM); 

                // Set the User-Defined Attributes
                video.setData(video.dataId(DKConstant.DK_CM_NAMESPACE_ATTR,"S_author"), "IBM"); 
                video.setData(video.dataId(DKConstant.DK_CM_NAMESPACE_ATTR,"S_length"), "48Hours"); 

                // Setting the MIME Type (type of content)
                video.setMimeType(thirdPartyMIMEType);

                // Set 3rd Party Info
                DKThirdPartyServerDef thirdParty = new DKThirdPartyServerDef();
                thirdParty.setHostName(thirdPartyHostName);
                thirdParty.setPort(thirdPartyPort);
                thirdParty.setProtocol(thirdPartyProtocol);
                thirdParty.setUserId(thirdPartyUserName);
                thirdParty.setPassword(thirdPartyPassword);
                thirdParty.setFileName(thirdPartyFullFileName);
                thirdParty.setFilePath(thirdPartyFilePath);
                thirdParty.setInputContentLength(thirdPartyContentLength);

                video.add(thirdParty,0);  // Add from the 3rd Party specified above.

            System.out.println("Created Video Resource Item.");

            //-------------------------------------------------------------
            // Recreating XDO objects given PIDs.
            //-------------------------------------------------------------
            // In order to retrieve an Item either the PID Information must
            // be known or Query / Search must be used.  This sample will 
            // use the PID Strings of the items added above.
            System.out.println("Recreating XDO Object Given PID...");
            
                // First we will need to know what items to retrieve.  The PID information is needed.
                String videoPidString = video.getPidObject().pidString();

                // Knowing Pid strings, let's create new, blank DDOs.
                video = (DKVideoStreamICM) dsICM.createDDO(videoPidString);

            System.out.println("Recreated XDO Object Given PID.");

            //-------------------------------------------------------------
            // Retrieving Video Item, without Retrieving Video Content
            //-------------------------------------------------------------
            // For more information on Retrieval Options, please refer to
            // SItemRetrievalICM.
            System.out.println("Retrieving Video Item, without Retrieving Vidoe Content...");

                video.retrieve(DKConstant.DK_CM_CONTENT_NO);      // Item will be retrieved and resource content will be saved in the specified file.

            System.out.println("Retrieved Video Item, without Retrieving Vidoe Content.");

            //-------------------------------------------------------------
            // Choose Options
            //-------------------------------------------------------------
            System.out.println("Choosing Options...");

                video.setPlayStart("00:00:00:00"); // Start at Beginning
                video.setPlayStop("00:00:00:00");  // Play entire file

            System.out.println("Choose Options.");

            //-------------------------------------------------------------
            // Write ivs File
            //-------------------------------------------------------------
            System.out.println("Writing ivs File...");

                    byte [] ivsfile = video.retrieveStreamInfo();

                    // For our example, write contents in memory to a file
                    FileOutputStream fout = new FileOutputStream(new File("tempVideo.ivs"));
                    fout.write(ivsfile);
                    fout.close();

            System.out.println("Wrote ivs File.");

            //-------------------------------------------------------------
            // Launch Player & Play Online Streaming Video
            //-------------------------------------------------------------
            // This particular Sample will only launch the player if it is
            // on an x86 machine.  Please launch seperately and play file
            // if not on an x86 platform.
            System.out.println("Launching Video Charger Player & Playing Online Streaming Video...");

                if(System.getProperty("os.arch").equalsIgnoreCase("x86"))
                    Runtime.getRuntime().exec(videoChargerPlayer + " tempVideo.ivs");
                else
                    System.out.println("Not an x86 platform.  Please launch the player manually.");

            System.out.println("Launched Video Charger Player & Played Online Streaming Video.");

            //-------------------------------------------------------------
            // Disconnect from datastore & Destroy Reference
            //-------------------------------------------------------------
            // See Sample SConnectDisconnectICM for more information
            System.out.println("Disconnecting from datastore & destroying reference...");            

                dsICM.disconnect();
                dsICM.destroy();

            System.out.println("Disconnected from datastore & destroying reference.");

            //-------------------------------------------------------------
            // Sample program completed without exception
            //-------------------------------------------------------------
            System.out.println("\n==========================================");
            System.out.println("Sample program completed.");
            System.out.println("==========================================\n");
        }
        //------------------------------------------------------------
        // Catch & Print Exceptions        
        //------------------------------------------------------------
        catch (DKException exc){
            SConnectDisconnectICM.printException(exc);  // Print the exception using the function listed below.
            throw(exc);
        } catch (Exception exc) {
            SConnectDisconnectICM.printException(exc);  // Print the exception using the function listed below.
            throw(exc);
        }
    }// end main

    //=================================================================
    // Wrapper Functions
    //=================================================================
    // The following are wrapper functions for functionality covered
    // in this sample.  These functions may be used by other samples.

    // No additional wrapper functions from this sample.

}//end class SVideoItemPlayICM
              
