/*BEGINPROLOGUE************************************************************************
 * @copyright(disclaimer)                                                             *
 * Licensed Materials - Property of IBM                                               *
 * IBM Enterprise Information Portal for Multiplatforms V8.1 (program number 5724-B43)*
 * (c ) Copyright IBM Corp. 1998,  2002. All Rights Reserved.                         *
 *                                                                                    *
 * US Government Users Restricted Rights                                              *
 * Use, duplication or disclosure restricted by GSA ADP Schedule                      *
 * Contract with IBM Corp.                                                            *
 *                                                                                    *
 * DISCLAIMER OF WARRANTIES :                                                         *
 *                                                                                    *
 * Permission is granted to copy and modify this Sample code, and to                  *
 * distribute modified versions provided that both the copyright                      *
 * notice,- and this permission notice and warranty disclaimer appear in all copies   * 
 * and modified versions.                                                             *
 *                                                                                    *
 * THIS SAMPLE CODE IS LICENSED TO YOU AS-IS.                                         *
 * IBM AND ITS SUPPLIERS AND LICENSORS DISCLAIM ALL WARRANTIES, EITHER EXPRESS OR     *
 * IMPLIED, IN SUCH SAMPLE CODE, INCLUDING THE WARRANTY OF NON-INFRINGEMENT AND THE   *
 * IMPLIED WARRANTIES OF MERCHANTABILITY OR FITNESS FOR A PARTICULAR PURPOSE. IN NO   *
 * EVENT WILL IBM OR ITS LICENSORS OR SUPPLIERS BE LIABLE FOR ANY DAMAGES ARISING OUT *
 * OF THE USE OF OR INABILITY TO USE THE SAMPLE CODE, DISTRIBUTION OF THE SAMPLE CODE,*
 * OR COMBINATION OF THE SAMPLE CODE WITH ANY OTHER CODE. IN NO EVENT SHALL IBM OR ITS*
 * LICENSORS AND SUPPLIERS BE LIABLE FOR ANY LOST REVENUE, LOST PROFITS OR DATA, OR   *
 * FOR DIRECT, INDIRECT, SPECIAL, CONSEQUENTIAL,INCIDENTAL OR PUNITIVE DAMAGES,       *
 * HOWEVER CAUSED AND REGARDLESS OF THE THEORY OF LIABILITY,-, EVEN IF IBM OR ITS     *
 * LICENSORS OR SUPPLIERS HAVE BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGES.       *
 *                                                                                    *
 * @endCopyright                                                                      *
 **************************************************************************************/

// based on sample connect/disconnect to OnDemand

/*
 * Filename       : TChangePasswordOD.java
 * Description    : test DKDatastoreOD for change password function
 * Change History :
 *
 * Date        Name  Description
 * ---------- -----  ----------------------------------------------------------
 * 1999/08/13  ATM   Initial creation, modified/added to TConnectOD.java
 *
 */

import com.ibm.mm.sdk.common.*;
import com.ibm.mm.sdk.server.*;
import java.io.*;

// TChangePasswordOD class
public class TChangePasswordOD
{

  // Main method
  public static void main(String argv[])
  {
      //*** for input parameters

      // everest - libsrv, edms1 uid/pw
      String libSrv = new String(""),           // library server
             userid = new String(""),           // userid
             pwd = new String(""),              // old pwd
             new_pwd = new String(""),          // new pwd
             connect_string = new String("");   // connect string

      //--------------------------------------------------------------
      // Checking for input parameters
      //--------------------------------------------------------------
      if (argv.length < 4)  // changed from 3 to 4 since 4th parameter is required
      {                     // for the new string
       System.out.println("Invoke Syntax: " );
       System.out.println("  java TChangePasswordOD " +
        "<libSrv> <userid> <old pw> <new pw> <connect string>" );
       return;
      }
      else
      {
       libSrv = argv[0];
       userid = argv[1];
       pwd = argv[2];
       if (argv.length > 3)
         new_pwd = argv[3];
       if (argv.length > 4)
         connect_string = argv[4];
      }

    try
    {
      DKDatastoreOD dsOD = new DKDatastoreOD();
      String userName = dsOD.userName();
      String dsName = dsOD.datastoreName();
      System.out.println("before connecting to datastore");
      System.out.println("  parameters returned from datastore - user: " +
                         userName + "; dsName: " + dsName);
      System.out.println("connecting to datastore ...");
      dsOD.connect(libSrv,userid,pwd,connect_string);
      System.out.println("  connection ok");
      System.out.println("  verifying parameters ...");
      System.out.println("    arguments received from command    - user: " +
                         userid + "; library server: " + libSrv);
      userName = dsOD.userName();
      dsName = dsOD.datastoreName();
      System.out.println("    parameters returned from datastore - user: " +
                         userName + "; dsName:         " + dsName);
      //-----------------------------------------------------------------------------
      //    Change password
      //-----------------------------------------------------------------------------
      dsOD.changePassword(userid, pwd, new_pwd);
      System.out.println("password has been changed to new password: " + new_pwd);

      dkUserManagement userMgt = (dkUserManagement)dsOD.datastoreDef().datastoreAdmin().userManagement();
      if (userMgt != null)
      {
        userMgt.changePassword(userid,new_pwd,pwd);
        System.out.println("password has been changed back using the dkUserManagement object.");
      }
      else
      {
        System.out.println("dkUserManagement object is null!.");
      }
      //dsOD.changePassword(userid, new_pwd, pwd);
      //System.out.println("password has been restored to old password: " + pwd);
      System.out.println("  -password change has executed successfully!");

      dsOD.disconnect();
      System.out.println("disconnecting from the datastore ...");
      userName = dsOD.userName();
      dsName = dsOD.datastoreName();
      System.out.println("  disconnection ok");
      System.out.println("  parameters returned from datastore - user: " +
                         userName + "; dsName: " + dsName);

      dsOD.destroy(); // Finished with the datstore
    }
    catch (DKException exc)
    {
      System.out.println("Exception name: " + exc.name());
      System.out.println("Exception message: " + exc.getMessage());
      exc.printStackTrace();
    }
    catch (Exception exc)
    {
      System.out.println("Exception message: " + exc.getMessage());
      exc.printStackTrace();
    }

  }
}

