/*BEGINPROLOGUE************************************************************************
 * @copyright(disclaimer)                                                             *
 * Licensed Materials - Property of IBM                                               *
 * IBM Enterprise Information Portal for Multiplatforms V8.1 (program number 5724-B43)*
 * (c ) Copyright IBM Corp. 1998,  2002. All Rights Reserved.                         *
 *                                                                                    *
 * US Government Users Restricted Rights                                              *
 * Use, duplication or disclosure restricted by GSA ADP Schedule                      *
 * Contract with IBM Corp.                                                            *
 *                                                                                    *
 * DISCLAIMER OF WARRANTIES :                                                         *
 *                                                                                    *
 * Permission is granted to copy and modify this Sample code, and to                  *
 * distribute modified versions provided that both the copyright                      *
 * notice,- and this permission notice and warranty disclaimer appear in all copies   * 
 * and modified versions.                                                             *
 *                                                                                    *
 * THIS SAMPLE CODE IS LICENSED TO YOU AS-IS.                                         *
 * IBM AND ITS SUPPLIERS AND LICENSORS DISCLAIM ALL WARRANTIES, EITHER EXPRESS OR     *
 * IMPLIED, IN SUCH SAMPLE CODE, INCLUDING THE WARRANTY OF NON-INFRINGEMENT AND THE   *
 * IMPLIED WARRANTIES OF MERCHANTABILITY OR FITNESS FOR A PARTICULAR PURPOSE. IN NO   *
 * EVENT WILL IBM OR ITS LICENSORS OR SUPPLIERS BE LIABLE FOR ANY DAMAGES ARISING OUT *
 * OF THE USE OF OR INABILITY TO USE THE SAMPLE CODE, DISTRIBUTION OF THE SAMPLE CODE,*
 * OR COMBINATION OF THE SAMPLE CODE WITH ANY OTHER CODE. IN NO EVENT SHALL IBM OR ITS*
 * LICENSORS AND SUPPLIERS BE LIABLE FOR ANY LOST REVENUE, LOST PROFITS OR DATA, OR   *
 * FOR DIRECT, INDIRECT, SPECIAL, CONSEQUENTIAL,INCIDENTAL OR PUNITIVE DAMAGES,       *
 * HOWEVER CAUSED AND REGARDLESS OF THE THEORY OF LIABILITY,-, EVEN IF IBM OR ITS     *
 * LICENSORS OR SUPPLIERS HAVE BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGES.       *
 *                                                                                    *
 * @endCopyright                                                                      *
 **************************************************************************************/

// sample parametric query to VI400 datastore using 
// RMI client

/*
 * Filename       : TClientSamplePQryV4.java
 * Description    : Connect to VI400 Datastore
 *                  Create a query string.
 *                  specify to retrieve folder and document.
 *                  Create a parametric query object and submit query
 *                  get the results in a collection.
 *                  display documents.
 *                  recursively display folder and its document/
 *                  folder content retrieving each document/folder
 *                  as it goes.
 *                  Disconnect from VI400 Datastore
 */

import com.ibm.mm.sdk.common.*;
import com.ibm.mm.sdk.client.*;
import java.io.*;
import java.lang.*;
import java.util.Date;

// TClientSamplePQryV4 class
public class TClientSamplePQryV4 implements DKConstantV4 
{
  // Main method
  public static void main(String argv[])
  {
    //*** for input parameters
      
    String libSrv = new String(""),     // library server
           userid = new String(""),     //userid 
           pw = new String("");         // pw

    //-------------------------------------------------------------- 
    // Checking for input parameters
    //-------------------------------------------------------------- 
    
    if (argv.length < 3)
    {
      System.out.println("Invoke Syntax: " );
      System.out.println("  java TClientConnectV4 " +
                         "<LibServer> <userid> <password>" );
      return;
    }
    else
    {
      libSrv = argv[0];
      userid = argv[1];
      pw = argv[2];
    } 
  
    // Indicate this is a Console Application on Windows NT/95 for error
    // message display in dos console code page. If this was a Windows
    // GUI (Graphic User Interface) Application this would not be necessary
    // because DK_CM_SS_WINDOWS is the default.
    DKEnvironment.setSubSystem(DK_CM_SS_CONSOLE);
    DKDatastoreV4 dsV4 = null;

    try 
    {
      dsV4 = new DKDatastoreV4();
      System.out.println("Starting Time: " + 
                         new Date(System.currentTimeMillis()));
      dkQuery pQry = null;
      DKResults pResults = null;
      DKNVPair parms[] = null;
      System.out.println("connecting to datastore..."); 
      dsV4.connect(libSrv,userid,pw,"");
      System.out.println("datastore connected"); 
       
      String cmd = "SEARCH=(INDEX_CLASS=DATATYPE,";
      cmd += "MAX_RESULTS=15,";
      cmd += "COND=(Data_FString <> NULL));";
      cmd += "OPTION=(CONTENT=YES;";
      cmd += "TYPE_QUERY=STATIC;TYPE_FILTER=FOLDERDOC)";                          
                                
      System.out.println("query string: " + cmd);
      System.out.println("creating query...");
      pQry = dsV4.createQuery(cmd, DK_CM_PARAMETRIC_QL_TYPE, parms);
      System.out.println("executing query...");
      pQry.execute(parms);
      System.out.println("query executed");
      System.out.println("number of query results = " + pQry.numberOfResults());
       
      System.out.println("processing query results...");
      pResults = (DKResults)pQry.result();
      processResults((dkCollection)pResults);
      System.out.println("query results processed");
      dsV4.disconnect();
      System.out.println("datastore disconnected"); 
      System.out.println("Ending Time: " + new Date(System.currentTimeMillis()));
    }
    catch (DKException exc) 
    {
      System.out.println("Exception name: " + exc.name());
      System.out.println("Exception message: " + exc.getMessage());
      exc.printStackTrace();
    }
    catch (Exception exc) 
    {
      System.out.println("Exception message: " + exc.getMessage());
      exc.printStackTrace();
    }
  }
 
  public static void processResults(dkCollection results) 
                throws DKException, Exception
  {
    Short    sVal;
    short    itemType = 0;
    DKDDO    item = null;
    short    i = 0;
    String   dataName;
    DKParts  parts = null;
    DKFolder folder = null;
    System.out.println("  number of items = " + results.cardinality() +
                       ", showing as follows ...");
 
    dkIterator iter = results.createIterator();
    
    while(iter.more()) 
    {
      item = (DKDDO) iter.next();
    
      if (item != null) 
      {
        item.retrieve();
        sVal = (Short)item.getPropertyByName(DK_CM_PROPERTY_ITEM_TYPE);
        itemType = sVal.shortValue();
        
        switch(itemType) 
        {
          case DK_CM_DOCUMENT:
            System.out.println("   Item is a document, number of data item(s): " + 
                               item.dataCount());
                               
            for (i = 1; i <= item.dataCount(); i++) 
            {
              dataName = item.getDataName(i);
              System.out.println("      data-item[" + i + "] = " + dataName + 
                                 ", value = " + item.getData(i));
            
              if (dataName != null && dataName.equals(DKPARTS) == true) // to do
              {
                parts = (DKParts)item.getData(i);
              
                if (parts != null) 
                  System.out.println("        -- this document has " + parts.cardinality() 
                                     + " part(s) ");
              }
            }
            break;
            
          case DK_CM_FOLDER:
            System.out.println("   Item is a folder, number of data item(s) : " +
                               item.dataCount());
                               
            for (i = 1; i <= item.dataCount(); i++) 
            {
              dataName = item.getDataName(i);
              System.out.println("      data-item[" + i + "] = " + dataName + 
                                 ", value = " + item.getData(i));
                                 
              if (dataName != null && dataName.equals(DKPARTS) == true) 
              {
                parts = (DKParts)item.getData(i);
                
                if (parts != null) 
                  System.out.println("        -- this folder has " + parts.cardinality() 
                                     + " part(s) ");
              }
              
              if (dataName != null && dataName.equals(DKFOLDER) == true) 
              {
                folder = (DKFolder)item.getData(i);
                if (folder != null)
                  System.out.println("        -- this folder has " + 
                                     folder.cardinality() + " document(s) ");
              }  
            }
            
            if (folder != null && folder.cardinality() > 0) 
            {
              System.out.println("\n     --------- list of folder contents --------  ");
              processResults(folder);
              System.out.println("     ----------- end of list ------------------ \n");
            }
            else 
              System.out.println("      empty folder ");
                    
            break;
            
          default:
            System.out.println("  Item is not recognized ");
            break;
        } 
      }
    } 
  }
}






